@@ -32,6 +32,9 @@
 #include <stdio.h>
 #include <string.h>
 #include <wtf/SafeStrerror.h>
+#include <wtf/text/MakeString.h>
+
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_BEGIN
 
 namespace JSC {
 
@@ -44,7 +47,8 @@ FunctionAllowlist::FunctionAllowlist(const char* filename)
     if (!f) {
         if (errno == ENOENT) {
             m_hasActiveAllowlist = true;
-            m_entries.add(String::fromLatin1(filename));
+            for (auto f : String::fromLatin1(filename).split(','))
+                m_entries.add(f);
         } else
             dataLogF("Failed to open file %s. Did you add the file-read-data entitlement to WebProcess.sb? Error code: %s\n", filename, safeStrerror(errno).data());
         return;
@@ -60,7 +64,7 @@ FunctionAllowlist::FunctionAllowlist(const char* filename)
 
         // Get rid of newlines at the ends of the strings.
         size_t length = strlen(line);
-        if (line[length - 1] == '\n') {
+        if (length && line[length - 1] == '\n') {
             line[length - 1] = '\0';
             length--;
         }
@@ -69,7 +73,7 @@ FunctionAllowlist::FunctionAllowlist(const char* filename)
         if (!length)
             continue;
         
-        m_entries.add(String({ line, length }));
+        m_entries.add(String(unsafeMakeSpan(line, length)));
     }
 
     int result = fclose(f);
@@ -89,11 +93,11 @@ bool FunctionAllowlist::contains(CodeBlock* codeBlock) const
     if (m_entries.contains(name))
         return true;
 
-    String hash = String::fromUTF8(codeBlock->hashAsStringIfPossible().span());
+    String hash = makeString(codeBlock->hash());
     if (m_entries.contains(hash))
         return true;
 
-    return m_entries.contains(name + '#' + hash);
+    return m_entries.contains(makeString(name, '#', hash));
 }
 
 bool FunctionAllowlist::shouldDumpWasmFunction(uint32_t index) const
@@ -115,5 +119,6 @@ bool FunctionAllowlist::containsWasmFunction(uint32_t index) const
 
 } // namespace JSC
 
-#endif // ENABLE(JIT)
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_END
 
+#endif // ENABLE(JIT)
