@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2018 Apple Inc. All rights reserved.
+ * Copyright (C) 2011-2021 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -27,66 +27,72 @@
  */
 
 #include "config.h"
-#include "MetaAllocator.h"
+#include <wtf/MetaAllocator.h>
 
-#include <wtf/DataLog.h>
-#include <wtf/FastMalloc.h>
-#include <wtf/ProcessID.h>
+#include <wtf/NeverDestroyed.h>
+#include <wtf/TZoneMallocInlines.h>
+#include <wtf/WTFConfig.h>
+
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_BEGIN
 
 namespace WTF {
 
+DEFINE_ALLOCATOR_WITH_HEAP_IDENTIFIER(MetaAllocatorHandle);
+DEFINE_ALLOCATOR_WITH_HEAP_IDENTIFIER(MetaAllocatorFreeSpace);
+
 MetaAllocator::~MetaAllocator()
 {
-    for (FreeSpaceNode* node = m_freeSpaceSizeMap.first(); node;) {
-        FreeSpaceNode* next = node->successor();
-        m_freeSpaceSizeMap.remove(node);
-        freeFreeSpaceNode(node);
-        node = next;
+    // Clear CheckedPtrs in these maps before we start freeing nodes.
+    m_freeSpaceStartAddressMap.clear();
+    m_freeSpaceEndAddressMap.clear();
+
+    for (CheckedPtr node = m_freeSpaceSizeMap.first(); node;) {
+        CheckedPtr next = node->successor();
+        m_freeSpaceSizeMap.remove(node.get());
+        freeFreeSpaceNode(WTFMove(node));
+        node = WTFMove(next);
     }
 #ifndef NDEBUG
     ASSERT(!m_mallocBalance);
 #endif
 }
 
-void MetaAllocatorTracker::notify(MetaAllocatorHandle* handle)
+void MetaAllocatorTracker::notify(MetaAllocatorHandle& handle)
 {
-    m_allocations.insert(handle);
+    m_allocations.insert(&handle);
 }
 
-void MetaAllocatorTracker::release(MetaAllocatorHandle* handle)
+void MetaAllocatorTracker::release(MetaAllocatorHandle& handle)
 {
-    m_allocations.remove(handle);
+    m_allocations.remove(&handle);
 }
 
-ALWAYS_INLINE void MetaAllocator::release(MetaAllocatorHandle* handle)
+void MetaAllocator::release(const Locker<Lock>&, MetaAllocatorHandle& handle)
 {
-    LockHolder locker(&m_lock);
-    if (handle->sizeInBytes()) {
-        void* start = handle->start().untaggedPtr();
-        size_t sizeInBytes = handle->sizeInBytes();
-        decrementPageOccupancy(start, sizeInBytes);
-        addFreeSpaceFromReleasedHandle(FreeSpacePtr(start), sizeInBytes);
+    if (handle.sizeInBytes()) {
+        MemoryPtr start = handle.start();
+        size_t sizeInBytes = handle.sizeInBytes();
+        decrementPageOccupancy(start.untaggedPtr(), sizeInBytes);
+        addFreeSpaceFromReleasedHandle(start.retagged<FreeSpacePtrTag>(), sizeInBytes);
     }
 
-    if (UNLIKELY(!!m_tracker))
+    if (!!m_tracker) [[unlikely]]
         m_tracker->release(handle);
 }
 
-MetaAllocatorHandle::MetaAllocatorHandle(MetaAllocator* allocator, void* start, size_t sizeInBytes, void* ownerUID)
+MetaAllocatorHandle::MetaAllocatorHandle(MetaAllocator& allocator, MetaAllocatorHandle::MemoryPtr start, size_t sizeInBytes)
     : m_allocator(allocator)
     , m_start(start)
-    , m_end(reinterpret_cast<char*>(start) + sizeInBytes)
-    , m_ownerUID(ownerUID)
+    , m_end(start + sizeInBytes)
 {
-    ASSERT(allocator);
     ASSERT(start);
     ASSERT(sizeInBytes);
 }
 
 MetaAllocatorHandle::~MetaAllocatorHandle()
 {
-    ASSERT(m_allocator);
-    m_allocator->release(this);
+    Locker locker { allocator().m_lock };
+    allocator().release(locker, *this);
 }
 
 void MetaAllocatorHandle::shrink(size_t newSizeInBytes)
@@ -94,26 +100,28 @@ void MetaAllocatorHandle::shrink(size_t newSizeInBytes)
     size_t sizeInBytes = this->sizeInBytes();
     ASSERT(newSizeInBytes <= sizeInBytes);
 
-    LockHolder locker(&m_allocator->m_lock);
+    MetaAllocator& allocator = this->allocator();
+    Locker locker { allocator.m_lock };
 
-    newSizeInBytes = m_allocator->roundUp(newSizeInBytes);
+    newSizeInBytes = allocator.roundUp(newSizeInBytes);
     
     ASSERT(newSizeInBytes <= sizeInBytes);
 
     if (newSizeInBytes == sizeInBytes)
         return;
 
-    uintptr_t freeStart = m_start.untaggedPtr<uintptr_t>() + newSizeInBytes;
+    MemoryPtr freeStart = m_start + newSizeInBytes;
     size_t freeSize = sizeInBytes - newSizeInBytes;
-    uintptr_t freeEnd = freeStart + freeSize;
+    uintptr_t freeStartValue = freeStart.untaggedPtr<uintptr_t>();
+    uintptr_t freeEnd = freeStartValue + freeSize;
     
-    uintptr_t firstCompletelyFreePage = (freeStart + m_allocator->m_pageSize - 1) & ~(m_allocator->m_pageSize - 1);
+    uintptr_t firstCompletelyFreePage = roundUpToMultipleOf(allocator.m_pageSize, freeStartValue);
     if (firstCompletelyFreePage < freeEnd)
-        m_allocator->decrementPageOccupancy(reinterpret_cast<void*>(firstCompletelyFreePage), freeSize - (firstCompletelyFreePage - freeStart));
+        allocator.decrementPageOccupancy(reinterpret_cast<void*>(firstCompletelyFreePage), freeSize - (firstCompletelyFreePage - freeStartValue));
 
-    m_allocator->addFreeSpaceFromReleasedHandle(MetaAllocator::FreeSpacePtr(freeStart), freeSize);
+    allocator.addFreeSpaceFromReleasedHandle(freeStart.retagged<FreeSpacePtrTag>(), freeSize);
 
-    m_end = m_start + newSizeInBytes;
+    m_end = freeStart;
 }
 
 void MetaAllocatorHandle::dump(PrintStream& out) const
@@ -121,13 +129,13 @@ void MetaAllocatorHandle::dump(PrintStream& out) const
     out.print(RawPointer(start().untaggedPtr()), "...", RawPointer(end().untaggedPtr()));
 }
 
-MetaAllocator::MetaAllocator(size_t allocationGranule, size_t pageSize)
+MetaAllocator::MetaAllocator(Lock& lock, size_t allocationGranule, size_t pageSize)
     : m_allocationGranule(allocationGranule)
     , m_pageSize(pageSize)
     , m_bytesAllocated(0)
     , m_bytesReserved(0)
     , m_bytesCommitted(0)
-    , m_tracker(0)
+    , m_lock(lock)
 #ifndef NDEBUG
     , m_mallocBalance(0)
 #endif
@@ -151,10 +159,8 @@ MetaAllocator::MetaAllocator(size_t allocationGranule, size_t pageSize)
     ASSERT(static_cast<size_t>(1) << m_logAllocationGranule == m_allocationGranule);
 }
 
-RefPtr<MetaAllocatorHandle> MetaAllocator::allocate(size_t sizeInBytes, void* ownerUID)
+RefPtr<MetaAllocatorHandle> MetaAllocator::allocate(const Locker<Lock>&, size_t sizeInBytes)
 {
-    LockHolder locker(&m_lock);
-
     if (!sizeInBytes)
         return nullptr;
     
@@ -189,17 +195,16 @@ RefPtr<MetaAllocatorHandle> MetaAllocator::allocate(size_t sizeInBytes, void* ow
     m_numAllocations++;
 #endif
 
-    auto handle = adoptRef(*new MetaAllocatorHandle(this, start.untaggedPtr(), sizeInBytes, ownerUID));
+    auto handle = adoptRef(*new MetaAllocatorHandle(*this, start.retagged<HandleMemoryPtrTag>(), sizeInBytes));
 
-    if (UNLIKELY(!!m_tracker))
-        m_tracker->notify(handle.ptr());
+    if (!!m_tracker) [[unlikely]]
+        m_tracker->notify(*handle.ptr());
 
-    return WTFMove(handle);
+    return handle;
 }
 
-MetaAllocator::Statistics MetaAllocator::currentStatistics()
+MetaAllocator::Statistics MetaAllocator::currentStatistics(const Locker<Lock>&)
 {
-    LockHolder locker(&m_lock);
     Statistics result;
     result.bytesAllocated = m_bytesAllocated;
     result.bytesReserved = m_bytesReserved;
@@ -209,15 +214,15 @@ MetaAllocator::Statistics MetaAllocator::currentStatistics()
 
 MetaAllocator::FreeSpacePtr MetaAllocator::findAndRemoveFreeSpace(size_t sizeInBytes)
 {
-    FreeSpaceNode* node = m_freeSpaceSizeMap.findLeastGreaterThanOrEqual(sizeInBytes);
+    CheckedPtr node = m_freeSpaceSizeMap.findLeastGreaterThanOrEqual(sizeInBytes);
     
     if (!node)
-        return 0;
+        return nullptr;
     
     size_t nodeSizeInBytes = node->sizeInBytes();
-    ASSERT(nodeSizeInBytes >= sizeInBytes);
+    RELEASE_ASSERT(nodeSizeInBytes >= sizeInBytes);
 
-    m_freeSpaceSizeMap.remove(node);
+    m_freeSpaceSizeMap.remove(node.get());
 
     FreeSpacePtr result;
 
@@ -227,7 +232,7 @@ MetaAllocator::FreeSpacePtr MetaAllocator::findAndRemoveFreeSpace(size_t sizeInB
         
         m_freeSpaceStartAddressMap.remove(node->m_start);
         m_freeSpaceEndAddressMap.remove(node->m_end);
-        freeFreeSpaceNode(node);
+        freeFreeSpaceNode(WTFMove(node));
     } else {
         // Try to be a good citizen and ensure that the returned chunk of memory
         // straddles as few pages as possible, but only insofar as doing so will
@@ -250,9 +255,10 @@ MetaAllocator::FreeSpacePtr MetaAllocator::findAndRemoveFreeSpace(size_t sizeInB
             m_freeSpaceStartAddressMap.remove(node->m_start);
 
             node->m_start += sizeInBytes;
+            RELEASE_ASSERT(nodeStartAsInt < node->m_start.untaggedPtr<uintptr_t>() && node->m_start.untaggedPtr<uintptr_t>() < node->m_end.untaggedPtr<uintptr_t>());
 
-            m_freeSpaceSizeMap.insert(node);
-            m_freeSpaceStartAddressMap.add(node->m_start, node);
+            m_freeSpaceSizeMap.insert(node.get());
+            m_freeSpaceStartAddressMap.add(node->m_start, node.get());
         } else {
             // Allocate in the right size of the returned chunk, and slide the node to the left;
 
@@ -262,8 +268,8 @@ MetaAllocator::FreeSpacePtr MetaAllocator::findAndRemoveFreeSpace(size_t sizeInB
 
             node->m_end = result;
 
-            m_freeSpaceSizeMap.insert(node);
-            m_freeSpaceEndAddressMap.add(result, node);
+            m_freeSpaceSizeMap.insert(node.get());
+            m_freeSpaceEndAddressMap.add(result, node.get());
         }
     }
     
@@ -285,17 +291,18 @@ void MetaAllocator::addFreeSpaceFromReleasedHandle(FreeSpacePtr start, size_t si
 
 void MetaAllocator::addFreshFreeSpace(void* start, size_t sizeInBytes)
 {
-    LockHolder locker(&m_lock);
+    Config::AssertNotFrozenScope assertNotFrozenScope;
+    Locker locker { m_lock };
     m_bytesReserved += sizeInBytes;
-    addFreeSpace(FreeSpacePtr(start), sizeInBytes);
+    addFreeSpace(FreeSpacePtr::fromUntaggedPtr(start), sizeInBytes);
 }
 
 size_t MetaAllocator::debugFreeSpaceSize()
 {
 #ifndef NDEBUG
-    LockHolder locker(&m_lock);
+    Locker locker { m_lock };
     size_t result = 0;
-    for (FreeSpaceNode* node = m_freeSpaceSizeMap.first(); node; node = node->successor())
+    for (CheckedPtr node = m_freeSpaceSizeMap.first(); node; node = node->successor())
         result += node->sizeInBytes();
     return result;
 #else
@@ -308,8 +315,8 @@ void MetaAllocator::addFreeSpace(FreeSpacePtr start, size_t sizeInBytes)
 {
     FreeSpacePtr end = start + sizeInBytes;
 
-    HashMap<FreeSpacePtr, FreeSpaceNode*>::iterator leftNeighbor = m_freeSpaceEndAddressMap.find(start);
-    HashMap<FreeSpacePtr, FreeSpaceNode*>::iterator rightNeighbor = m_freeSpaceStartAddressMap.find(end);
+    auto leftNeighbor = m_freeSpaceEndAddressMap.find(start);
+    auto rightNeighbor = m_freeSpaceStartAddressMap.find(end);
 
     if (leftNeighbor != m_freeSpaceEndAddressMap.end()) {
         // We have something we can coalesce with on the left. Remove it from the tree, and
@@ -317,13 +324,13 @@ void MetaAllocator::addFreeSpace(FreeSpacePtr start, size_t sizeInBytes)
         
         ASSERT(leftNeighbor->value->m_end == leftNeighbor->key);
 
-        FreeSpaceNode* leftNode = leftNeighbor->value;
+        CheckedPtr leftNode = leftNeighbor->value;
 
         FreeSpacePtr leftEnd = leftNode->m_end;
 
         ASSERT(leftEnd == start);
         
-        m_freeSpaceSizeMap.remove(leftNode);
+        m_freeSpaceSizeMap.remove(leftNode.get());
         m_freeSpaceEndAddressMap.remove(leftEnd);
         
         // Now check if there is also something to coalesce with on the right.
@@ -333,7 +340,7 @@ void MetaAllocator::addFreeSpace(FreeSpacePtr start, size_t sizeInBytes)
             
             ASSERT(rightNeighbor->value->m_start == rightNeighbor->key);
             
-            FreeSpaceNode* rightNode = rightNeighbor->value;
+            CheckedPtr rightNode = rightNeighbor->value;
             FreeSpacePtr rightStart = rightNeighbor->key;
             size_t rightSize = rightNode->sizeInBytes();
             FreeSpacePtr rightEnd = rightNode->m_end;
@@ -341,50 +348,50 @@ void MetaAllocator::addFreeSpace(FreeSpacePtr start, size_t sizeInBytes)
             ASSERT(rightStart == end);
             ASSERT(leftNode->m_start + (leftNode->sizeInBytes() + sizeInBytes + rightSize) == rightEnd);
 
-            m_freeSpaceSizeMap.remove(rightNode);
+            m_freeSpaceSizeMap.remove(rightNode.get());
             m_freeSpaceStartAddressMap.remove(rightStart);
             m_freeSpaceEndAddressMap.remove(rightEnd);
-            
-            freeFreeSpaceNode(rightNode);
 
             leftNode->m_end += (sizeInBytes + rightSize);
 
-            m_freeSpaceSizeMap.insert(leftNode);
-            m_freeSpaceEndAddressMap.add(rightEnd, leftNode);
+            m_freeSpaceSizeMap.insert(leftNode.get());
+            m_freeSpaceEndAddressMap.add(rightEnd, leftNode.get());
+
+            freeFreeSpaceNode(WTFMove(rightNode));
         } else {
             leftNode->m_end += sizeInBytes;
 
-            m_freeSpaceSizeMap.insert(leftNode);
-            m_freeSpaceEndAddressMap.add(end, leftNode);
+            m_freeSpaceSizeMap.insert(leftNode.get());
+            m_freeSpaceEndAddressMap.add(end, leftNode.get());
         }
     } else {
         // Cannot coalesce with left; try to see if we can coalesce with right.
         
         if (rightNeighbor != m_freeSpaceStartAddressMap.end()) {
-            FreeSpaceNode* rightNode = rightNeighbor->value;
+            CheckedPtr rightNode = rightNeighbor->value;
             FreeSpacePtr rightStart = rightNeighbor->key;
 
             ASSERT(rightStart == end);
             ASSERT(start + (sizeInBytes + rightNode->sizeInBytes()) == rightNode->m_end);
 
-            m_freeSpaceSizeMap.remove(rightNode);
+            m_freeSpaceSizeMap.remove(rightNode.get());
             m_freeSpaceStartAddressMap.remove(rightStart);
 
             rightNode->m_start = start;
 
-            m_freeSpaceSizeMap.insert(rightNode);
-            m_freeSpaceStartAddressMap.add(start, rightNode);
+            m_freeSpaceSizeMap.insert(rightNode.get());
+            m_freeSpaceStartAddressMap.add(start, rightNode.get());
         } else {
             // Nothing to coalesce with, so create a new free space node and add it.
             
-            FreeSpaceNode* node = allocFreeSpaceNode();
+            CheckedPtr node = allocFreeSpaceNode();
 
             node->m_start = start;
             node->m_end = start + sizeInBytes;
 
-            m_freeSpaceSizeMap.insert(node);
-            m_freeSpaceStartAddressMap.add(start, node);
-            m_freeSpaceEndAddressMap.add(end, node);
+            m_freeSpaceSizeMap.insert(node.get());
+            m_freeSpaceStartAddressMap.add(start, node.get());
+            m_freeSpaceEndAddressMap.add(end, node.get());
         }
     }
     
@@ -397,32 +404,60 @@ void MetaAllocator::incrementPageOccupancy(void* address, size_t sizeInBytes)
 {
     uintptr_t firstPage = reinterpret_cast<uintptr_t>(address) >> m_logPageSize;
     uintptr_t lastPage = (reinterpret_cast<uintptr_t>(address) + sizeInBytes - 1) >> m_logPageSize;
+
+    uintptr_t currentPageStart = 0;
+    size_t count = 0;
+    auto flushNeedPages = [&] {
+        if (!currentPageStart)
+            return;
+        notifyNeedPage(reinterpret_cast<void*>(currentPageStart << m_logPageSize), count);
+        currentPageStart = 0;
+        count = 0;
+    };
     
     for (uintptr_t page = firstPage; page <= lastPage; ++page) {
-        HashMap<uintptr_t, size_t>::iterator iter = m_pageOccupancyMap.find(page);
-        if (iter == m_pageOccupancyMap.end()) {
-            m_pageOccupancyMap.add(page, 1);
+        auto result = m_pageOccupancyMap.add(page, 1);
+        if (result.isNewEntry) {
             m_bytesCommitted += m_pageSize;
-            notifyNeedPage(reinterpret_cast<void*>(page << m_logPageSize));
-        } else
-            iter->value++;
+            if (!currentPageStart)
+                currentPageStart = page;
+            ++count;
+        } else {
+            result.iterator->value++;
+            flushNeedPages();
+        }
     }
+    flushNeedPages();
 }
 
 void MetaAllocator::decrementPageOccupancy(void* address, size_t sizeInBytes)
 {
     uintptr_t firstPage = reinterpret_cast<uintptr_t>(address) >> m_logPageSize;
     uintptr_t lastPage = (reinterpret_cast<uintptr_t>(address) + sizeInBytes - 1) >> m_logPageSize;
+
+    uintptr_t currentPageStart = 0;
+    size_t count = 0;
+    auto flushFreePages = [&] {
+        if (!currentPageStart)
+            return;
+        notifyPageIsFree(reinterpret_cast<void*>(currentPageStart << m_logPageSize), count);
+        currentPageStart = 0;
+        count = 0;
+    };
     
     for (uintptr_t page = firstPage; page <= lastPage; ++page) {
-        HashMap<uintptr_t, size_t>::iterator iter = m_pageOccupancyMap.find(page);
+        UncheckedKeyHashMap<uintptr_t, size_t>::iterator iter = m_pageOccupancyMap.find(page);
         ASSERT(iter != m_pageOccupancyMap.end());
         if (!--(iter->value)) {
             m_pageOccupancyMap.remove(iter);
             m_bytesCommitted -= m_pageSize;
-            notifyPageIsFree(reinterpret_cast<void*>(page << m_logPageSize));
-        }
+            if (!currentPageStart)
+                currentPageStart = page;
+            ++count;
+        } else
+            flushFreePages();
     }
+    flushFreePages();
 }
 
 bool MetaAllocator::isInAllocatedMemory(const AbstractLocker&, void* address)
@@ -444,15 +479,16 @@ MetaAllocator::FreeSpaceNode* MetaAllocator::allocFreeSpaceNode()
 #ifndef NDEBUG
     m_mallocBalance++;
 #endif
-    return new (NotNull, fastMalloc(sizeof(FreeSpaceNode))) FreeSpaceNode();
+    return new FreeSpaceNode;
 }
 
-void MetaAllocator::freeFreeSpaceNode(FreeSpaceNode* node)
+void MetaAllocator::freeFreeSpaceNode(CheckedPtr<FreeSpaceNode>&& node)
 {
 #ifndef NDEBUG
     m_mallocBalance--;
 #endif
-    fastFree(node);
+    SUPPRESS_UNCHECKED_LOCAL auto* nodePtr = std::exchange(node, nullptr).unsafeGet(); // NOLINT
+    delete nodePtr;
 }
 
 #if ENABLE(META_ALLOCATOR_PROFILE)
@@ -466,4 +502,4 @@ void MetaAllocator::dumpProfile()
 
 } // namespace WTF
 
-
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_END
