@@ -30,6 +30,8 @@
 #include "CanvasObserver.h"
 #include "MediaStreamTrack.h"
 #include "Timer.h"
+#include <wtf/CheckedRef.h>
+#include <wtf/TZoneMalloc.h>
 #include <wtf/TypeCasts.h>
 #include <wtf/WeakPtr.h>
 
@@ -52,7 +54,9 @@ public:
     RefPtr<MediaStreamTrack> clone() final;
 
 private:
-    class Source final : public RealtimeMediaSource, private CanvasObserver, private CanvasDisplayBufferObserver, public ThreadSafeRefCountedAndCanMakeThreadSafeWeakPtr<Source, WTF::DestructionThread::MainRunLoop> {
+    class Source final : public RealtimeMediaSource, private CanvasObserver, private CanvasDisplayBufferObserver, public ThreadSafeRefCountedAndCanMakeThreadSafeWeakPtr<Source, WTF::DestructionThread::MainRunLoop>, public CanMakeThreadSafeCheckedPtr<Source> {
+        WTF_MAKE_TZONE_ALLOCATED(Source);
+        WTF_OVERRIDE_DELETE_FOR_CHECKED_PTR(Source);
     public:
         static Ref<Source> create(HTMLCanvasElement&, std::optional<double>&& frameRequestRate);
         
@@ -62,6 +66,9 @@ private:
 
         WTF_ABSTRACT_THREAD_SAFE_REF_COUNTED_AND_CAN_MAKE_WEAK_PTR_IMPL;
 
+        // CanvasObserver.
+        OVERRIDE_ABSTRACT_CAN_MAKE_CHECKEDPTR(CanMakeThreadSafeCheckedPtr);
+
     private:
         Source(HTMLCanvasElement&, std::optional<double>&&);
 
