@@ -23,6 +23,7 @@
 #if ENABLE(WEB_RTC) && USE(GSTREAMER_WEBRTC)
 
 #include "GStreamerCommon.h"
+#include "GStreamerWebRTCUtils.h"
 #include "JSDOMMapLike.h"
 #include "JSRTCStatsReport.h"
 
@@ -33,6 +34,7 @@
 #include <wtf/MainThread.h>
 #include <wtf/TZoneMallocInlines.h>
 #include <wtf/glib/WTFGType.h>
+#include <wtf/text/StringToIntegerConversion.h>
 
 GST_DEBUG_CATEGORY(webkit_webrtc_stats_debug);
 #define GST_CAT_DEFAULT webkit_webrtc_stats_debug
@@ -41,7 +43,7 @@ namespace WebCore {
 
 RTCStatsReport::Stats::Stats(Type type, const GstStructure* structure)
     : type(type)
-    , id(gstStructureGetString(structure, "id"_s).toString())
+    , id(gstStructureGetString(structure, "id"_s).span())
 {
     if (auto value = gstStructureGet<double>(structure, "timestamp"_s))
         timestamp = Seconds::fromMicroseconds(*value).milliseconds();
@@ -49,9 +51,9 @@ RTCStatsReport::Stats::Stats(Type type, const GstStructure* structure)
 
 RTCStatsReport::RtpStreamStats::RtpStreamStats(Type type, const GstStructure* structure)
     : Stats(type, structure)
-    , kind(gstStructureGetString(structure, "kind"_s).toString())
-    , transportId(gstStructureGetString(structure, "transport-id"_s).toString())
-    , codecId(gstStructureGetString(structure, "codec-id"_s).toString())
+    , kind(gstStructureGetString(structure, "kind"_s).span())
+    , transportId(gstStructureGetString(structure, "transport-id"_s).span())
+    , codecId(gstStructureGetString(structure, "codec-id"_s).span())
 {
     if (auto value = gstStructureGet<unsigned>(structure, "ssrc"_s))
         ssrc = *value;
@@ -66,8 +68,8 @@ RTCStatsReport::SentRtpStreamStats::SentRtpStreamStats(Type type, const GstStruc
 
 RTCStatsReport::CodecStats::CodecStats(const GstStructure* structure)
     : Stats(Type::Codec, structure)
-    , mimeType(gstStructureGetString(structure, "mime-type"_s).toString())
-    , sdpFmtpLine(gstStructureGetString(structure, "sdp-fmtp-line"_s).toString())
+    , mimeType(gstStructureGetString(structure, "mime-type"_s).span())
+    , sdpFmtpLine(gstStructureGetString(structure, "sdp-fmtp-line"_s).span())
 {
     clockRate = gstStructureGet<unsigned>(structure, "clock-rate"_s);
     channels = gstStructureGet<unsigned>(structure, "channels"_s);
@@ -99,7 +101,7 @@ RTCStatsReport::ReceivedRtpStreamStats::ReceivedRtpStreamStats(Type type, const
 
 RTCStatsReport::RemoteInboundRtpStreamStats::RemoteInboundRtpStreamStats(const GstStructure* structure)
     : ReceivedRtpStreamStats(Type::RemoteInboundRtp, structure)
-    , localId(gstStructureGetString(structure, "local-id"_s).toString())
+    , localId(gstStructureGetString(structure, "local-id"_s).span())
 {
     roundTripTime = gstStructureGet<double>(structure, "round-trip-time"_s);
     fractionLost = gstStructureGet<double>(structure, "fraction-lost"_s);
@@ -111,7 +113,7 @@ RTCStatsReport::RemoteInboundRtpStreamStats::RemoteInboundRtpStreamStats(const G
 
 RTCStatsReport::RemoteOutboundRtpStreamStats::RemoteOutboundRtpStreamStats(const GstStructure* structure)
     : SentRtpStreamStats(Type::RemoteOutboundRtp, structure)
-    , localId(gstStructureGetString(structure, "local-id"_s).toString())
+    , localId(gstStructureGetString(structure, "local-id"_s).span())
 {
     remoteTimestamp = gstStructureGet<double>(structure, "remote-timestamp"_s);
 
@@ -134,13 +136,15 @@ RTCStatsReport::InboundRtpStreamStats::InboundRtpStreamStats(const GstStructure*
 
     decoderImplementation = "GStreamer"_s;
 
+    framesPerSecond = gstStructureGet<double>(structure, "frames-per-second"_s);
+    totalDecodeTime = gstStructureGet<double>(structure, "total-decode-time"_s);
     framesDecoded = gstStructureGet<uint64_t>(structure, "frames-decoded"_s);
     framesDropped = gstStructureGet<uint64_t>(structure, "frames-dropped"_s);
     frameWidth = gstStructureGet<unsigned>(structure, "frame-width"_s);
     frameHeight = gstStructureGet<unsigned>(structure, "frame-height"_s);
 
     if (auto identifier = gstStructureGetString(structure, "track-identifier"_s))
-        trackIdentifier = identifier.toString();
+        trackIdentifier = identifier.span();
 
     // FIXME:
     // stats.fractionLost =
@@ -156,7 +160,7 @@ RTCStatsReport::InboundRtpStreamStats::InboundRtpStreamStats(const GstStructure*
 
 RTCStatsReport::OutboundRtpStreamStats::OutboundRtpStreamStats(const GstStructure* structure)
     : SentRtpStreamStats(Type::OutboundRtp, structure)
-    , remoteId(gstStructureGetString(structure, "remote-id"_s).toString())
+    , remoteId(gstStructureGetString(structure, "remote-id"_s).span())
 {
     firCount = gstStructureGet<unsigned>(structure, "fir-count"_s);
     pliCount = gstStructureGet<unsigned>(structure, "pli-count"_s);
@@ -170,9 +174,9 @@ RTCStatsReport::OutboundRtpStreamStats::OutboundRtpStreamStats(const GstStructur
     framesPerSecond = gstStructureGet<double>(structure, "frames-per-second"_s);
 
     if (auto midValue = gstStructureGetString(structure, "mid"_s))
-        mid = midValue.toString();
+        mid = midValue.span();
     if (auto ridValue = gstStructureGetString(structure, "rid"_s))
-        rid = ridValue.toString();
+        rid = ridValue.span();
 }
 
 RTCStatsReport::PeerConnectionStats::PeerConnectionStats(const GstStructure* structure)
@@ -184,11 +188,36 @@ RTCStatsReport::PeerConnectionStats::PeerConnectionStats(const GstStructure* str
 
 RTCStatsReport::TransportStats::TransportStats(const GstStructure* structure)
     : Stats(Type::Transport, structure)
-    , selectedCandidatePairId(gstStructureGetString(structure, "selected-candidate-pair-id"_s).toString())
+    , selectedCandidatePairId(gstStructureGetString(structure, "selected-candidate-pair-id"_s).span())
 {
-    // FIXME: This field is required, GstWebRTC doesn't provide it, so hard-code a value here.
-    dtlsState = RTCDtlsTransportState::Connected;
+    // https://gitlab.freedesktop.org/gstreamer/gstreamer/-/commit/9e38ee7526ecbb12320d1aef29a0c74b815eb4ef
+    if (gst_structure_has_field_typed(structure, "dtls-state", GST_TYPE_WEBRTC_DTLS_TRANSPORT_STATE)) {
+        GstWebRTCDTLSTransportState state;
+        gst_structure_get(structure, "dtls-state", GST_TYPE_WEBRTC_DTLS_TRANSPORT_STATE, &state, nullptr);
+        dtlsState = toRTCDtlsTransportState(state);
+    } else {
+        // Our GStreamer version is likely too old, but this field being required, hard-code it to Connected.
+        dtlsState = RTCDtlsTransportState::Connected;
+    }
 
+    // https://gitlab.freedesktop.org/gstreamer/gstreamer/-/commit/9e38ee7526ecbb12320d1aef29a0c74b815eb4ef
+#if GST_CHECK_VERSION(1, 27, 0)
+    if (gst_structure_has_field_typed(structure, "dtls-role", GST_TYPE_WEBRTC_DTLS_ROLE)) {
+        GstWebRTCDTLSRole role;
+        gst_structure_get(structure, "dtls-role", GST_TYPE_WEBRTC_DTLS_ROLE, &role, nullptr);
+        switch (role) {
+        case GST_WEBRTC_DTLS_ROLE_CLIENT:
+            dtlsRole = DtlsRole::Client;
+            break;
+        case GST_WEBRTC_DTLS_ROLE_SERVER:
+            dtlsRole = DtlsRole::Server;
+            break;
+        case GST_WEBRTC_DTLS_ROLE_UNKNOWN:
+            dtlsRole = DtlsRole::Unknown;
+            break;
+        }
+    }
+#endif
     // FIXME
     // stats.bytesSent =
     // stats.bytesReceived =
@@ -200,38 +229,52 @@ RTCStatsReport::TransportStats::TransportStats(const GstStructure* structure)
     // stats.srtpCipher =
 }
 
-static inline RTCIceCandidateType iceCandidateType(StringView type)
-{
-    if (type == "host"_s)
-        return RTCIceCandidateType::Host;
-    if (type == "srflx"_s)
-        return RTCIceCandidateType::Srflx;
-    if (type == "prflx"_s)
-        return RTCIceCandidateType::Prflx;
-    if (type == "relay"_s)
-        return RTCIceCandidateType::Relay;
-    ASSERT_NOT_REACHED();
-    return RTCIceCandidateType::Host;
-}
-
 RTCStatsReport::IceCandidateStats::IceCandidateStats(GstWebRTCStatsType statsType, const GstStructure* structure)
     : Stats(statsType == GST_WEBRTC_STATS_REMOTE_CANDIDATE ? Type::RemoteCandidate : Type::LocalCandidate, structure)
-    , transportId(gstStructureGetString(structure, "transport-id"_s).toString())
-    , address(gstStructureGetString(structure, "address"_s).toString())
-    , protocol(gstStructureGetString(structure, "protocol"_s).toString())
-    , url(gstStructureGetString(structure, "url"_s).toString())
+    , transportId(gstStructureGetString(structure, "transport-id"_s).span())
+    , protocol(gstStructureGetString(structure, "protocol"_s).span())
+    , url(gstStructureGetString(structure, "url"_s).span())
+    , foundation(gstStructureGetString(structure, "foundation"_s).span())
+    , usernameFragment(gstStructureGetString(structure, "username-fragment"_s).span())
 {
+    // NOTE: We have the address field in the structure but we don't expose it for privacy reasons.
+    // Covered by test: webrtc/candidate-stats.html
     port = gstStructureGet<unsigned>(structure, "port"_s);
     priority = gstStructureGet<unsigned>(structure, "priority"_s);
 
-    if (auto value = gstStructureGetString(structure, "candidate-type"_s))
-        candidateType = iceCandidateType(value);
+#if GST_CHECK_VERSION(1, 27, 0)
+    GstWebRTCICETcpCandidateType gstTcpType;
+    if (gst_structure_get(structure, "tcp-type", &gstTcpType, nullptr)) {
+        switch (gstTcpType) {
+        case GST_WEBRTC_ICE_TCP_CANDIDATE_TYPE_ACTIVE:
+            tcpType = RTCIceTcpCandidateType::Active;
+            break;
+        case GST_WEBRTC_ICE_TCP_CANDIDATE_TYPE_PASSIVE:
+            tcpType = RTCIceTcpCandidateType::Passive;
+            break;
+        case GST_WEBRTC_ICE_TCP_CANDIDATE_TYPE_SO:
+            tcpType = RTCIceTcpCandidateType::So;
+            break;
+        case GST_WEBRTC_ICE_TCP_CANDIDATE_TYPE_NONE:
+            break;
+        };
+    }
+#endif
+
+    candidateType = RTCIceCandidateType::Host;
+
+    auto value = gstStructureGetString(structure, "candidate-type"_s);
+    if (!value) [[unlikely]]
+        return;
+
+    if (auto iceCandidateType = toRTCIceCandidateType(StringView::fromLatin1(value.utf8())))
+        candidateType = *iceCandidateType;
 }
 
 RTCStatsReport::IceCandidatePairStats::IceCandidatePairStats(const GstStructure* structure)
     : Stats(Type::CandidatePair, structure)
-    , localCandidateId(gstStructureGetString(structure, "local-candidate-id"_s).toString())
-    , remoteCandidateId(gstStructureGetString(structure, "remote-candidate-id"_s).toString())
+    , localCandidateId(gstStructureGetString(structure, "local-candidate-id"_s).span())
+    , remoteCandidateId(gstStructureGetString(structure, "remote-candidate-id"_s).span())
 {
     // FIXME
     // stats.transportId =
@@ -258,6 +301,44 @@ RTCStatsReport::IceCandidatePairStats::IceCandidatePairStats(const GstStructure*
     // stats.consentResponsesSent =
 }
 
+RTCStatsReport::CertificateStats::CertificateStats(const GstStructure* structure)
+    : Stats(Type::Certificate, structure)
+    , fingerprint(gstStructureGetString(structure, "fingerprint"_s).span())
+    , fingerprintAlgorithm(gstStructureGetString(structure, "fingerprint-algorithm"_s).span())
+    , base64Certificate(gstStructureGetString(structure, "base64-certificate"_s).span())
+
+{
+    // FIXME: Fill issuerCertificateId if present.
+}
+
+RTCStatsReport::MediaSourceStats::MediaSourceStats(Type type, const GstStructure* structure)
+    : Stats(type, structure)
+    , trackIdentifier(gstStructureGetString(structure, "track-identifier"_s).span())
+    , kind(gstStructureGetString(structure, "kind"_s).span())
+{
+}
+
+RTCStatsReport::AudioSourceStats::AudioSourceStats(const GstStructure* structure)
+    : MediaSourceStats(Type::MediaSource, structure)
+    , audioLevel(gstStructureGet<double>(structure, "audio-level"_s))
+    , totalAudioEnergy(gstStructureGet<double>(structure, "total-audio-energy"_s))
+    , totalSamplesDuration(gstStructureGet<double>(structure, "total-samples-duration"_s))
+{
+    /* FIXME:
+        std::optional<double> echoReturnLoss;
+        std::optional<double> echoReturnLossEnhancement;
+     */
+}
+
+RTCStatsReport::VideoSourceStats::VideoSourceStats(const GstStructure* structure)
+    : MediaSourceStats(Type::MediaSource, structure)
+    , width(gstStructureGet<unsigned>(structure, "width"_s))
+    , height(gstStructureGet<unsigned>(structure, "height"_s))
+    , frames(gstStructureGet<unsigned>(structure, "frames"_s))
+    , framesPerSecond(gstStructureGet<double>(structure, "frames-per-second"_s))
+{
+}
+
 struct ReportHolder : public ThreadSafeRefCounted<ReportHolder> {
     WTF_MAKE_TZONE_ALLOCATED_INLINE(ReportHolder);
     WTF_MAKE_NONCOPYABLE(ReportHolder);
@@ -274,15 +355,29 @@ static gboolean fillReportCallback(const GValue* value, Ref<ReportHolder>& repor
         return TRUE;
 
     const GstStructure* structure = gst_value_get_structure(value);
-    GstWebRTCStatsType statsType;
-    if (!gst_structure_get(structure, "type", GST_TYPE_WEBRTC_STATS_TYPE, &statsType, nullptr))
-        return TRUE;
 
-    if (UNLIKELY(!reportHolder->adapter))
+    if (!reportHolder->adapter) [[unlikely]]
         return TRUE;
 
     auto& report = *reportHolder->adapter;
 
+    if (auto webkitStatsType = gstStructureGetString(structure, "webkit-stats-type"_s)) {
+        if (webkitStatsType == "audio-source-stats"_s) {
+            RTCStatsReport::AudioSourceStats stats(structure);
+            report.set<IDLDOMString, IDLDictionary<RTCStatsReport::AudioSourceStats>>(stats.id, WTFMove(stats));
+            return TRUE;
+        }
+        if (webkitStatsType == "video-source-stats"_s) {
+            RTCStatsReport::VideoSourceStats stats(structure);
+            report.set<IDLDOMString, IDLDictionary<RTCStatsReport::VideoSourceStats>>(stats.id, WTFMove(stats));
+            return TRUE;
+        }
+    }
+
+    GstWebRTCStatsType statsType;
+    if (!gst_structure_get(structure, "type", GST_TYPE_WEBRTC_STATS_TYPE, &statsType, nullptr))
+        return TRUE;
+
     switch (statsType) {
     case GST_WEBRTC_STATS_CODEC: {
         RTCStatsReport::CodecStats stats(structure);
@@ -330,26 +425,30 @@ static gboolean fillReportCallback(const GValue* value, Ref<ReportHolder>& repor
         break;
     case GST_WEBRTC_STATS_LOCAL_CANDIDATE:
     case GST_WEBRTC_STATS_REMOTE_CANDIDATE:
-        if (webkitGstCheckVersion(1, 22, 0)) {
+        if (gst_check_version(1, 22, 0)) {
             RTCStatsReport::IceCandidateStats stats(statsType, structure);
             report.set<IDLDOMString, IDLDictionary<RTCStatsReport::IceCandidateStats>>(stats.id, WTFMove(stats));
         }
         break;
     case GST_WEBRTC_STATS_CANDIDATE_PAIR:
-        if (webkitGstCheckVersion(1, 22, 0)) {
+        if (gst_check_version(1, 22, 0)) {
             RTCStatsReport::IceCandidatePairStats stats(structure);
             report.set<IDLDOMString, IDLDictionary<RTCStatsReport::IceCandidatePairStats>>(stats.id, WTFMove(stats));
         }
         break;
-    case GST_WEBRTC_STATS_CERTIFICATE:
-        // FIXME: Missing certificate stats support
+    case GST_WEBRTC_STATS_CERTIFICATE: {
+        // https://gitlab.freedesktop.org/gstreamer/gstreamer/-/merge_requests/10313
+        RTCStatsReport::CertificateStats stats(structure);
+        report.set<IDLDOMString, IDLDictionary<RTCStatsReport::CertificateStats>>(stats.id, WTFMove(stats));
         break;
     }
+    }
 
     return TRUE;
 }
 
 struct CallbackHolder {
+    RefPtr<GStreamerStatsCollector> collector;
     GStreamerStatsCollector::CollectorCallback callback;
     GStreamerStatsCollector::PreprocessCallback preprocessCallback;
     GRefPtr<GstPad> pad;
@@ -359,9 +458,16 @@ WEBKIT_DEFINE_ASYNC_DATA_STRUCT(CallbackHolder)
 
 void GStreamerStatsCollector::getStats(CollectorCallback&& callback, const GRefPtr<GstPad>& pad, PreprocessCallback&& preprocessCallback)
 {
+    static auto s_maximumReportAge = 300_ms;
     static std::once_flag debugRegisteredFlag;
     std::call_once(debugRegisteredFlag, [] {
         GST_DEBUG_CATEGORY_INIT(webkit_webrtc_stats_debug, "webkitwebrtcstats", 0, "WebKit WebRTC Stats");
+        auto expirationTime = StringView::fromLatin1(std::getenv("WEBKIT_GST_WEBRTC_STATS_CACHE_EXPIRATION_TIME_MS"));
+        if (expirationTime.isEmpty())
+            return;
+
+        if (auto milliseconds = WTF::parseInteger<int>(expirationTime))
+            s_maximumReportAge = Seconds::fromMilliseconds(*milliseconds);
     });
 
     if (!m_webrtcBin) {
@@ -369,7 +475,23 @@ void GStreamerStatsCollector::getStats(CollectorCallback&& callback, const GRefP
         return;
     }
 
+    auto now = MonotonicTime::now();
+    if (!pad) {
+        if (m_cachedGlobalReport && (now - m_cachedGlobalReport->generationTime < s_maximumReportAge)) {
+            GST_TRACE_OBJECT(m_webrtcBin.get(), "Returning cached global stats report");
+            callback(m_cachedGlobalReport->report.get());
+            return;
+        }
+    } else if (auto report = m_cachedReportsPerPad.getOptional(pad)) {
+        if (now - report->generationTime < s_maximumReportAge) {
+            GST_TRACE_OBJECT(m_webrtcBin.get(), "Returning cached stats report for pad %" GST_PTR_FORMAT, pad.get());
+            callback(report->report.get());
+            return;
+        }
+    }
+
     auto* holder = createCallbackHolder();
+    holder->collector = this;
     holder->callback = WTFMove(callback);
     holder->preprocessCallback = WTFMove(preprocessCallback);
     holder->pad = pad;
@@ -395,20 +517,35 @@ void GStreamerStatsCollector::getStats(CollectorCallback&& callback, const GRefP
             return;
         }
 
-        callOnMainThreadAndWait([holder, stats] {
+        callOnMainThreadAndWait([holder, stats] mutable {
             auto preprocessedStats = holder->preprocessCallback(holder->pad, stats);
             if (!preprocessedStats)
                 return;
-            holder->callback(RTCStatsReport::create([stats = WTFMove(preprocessedStats)](auto& mapAdapter) mutable {
+            auto report = RTCStatsReport::create([stats = WTFMove(preprocessedStats)](auto& mapAdapter) mutable {
                 auto holder = adoptRef(*new ReportHolder(&mapAdapter));
                 gstStructureForeach(stats.get(), [&](auto, const auto value) -> bool {
                     return fillReportCallback(value, holder);
                 });
-            }));
+            });
+            CachedReport cachedReport;
+            cachedReport.generationTime = MonotonicTime::now();
+            cachedReport.report = report.ptr();
+            if (holder->pad)
+                holder->collector->m_cachedReportsPerPad.set(holder->pad, WTFMove(cachedReport));
+            else
+                holder->collector->m_cachedGlobalReport = WTFMove(cachedReport);
+            holder->callback(WTFMove(report));
         });
     }, holder, reinterpret_cast<GDestroyNotify>(destroyCallbackHolder)));
 }
 
+void GStreamerStatsCollector::invalidateCache()
+{
+    ASSERT(isMainThread());
+    m_cachedGlobalReport = std::nullopt;
+    m_cachedReportsPerPad.clear();
+}
+
 #undef GST_CAT_DEFAULT
 
 } // namespace WebCore
