@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012, Google Inc. All rights reserved.
+ * Copyright (C) 2012 Google Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -29,27 +29,37 @@
 
 #include "AudioContext.h"
 #include "AudioNodeInput.h"
+#include "ContextDestructionObserverInlines.h"
 #include "Document.h"
 #include "MediaStream.h"
 #include "MediaStreamAudioSource.h"
-#include <wtf/IsoMallocInlines.h>
 #include <wtf/Locker.h>
+#include <wtf/TZoneMallocInlines.h>
 
 namespace WebCore {
 
-WTF_MAKE_ISO_ALLOCATED_IMPL(MediaStreamAudioDestinationNode);
+WTF_MAKE_TZONE_OR_ISO_ALLOCATED_IMPL(MediaStreamAudioDestinationNode);
 
-Ref<MediaStreamAudioDestinationNode> MediaStreamAudioDestinationNode::create(BaseAudioContext& context, size_t numberOfChannels)
+ExceptionOr<Ref<MediaStreamAudioDestinationNode>> MediaStreamAudioDestinationNode::create(BaseAudioContext& context, const AudioNodeOptions& options)
 {
-    return adoptRef(*new MediaStreamAudioDestinationNode(context, numberOfChannels));
+    // This behavior is not part of the specification. This is done for consistency with Blink.
+    if (context.isStopped() || !context.scriptExecutionContext())
+        return Exception { ExceptionCode::NotAllowedError, "Cannot create a MediaStreamAudioDestinationNode in a detached frame"_s };
+
+    auto node = adoptRef(*new MediaStreamAudioDestinationNode(context));
+
+    auto result = node->handleAudioNodeOptions(options, { 2, ChannelCountMode::Explicit, ChannelInterpretation::Speakers });
+    if (result.hasException())
+        return result.releaseException();
+
+    return node;
 }
 
-MediaStreamAudioDestinationNode::MediaStreamAudioDestinationNode(BaseAudioContext& context, size_t numberOfChannels)
-    : AudioBasicInspectorNode(context, context.sampleRate(), numberOfChannels)
+MediaStreamAudioDestinationNode::MediaStreamAudioDestinationNode(BaseAudioContext& context)
+    : AudioBasicInspectorNode(context, NodeTypeMediaStreamAudioDestination)
     , m_source(MediaStreamAudioSource::create(context.sampleRate()))
     , m_stream(MediaStream::create(*context.document(), MediaStreamPrivate::create(context.document()->logger(), m_source.copyRef())))
 {
-    setNodeType(NodeTypeMediaStreamAudioDestination);
     initialize();
 }
 
@@ -60,11 +70,14 @@ MediaStreamAudioDestinationNode::~MediaStreamAudioDestinationNode()
 
 void MediaStreamAudioDestinationNode::process(size_t numberOfFrames)
 {
-    m_source->consumeAudio(*input(0)->bus(), numberOfFrames);
+    m_source->consumeAudio(input(0)->bus(), numberOfFrames);
 }
 
-void MediaStreamAudioDestinationNode::reset()
+void MediaStreamAudioDestinationNode::checkNumberOfChannelsForInput(AudioNodeInput* input)
 {
+    ASSERT(context().isAudioThread() && context().isGraphOwner());
+    m_source->setNumberOfChannels(input->numberOfChannels());
+    AudioBasicInspectorNode::checkNumberOfChannelsForInput(input);
 }
 
 } // namespace WebCore
