@@ -29,6 +29,8 @@
 #include "AXLogger.h"
 #include "AXObjectCache.h"
 #include "AXTreeStore.h"
+#include "AXTreeStoreInlines.h"
+#include "AXUtilities.h"
 #include "BoundaryPointInlines.h"
 #include "HTMLInputElement.h"
 #include "Logging.h"
@@ -213,6 +215,8 @@ RefPtr<AXCoreObject> AXTextMarker::object() const
     if (isNull())
         return nullptr;
 
+    // FIXME: The isNull() check should allow us to deref *treeID() and *objectID() safely
+    // in this function for a bit more performance.
 #if ENABLE(ACCESSIBILITY_ISOLATED_TREE)
     if (!isMainThread()) {
         auto tree = std::get<RefPtr<AXIsolatedTree>>(axTreeForID(treeID()));
@@ -237,7 +241,7 @@ String AXTextMarker::debugDescription() const
 
     return makeString("{"_s
         , id
-        , object ? makeString(separator, "role "_s, accessibilityRoleToString(object->role())) : ""_s
+        , object ? makeString(separator, "role "_s, roleToString(object->role())) : ""_s
         , isIgnored() ? makeString(separator, "ignored"_s) : ""_s
         // Anchor type and other fields below are not used for text markers processed off the main-thread.
         , isMainThread() ? makeString(separator, "anchor "_s, m_data.anchorType) : ""_s
@@ -546,7 +550,7 @@ String listMarkerTextOnSameLine(const AXTextMarker& marker)
 }
 #endif // ENABLE(AX_THREAD_TEXT_APIS)
 
-String AXTextMarkerRange::toString(IncludeListMarkerText includeListMarkerText) const
+String AXTextMarkerRange::toString(IncludeListMarkerText includeListMarkerText, IncludeImageAltText includeImageAltText) const
 {
 #if !ENABLE(AX_THREAD_TEXT_APIS)
     UNUSED_PARAM(includeListMarkerText);
@@ -573,7 +577,12 @@ String AXTextMarkerRange::toString(IncludeListMarkerText includeListMarkerText)
             return result.toString();
         }
 
-        auto emitNewlineOnExit = [&] (AXIsolatedObject& object) {
+        auto emitAuxiliaryText = [&] (AXIsolatedObject& object) {
+            if (includeImageAltText == IncludeImageAltText::Yes && object.isImage()) {
+                // This is an image, so add alt text (if it has any).
+                result.append(object.description());
+            }
+
             // FIXME: This function should not just be emitting newlines, but instead handling every character type in TextEmissionBehavior.
             auto behavior = object.textEmissionBehavior();
             if (behavior != TextEmissionBehavior::Newline && behavior != TextEmissionBehavior::DoubleNewline)
@@ -591,22 +600,27 @@ String AXTextMarkerRange::toString(IncludeListMarkerText includeListMarkerText)
 
         // FIXME: If we've been given reversed markers, i.e. the end marker actually comes before the start marker,
         // we may want to detect this and try searching AXDirection::Previous?
-        RefPtr current = findObjectWithRuns(*start.isolatedObject(), AXDirection::Next, std::nullopt, emitNewlineOnExit);
+        RefPtr current = findObjectWithRuns(*start.isolatedObject(), AXDirection::Next, std::nullopt, emitAuxiliaryText);
         while (current && current->objectID() != end.objectID()) {
             result.append(current->textRuns()->toStringView());
-            current = findObjectWithRuns(*current, AXDirection::Next, std::nullopt, emitNewlineOnExit);
+            current = findObjectWithRuns(*current, AXDirection::Next, std::nullopt, emitAuxiliaryText);
         }
         result.append(end.runs()->substring(0, end.offset()));
         return result.toString();
     }
 #endif // ENABLE(AX_THREAD_TEXT_APIS)
 
-    return Accessibility::retrieveValueFromMainThread<String>([this] () -> String {
+    return Accessibility::retrieveValueFromMainThread<String>([this, includeImageAltText] () -> String {
+
         auto range = simpleRange();
         if (!range)
             return { };
 
-        TextIterator it = TextIterator(*range, { TextIteratorBehavior::IgnoresFullSizeKana });
+        OptionSet<TextIteratorBehavior> behaviors = { TextIteratorBehavior::IgnoresFullSizeKana };
+        if (includeImageAltText == IncludeImageAltText::Yes)
+            behaviors.add(TextIteratorBehavior::EmitsImageAltText);
+
+        TextIterator it = TextIterator(*range, behaviors);
         if (it.atEnd())
             return { };
 
@@ -934,14 +948,16 @@ AXTextMarker AXTextMarker::findLastBefore(std::optional<AXID> stopAtID) const
         return textRunMarker.findLastBefore(stopAtID);
     }
 
-    AXTextMarker marker;
-    auto newMarker = *this;
+    RefPtr lastObjectWithRuns = isolatedObject();
     // FIXME: Do we need to compare both tree ID and object ID here?
-    while (newMarker.isValid() && (!stopAtID || *stopAtID != newMarker.objectID())) {
-        marker = WTFMove(newMarker);
-        newMarker = marker.findMarker(AXDirection::Next, CoalesceObjectBreaks::No, IgnoreBRs::No, stopAtID);
+    while (!stopAtID || *stopAtID != lastObjectWithRuns->objectID()) {
+        RefPtr newObject = findObjectWithRuns(*lastObjectWithRuns, AXDirection::Next, stopAtID);
+        if (!newObject)
+            break;
+        lastObjectWithRuns = WTFMove(newObject);
     }
-    return marker;
+
+    return AXTextMarker { *lastObjectWithRuns, lastObjectWithRuns->textRuns()->totalLength() };
 }
 
 AXTextMarkerRange AXTextMarker::rangeWithSameStyle() const
@@ -1163,7 +1179,7 @@ AXTextMarker AXTextMarker::findLine(AXDirection direction, AXTextUnitBoundary bo
                 ASSERT(adjacentRunIndex - runIndex == 2);
                 // This scenario really should only happen with single "entity" runs (where an entity could be an ASCII
                 // character, or a multi-byte emoji that occupies multiple indices but is one atomic entity).
-                ASSERT(!currentRuns->containsOnlyASCII || (currentRuns->runLength(runIndex) == 1 && currentRuns->runLength(adjacentRunIndex) == 1));
+                AX_BROKEN_ASSERT(!currentRuns->containsOnlyASCII || (currentRuns->runLength(runIndex) == 1 && currentRuns->runLength(adjacentRunIndex) == 1));
                 // The next line end is simply the adjacent marker with an upstream affinity (with an ASSERT to verify this).
                 ASSERT(currentRuns->indexForOffset(adjacentMarker.offset(), Affinity::Upstream) == runIndex + 1);
                 adjacentMarker.setAffinity(Affinity::Upstream);
@@ -1486,26 +1502,28 @@ AXTextMarkerRange AXTextMarker::lineRange(LineRangeType type, IncludeTrailingLin
         auto startMarker = atLineStart() ? *this : previousLineStart();
         auto endMarker = atLineEnd() ? *this : nextLineEnd(includeTrailingLineBreak);
         return AXTextMarkerRange(startMarker, endMarker);
-    } else if (type == LineRangeType::Left) {
+    }
+
+    if (type == LineRangeType::Left) {
         // Move backwards off a line start (because this is a "left-line" request).
         auto startMarker = atLineStart() ? findMarker(AXDirection::Previous) : *this;
         if (!startMarker.atLineStart())
             startMarker = startMarker.previousLineStart();
 
-        auto endMarker = startMarker.nextLineEnd(includeTrailingLineBreak);
-        return { WTFMove(startMarker), WTFMove(endMarker) };
-    } else {
-        ASSERT(type == LineRangeType::Right);
-
-        // Move forwards off a line end (because this a "right-line" request).
-        auto startMarker = atLineEnd() ? findMarker(AXDirection::Next) : *this;
-        if (!startMarker.atLineStart())
-            startMarker = startMarker.previousLineStart();
-
         auto endMarker = startMarker.nextLineEnd(includeTrailingLineBreak);
         return { WTFMove(startMarker), WTFMove(endMarker) };
     }
 
+    ASSERT(type == LineRangeType::Right);
+
+    // Move forwards off a line end (because this a "right-line" request).
+    auto startMarker = atLineEnd() ? findMarker(AXDirection::Next) : *this;
+    if (!startMarker.atLineStart())
+        startMarker = startMarker.previousLineStart();
+
+    auto endMarker = startMarker.nextLineEnd(includeTrailingLineBreak);
+    return { WTFMove(startMarker), WTFMove(endMarker) };
+
     return { };
 }
 
@@ -1643,7 +1661,7 @@ AXIsolatedObject* findObjectWithRuns(AXIsolatedObject& start, AXDirection direct
             exitObject(*current);
             current = nextInPreOrder(*current);
         }
-        return current.get();
+        return current.unsafeGet();
     }
     ASSERT(direction == AXDirection::Previous);
 
@@ -1669,7 +1687,7 @@ AXIsolatedObject* findObjectWithRuns(AXIsolatedObject& start, AXDirection direct
         exitObject(*current);
         current = previousInPreOrder(*current);
     }
-    return current.get();
+    return current.unsafeGet();
 }
 
 } // namespace Accessibility
