@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2016 Apple Inc. All rights reserved.
+ * Copyright (C) 2024 Samuel Weinig <sam@webkit.org>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -30,15 +31,15 @@
 #include "CSSFontSelector.h"
 #include "CSSParser.h"
 #include "CSSPrimitiveValue.h"
-#include "CSSPropertyParserHelpers.h"
-#include "CSSPropertyParserWorkerSafe.h"
+#include "CSSPropertyParserConsumer+Font.h"
 #include "CSSSegmentedFontFace.h"
 #include "CSSValueList.h"
 #include "CSSValuePool.h"
 #include "FontCache.h"
 #include "FontSelectionValueInlines.h"
-#include "StyleBuilderConverter.h"
+#include "StylePrimitiveNumericTypes+Conversions.h"
 #include "StyleProperties.h"
+#include <ranges>
 
 namespace WebCore {
 
@@ -106,7 +107,7 @@ bool CSSFontFaceSet::hasFace(const CSSFontFace& face) const
 void CSSFontFaceSet::updateStyleIfNeeded()
 {
     if (m_owningFontSelector)
-        m_owningFontSelector->updateStyleIfNeeded();
+        Ref { *m_owningFontSelector }->updateStyleIfNeeded();
 }
 
 void CSSFontFaceSet::ensureLocalFontFacesForFamilyRegistered(const AtomString& familyName)
@@ -115,22 +116,22 @@ void CSSFontFaceSet::ensureLocalFontFacesForFamilyRegistered(const AtomString& f
     if (m_locallyInstalledFacesLookupTable.contains(familyName))
         return;
 
-    if (!m_owningFontSelector->scriptExecutionContext())
+    Ref owningFontSelector = *m_owningFontSelector;
+    if (!owningFontSelector->scriptExecutionContext())
         return;
-    auto allowUserInstalledFonts = m_owningFontSelector->scriptExecutionContext()->settingsValues().shouldAllowUserInstalledFonts ? AllowUserInstalledFonts::Yes : AllowUserInstalledFonts::No;
-    auto capabilities = FontCache::forCurrentThread().getFontSelectionCapabilitiesInFamily(familyName, allowUserInstalledFonts);
+    auto allowUserInstalledFonts = owningFontSelector->protectedScriptExecutionContext()->settingsValues().shouldAllowUserInstalledFonts ? AllowUserInstalledFonts::Yes : AllowUserInstalledFonts::No;
+    auto capabilities = FontCache::forCurrentThread()->getFontSelectionCapabilitiesInFamily(familyName, allowUserInstalledFonts);
     if (capabilities.isEmpty())
         return;
 
     Vector<Ref<CSSFontFace>> faces;
     for (auto item : capabilities) {
-        auto face = CSSFontFace::create(*m_owningFontSelector, nullptr, nullptr, true);
-        
-        // FIXME: Don't use a list here. https://bugs.webkit.org/show_bug.cgi?id=196381
-        auto& pool = m_owningFontSelector->scriptExecutionContext()->cssValuePool();
-        face->setFamilies(CSSValueList::createCommaSeparated(pool.createFontFamilyValue(familyName)).get());
+        auto face = CSSFontFace::create(owningFontSelector, nullptr, nullptr, true);
+
+        auto& pool = owningFontSelector->protectedScriptExecutionContext()->cssValuePool();
+        face->setFamily(pool.createFontFamilyValue(familyName));
         face->setFontSelectionCapabilities(item);
-        face->adoptSource(makeUnique<CSSFontFaceSource>(face.get(), familyName));
+        face->adoptSource(makeUniqueWithoutRefCountedCheck<CSSFontFaceSource>(face.get(), familyName));
         ASSERT(!face->computeFailureState());
         faces.append(WTFMove(face));
     }
@@ -159,6 +160,8 @@ String CSSFontFaceSet::familyNameFromPrimitive(const CSSPrimitiveValue& value)
         return pictographFamily.get();
     case CSSValueSystemUi:
         return systemUiFamily.get();
+    case CSSValueMath:
+        return mathFamily.get();
     default:
         return { };
     }
@@ -166,30 +169,28 @@ String CSSFontFaceSet::familyNameFromPrimitive(const CSSPrimitiveValue& value)
 
 void CSSFontFaceSet::addToFacesLookupTable(CSSFontFace& face)
 {
-    auto families = face.families();
-    if (!families) {
+    auto family = face.familyCSSValue();
+    if (!family) {
         // If the font has failed, there's no point in actually adding it to m_facesLookupTable,
         // because no font requests can actually use it for anything. So, let's just ... not add it.
         return;
     }
 
-    for (auto& item : *families) {
-        auto familyName = AtomString { CSSFontFaceSet::familyNameFromPrimitive(downcast<CSSPrimitiveValue>(item)) };
-        if (familyName.isEmpty())
-            continue;
+    auto familyName = AtomString { CSSFontFaceSet::familyNameFromPrimitive(downcast<CSSPrimitiveValue>(*family)) };
+    if (familyName.isNull())
+        return;
 
-        auto addResult = m_facesLookupTable.add(familyName, Vector<Ref<CSSFontFace>>());
-        auto& familyFontFaces = addResult.iterator->value;
-        if (addResult.isNewEntry) {
-            // m_locallyInstalledFontFaces grows without bound, eventually encorporating every font installed on the system.
-            // This is by design.
-            if (m_owningFontSelector)
-                ensureLocalFontFacesForFamilyRegistered(familyName);
-            familyFontFaces = { };
-        }
-
-        familyFontFaces.append(face);
+    auto addResult = m_facesLookupTable.add(familyName, Vector<Ref<CSSFontFace>>());
+    auto& familyFontFaces = addResult.iterator->value;
+    if (addResult.isNewEntry) {
+        // m_locallyInstalledFontFaces grows without bound, eventually incorporating every font installed on the system.
+        // This is by design.
+        if (m_owningFontSelector)
+            ensureLocalFontFacesForFamilyRegistered(familyName);
+        familyFontFaces = { };
     }
+
+    familyFontFaces.append(face);
 }
 
 void CSSFontFaceSet::add(CSSFontFace& face)
@@ -219,32 +220,30 @@ void CSSFontFaceSet::add(CSSFontFace& face)
     }
 }
 
-void CSSFontFaceSet::removeFromFacesLookupTable(const CSSFontFace& face, const CSSValueList& familiesToSearchFor)
+void CSSFontFaceSet::removeFromFacesLookupTable(const CSSFontFace& face, const CSSValue& familyToSearchFor)
 {
-    for (auto& item : familiesToSearchFor) {
-        String familyName = CSSFontFaceSet::familyNameFromPrimitive(downcast<CSSPrimitiveValue>(item));
-        if (familyName.isEmpty())
-            continue;
+    auto familyName = CSSFontFaceSet::familyNameFromPrimitive(downcast<CSSPrimitiveValue>(familyToSearchFor));
+    if (familyName.isNull())
+        return;
 
-        auto iterator = m_facesLookupTable.find(familyName);
-        if (iterator == m_facesLookupTable.end()) {
-            // The font may have failed even before addToFacesLookupTable() was called on it,
-            // which means we never added it (because there's no point in adding a failed font).
-            // So, if it was never added, removing it is free! Woohoo!
-            return;
-        }
-        bool found = false;
-        for (size_t i = 0; i < iterator->value.size(); ++i) {
-            if (iterator->value[i].ptr() == &face) {
-                found = true;
-                iterator->value.remove(i);
-                break;
-            }
-        }
-        ASSERT_UNUSED(found, found);
-        if (!iterator->value.size())
-            m_facesLookupTable.remove(iterator);
+    auto iterator = m_facesLookupTable.find(familyName);
+    if (iterator == m_facesLookupTable.end()) {
+        // The font may have failed even before addToFacesLookupTable() was called on it,
+        // which means we never added it (because there's no point in adding a failed font).
+        // So, if it was never added, removing it is free! Woohoo!
+        return;
     }
+    bool found = false;
+    for (size_t i = 0; i < iterator->value.size(); ++i) {
+        if (iterator->value[i].ptr() == &face) {
+            found = true;
+            iterator->value.removeAt(i);
+            break;
+        }
+    }
+    ASSERT_UNUSED(found, found);
+    if (!iterator->value.size())
+        m_facesLookupTable.remove(iterator);
 }
 
 void CSSFontFaceSet::remove(const CSSFontFace& face)
@@ -253,12 +252,12 @@ void CSSFontFaceSet::remove(const CSSFontFace& face)
 
     m_cache.clear();
 
-    m_fontModifiedObservers.forEach([] (auto& observer) {
+    m_fontModifiedObservers.forEach([](auto& observer) {
         observer();
     });
     
-    if (auto families = face.families())
-        removeFromFacesLookupTable(face, *families);
+    if (auto family = face.familyCSSValue())
+        removeFromFacesLookupTable(face, *family);
 
     if (face.cssConnection()) {
         ASSERT(m_constituentCSSConnections.get(face.cssConnection()) == &face);
@@ -269,8 +268,8 @@ void CSSFontFaceSet::remove(const CSSFontFace& face)
         if (m_faces[i].ptr() == &face) {
             if (i < m_facesPartitionIndex)
                 --m_facesPartitionIndex;
-            m_faces[i]->removeClient(*this);
-            m_faces.remove(i);
+            Ref { m_faces[i] }->removeClient(*this);
+            m_faces.removeAt(i);
             if (face.status() == CSSFontFace::Status::Loading || face.status() == CSSFontFace::Status::TimedOut)
                 decrementActiveCount();
             return;
@@ -320,10 +319,10 @@ CSSFontFace& CSSFontFaceSet::operator[](size_t i)
     return m_faces[i];
 }
 
-static FontSelectionRequest computeFontSelectionRequest(CSSPropertyParserHelpers::FontRaw& font)
+static FontSelectionRequest computeFontSelectionRequest(CSSPropertyParserHelpers::UnresolvedFont& font)
 {
-    auto weightSelectionValue = font.weight
-        ? WTF::switchOn(*font.weight, [&] (CSSValueID keyword) {
+    auto weightSelectionValue = WTF::switchOn(font.weight,
+        [&](CSSValueID keyword) {
             switch (keyword) {
             case CSSValueNormal:
                 return normalWeightValue();
@@ -336,70 +335,92 @@ static FontSelectionRequest computeFontSelectionRequest(CSSPropertyParserHelpers
                 ASSERT_NOT_REACHED();
                 return normalWeightValue();
             }
-        }, [&] (double weight) {
-            return FontSelectionValue::clampFloat(weight);
-        }) : normalWeightValue();
+        },
+        [&](const CSSPropertyParserHelpers::UnresolvedFontWeightNumber& weight) {
+            // FIXME: Figure out correct behavior when conversion data is required.
+            if (requiresConversionData(weight))
+                return normalWeightValue();
+            return FontSelectionValue::clampFloat(Style::toStyleNoConversionDataRequired(weight).value);
+        }
+    );
 
-    // Because this is a FontRaw, we know we should be able to dereference stretchSelectionValue as
-    // consumeFontStretchKeywordValueRaw only returns results valid to pass to fontStretchValue.
-    auto stretchSelectionValue = fontStretchValue(font.stretch.value_or(CSSValueNormal));
-    ASSERT(stretchSelectionValue);
+    auto widthSelectionValue = WTF::switchOn(font.width,
+        [&](CSSValueID ident) -> FontSelectionValue {
+            return *fontWidthValue(ident);
+        },
+        [&](const CSSPropertyParserHelpers::UnresolvedFontWidthPercentage& percent) -> FontSelectionValue  {
+            // FIXME: Figure out correct behavior when conversion data is required.
+            if (requiresConversionData(percent))
+                return normalWidthValue();
+            return FontSelectionValue::clampFloat(Style::toStyleNoConversionDataRequired(percent).value);
+        }
+    );
 
-    auto styleKeyword = font.style ? font.style->style : CSSValueNormal;
-    auto styleSelectionValue = [&] () -> std::optional<FontSelectionValue> {
-        if (styleKeyword == CSSValueNormal)
-            return std::nullopt;
-        if (styleKeyword == CSSValueItalic)
-            return italicValue();
-        ASSERT(font.style && styleKeyword == CSSValueOblique);
-        float degrees = 0;
-        if (font.style->angle)
-            degrees = static_cast<float>(CSSPrimitiveValue::computeDegrees(font.style->angle->type, font.style->angle->value));
-        return FontSelectionValue(degrees);
-    }();
+    auto styleSelectionValue = WTF::switchOn(font.style,
+        [&](CSSValueID ident) -> std::optional<FontSelectionValue> {
+            switch (ident) {
+            case CSSValueNormal:
+                return std::nullopt;
+            case CSSValueItalic:
+                return italicValue();
+            case CSSValueOblique:
+                return FontSelectionValue(0.0f); // FIXME: Spec says this should be 14deg.
+            default:
+                ASSERT_NOT_REACHED();
+                return std::nullopt;
+            }
+        },
+        [&](const CSSPropertyParserHelpers::UnresolvedFontStyleObliqueAngle& angle) -> std::optional<FontSelectionValue> {
+            // FIXME: Figure out correct behavior when conversion data is required.
+            if (requiresConversionData(angle))
+                return std::nullopt;
+            return FontSelectionValue::clampFloat(Style::toStyleNoConversionDataRequired(angle).value);
+        }
+    );
 
-    return { weightSelectionValue, *stretchSelectionValue, styleSelectionValue };
+    return { weightSelectionValue, widthSelectionValue, styleSelectionValue };
 }
 
-using CodePointsMap = HashSet<UChar32, DefaultHash<UChar32>, WTF::UnsignedWithZeroKeyHashTraits<UChar32>>;
+using CodePointsMap = HashSet<uint32_t, DefaultHash<uint32_t>, WTF::UnsignedWithZeroKeyHashTraits<uint32_t>>;
 static CodePointsMap codePointsFromString(StringView stringView)
 {
     CodePointsMap result;
     auto graphemeClusters = stringView.graphemeClusters();
     for (auto cluster : graphemeClusters) {
         ASSERT(cluster.length() > 0);
-        UChar32 character = 0;
+        char32_t character = 0;
         if (cluster.is8Bit())
             character = cluster[0];
-        else
-            U16_GET(cluster.characters16(), 0, 0, cluster.length(), character);
+        else {
+            auto characters = cluster.span16();
+            U16_GET(characters, 0, 0, characters.size(), character);
+        }
         result.add(character);
     }
     return result;
 }
 
-ExceptionOr<Vector<std::reference_wrapper<CSSFontFace>>> CSSFontFaceSet::matchingFacesExcludingPreinstalledFonts(const String& fontShorthand, const String& string)
+ExceptionOr<Vector<std::reference_wrapper<CSSFontFace>>> CSSFontFaceSet::matchingFacesExcludingPreinstalledFonts(ScriptExecutionContext& context, const String& fontShorthand, const String& string)
 {
-    auto font = CSSPropertyParserWorkerSafe::parseFont(fontShorthand, HTMLStandardMode);
+    auto font = CSSPropertyParserHelpers::parseUnresolvedFont(fontShorthand, context);
     if (!font)
-        return Exception { SyntaxError };
+        return Exception { ExceptionCode::SyntaxError };
 
     HashSet<AtomString> uniqueFamilies;
     Vector<AtomString> familyOrder;
     for (auto& familyRaw : font->family) {
-        AtomString familyAtom;
-        WTF::switchOn(familyRaw, [&] (CSSValueID familyKeyword) {
-            if (familyKeyword != CSSValueWebkitBody)
-                familyAtom = familyNamesData->at(CSSPropertyParserHelpers::genericFontFamilyIndex(familyKeyword));
-            else {
-                ASSERT(m_owningFontSelector && m_owningFontSelector->scriptExecutionContext());
-                familyAtom = AtomString { m_owningFontSelector->scriptExecutionContext()->settingsValues().fontGenericFamilies.standardFontFamily() };
+        auto familyAtom = WTF::switchOn(familyRaw,
+            [&](CSSValueID familyKeyword) -> AtomString {
+                if (familyKeyword == CSSValueWebkitBody)
+                    return AtomString { context.settingsValues().fontGenericFamilies.standardFontFamily() };
+                return *familyNamesData->at(CSSPropertyParserHelpers::genericFontFamilyIndex(familyKeyword));
+            },
+            [&](const AtomString& familyString) -> AtomString  {
+                return familyString;
             }
-        }, [&] (const AtomString& familyString) {
-            familyAtom = familyString;
-        });
+        );
 
-        if (!familyAtom.isEmpty() && uniqueFamilies.add(familyAtom).isNewEntry)
+        if (!familyAtom.isNull() && uniqueFamilies.add(familyAtom).isNewEntry)
             familyOrder.append(familyAtom);
     }
 
@@ -430,9 +451,9 @@ ExceptionOr<Vector<std::reference_wrapper<CSSFontFace>>> CSSFontFaceSet::matchin
     });
 }
 
-ExceptionOr<bool> CSSFontFaceSet::check(const String& font, const String& text)
+ExceptionOr<bool> CSSFontFaceSet::check(ScriptExecutionContext& context, const String& font, const String& text)
 {
-    auto matchingFaces = this->matchingFacesExcludingPreinstalledFonts(font, text);
+    auto matchingFaces = this->matchingFacesExcludingPreinstalledFonts(context, font, text);
     if (matchingFaces.hasException())
         return matchingFaces.releaseException();
 
@@ -485,15 +506,15 @@ CSSSegmentedFontFace* CSSFontFaceSet::fontFace(FontSelectionRequest request, con
             return face.get().fontSelectionCapabilities();
         });
         FontSelectionAlgorithm fontSelectionAlgorithm(request, capabilities);
-        std::stable_sort(candidateFontFaces.begin(), candidateFontFaces.end(), [&fontSelectionAlgorithm](const CSSFontFace& first, const CSSFontFace& second) {
-            auto firstCapabilities = first.fontSelectionCapabilities();
-            auto secondCapabilities = second.fontSelectionCapabilities();
+        std::ranges::stable_sort(candidateFontFaces, [&fontSelectionAlgorithm](auto& first, auto& second) {
+            auto firstCapabilities = first.get().fontSelectionCapabilities();
+            auto secondCapabilities = second.get().fontSelectionCapabilities();
             
-            auto stretchDistanceFirst = fontSelectionAlgorithm.stretchDistance(firstCapabilities).distance;
-            auto stretchDistanceSecond = fontSelectionAlgorithm.stretchDistance(secondCapabilities).distance;
-            if (stretchDistanceFirst < stretchDistanceSecond)
+            auto widthDistanceFirst = fontSelectionAlgorithm.widthDistance(firstCapabilities).distance;
+            auto widthDistanceSecond = fontSelectionAlgorithm.widthDistance(secondCapabilities).distance;
+            if (widthDistanceFirst < widthDistanceSecond)
                 return true;
-            if (stretchDistanceFirst > stretchDistanceSecond)
+            if (widthDistanceFirst > widthDistanceSecond)
                 return false;
 
             auto styleDistanceFirst = fontSelectionAlgorithm.styleDistance(firstCapabilities).distance;
@@ -537,16 +558,16 @@ void CSSFontFaceSet::fontStateChanged(CSSFontFace& face, CSSFontFace::Status old
     }
 }
 
-void CSSFontFaceSet::fontPropertyChanged(CSSFontFace& face, CSSValueList* oldFamilies)
+void CSSFontFaceSet::fontPropertyChanged(CSSFontFace& face, CSSValue* oldFamily)
 {
     m_cache.clear();
 
-    if (oldFamilies) {
-        removeFromFacesLookupTable(face, *oldFamilies);
+    if (oldFamily) {
+        removeFromFacesLookupTable(face, *oldFamily);
         addToFacesLookupTable(face);
     }
 
-    m_fontModifiedObservers.forEach([] (auto& observer) {
+    m_fontModifiedObservers.forEach([](auto& observer) {
         observer();
     });
 }
