@@ -27,7 +27,7 @@
 #include "config.h"
 #include "CSSFontSelector.h"
 
-#include "CachedFont.h"
+#include "CSSCounterStyleRegistry.h"
 #include "CSSFontFace.h"
 #include "CSSFontFaceSource.h"
 #include "CSSFontFeatureValuesRule.h"
@@ -36,8 +36,9 @@
 #include "CSSSegmentedFontFace.h"
 #include "CSSValueKeywords.h"
 #include "CSSValueList.h"
+#include "CachedFont.h"
 #include "CachedResourceLoader.h"
-#include "Document.h"
+#include "DocumentSettingsValues.h"
 #include "Font.h"
 #include "FontCache.h"
 #include "FontCascadeDescription.h"
@@ -48,7 +49,6 @@
 #include "LocalFrame.h"
 #include "Logging.h"
 #include "ResourceLoadObserver.h"
-#include "Settings.h"
 #include "StyleProperties.h"
 #include "StyleResolver.h"
 #include "StyleRule.h"
@@ -73,21 +73,24 @@ CSSFontSelector::CSSFontSelector(ScriptExecutionContext& context)
     : ActiveDOMObject(&context)
     , m_context(context)
     , m_cssFontFaceSet(CSSFontFaceSet::create(this))
-    , m_fontModifiedObserver([this] { fontModified(); })
+    , m_fontModifiedObserver(CSSFontFaceSet::FontModifiedObserver::create([weakThis = WeakPtr { *this }] {
+        if (RefPtr protectedThis = weakThis.get())
+            protectedThis->fontModified();
+    }))
     , m_uniqueId(++fontSelectorId)
     , m_version(0)
 {
     if (is<Document>(context)) {
         m_fontFamilyNames.reserveInitialCapacity(familyNames->size());
         for (auto& familyName : familyNames.get())
-            m_fontFamilyNames.uncheckedConstructAndAppend(familyName);
+            m_fontFamilyNames.constructAndAppend(familyName);
     } else {
-        m_fontFamilyNames.reserveInitialCapacity(familyNamesData->size());
-        for (auto& familyName : familyNamesData.get())
-            m_fontFamilyNames.uncheckedAppend(familyName);
+        m_fontFamilyNames.appendContainerWithMapping(familyNamesData.get(), [](auto& familyName) {
+            return AtomString { *familyName };
+        });
     }
 
-    FontCache::forCurrentThread().addClient(*this);
+    FontCache::forCurrentThread()->addClient(*this);
     m_cssFontFaceSet->addFontModifiedObserver(m_fontModifiedObserver);
     LOG(Fonts, "CSSFontSelector %p ctor", this);
 }
@@ -98,7 +101,7 @@ CSSFontSelector::~CSSFontSelector()
 
     clearFonts();
 
-    if (auto fontCache = FontCache::forCurrentThreadIfNotDestroyed())
+    if (CheckedPtr fontCache = FontCache::forCurrentThreadIfNotDestroyed())
         fontCache->removeClient(*this);
 }
 
@@ -111,7 +114,7 @@ FontFaceSet& CSSFontSelector::fontFaceSet()
 {
     if (!m_fontFaceSet) {
         ASSERT(m_context);
-        m_fontFaceSet = FontFaceSet::create(*m_context, m_cssFontFaceSet.get());
+        m_fontFaceSet = FontFaceSet::create(protectedScriptExecutionContext(), m_cssFontFaceSet.get());
     }
 
     return *m_fontFaceSet;
@@ -137,9 +140,9 @@ void CSSFontSelector::buildStarted()
     ASSERT(m_cssConnectionsEncounteredDuringBuild.isEmpty());
     ASSERT(m_stagingArea.isEmpty());
     for (size_t i = 0; i < m_cssFontFaceSet->faceCount(); ++i) {
-        CSSFontFace& face = m_cssFontFaceSet.get()[i];
-        if (face.cssConnection())
-            m_cssConnectionsPossiblyToRemove.add(&face);
+        Ref face = m_cssFontFaceSet.get()[i];
+        if (face->cssConnection())
+            m_cssConnectionsPossiblyToRemove.add(face.get());
     }
 
     m_paletteMap.clear();
@@ -154,7 +157,7 @@ void CSSFontSelector::buildCompleted()
 
     // Some font faces weren't re-added during the build process.
     for (auto& face : m_cssConnectionsPossiblyToRemove) {
-        auto* connection = face->cssConnection();
+        RefPtr connection = face->cssConnection();
         ASSERT(connection);
         if (!m_cssConnectionsEncounteredDuringBuild.contains(connection))
             m_cssFontFaceSet->remove(*face);
@@ -175,39 +178,33 @@ void CSSFontSelector::addFontFaceRule(StyleRuleFontFace& fontFaceRule, bool isIn
         return;
     }
 
-    const StyleProperties& style = fontFaceRule.properties();
-    RefPtr<CSSValue> fontFamily = style.getPropertyCSSValue(CSSPropertyFontFamily);
-    RefPtr<CSSValue> fontStyle = style.getPropertyCSSValue(CSSPropertyFontStyle);
-    RefPtr<CSSValue> fontWeight = style.getPropertyCSSValue(CSSPropertyFontWeight);
-    RefPtr<CSSValue> fontStretch = style.getPropertyCSSValue(CSSPropertyFontStretch);
-    RefPtr<CSSValue> src = style.getPropertyCSSValue(CSSPropertySrc);
-    RefPtr<CSSValue> unicodeRange = style.getPropertyCSSValue(CSSPropertyUnicodeRange);
-    RefPtr<CSSValue> featureSettings = style.getPropertyCSSValue(CSSPropertyFontFeatureSettings);
-    RefPtr<CSSValue> display = style.getPropertyCSSValue(CSSPropertyFontDisplay);
-    RefPtr<CSSValue> sizeAdjust = style.getPropertyCSSValue(CSSPropertySizeAdjust);
-    if (!is<CSSValueList>(fontFamily) || !is<CSSValueList>(src) || (unicodeRange && !is<CSSValueList>(*unicodeRange)))
+    Ref style = fontFaceRule.properties();
+    RefPtr fontFamily = style->getPropertyCSSValue(CSSPropertyFontFamily);
+    RefPtr fontStyle = style->getPropertyCSSValue(CSSPropertyFontStyle);
+    RefPtr fontWeight = style->getPropertyCSSValue(CSSPropertyFontWeight);
+    RefPtr fontWidth = style->getPropertyCSSValue(CSSPropertyFontWidth);
+    RefPtr srcList = dynamicDowncast<CSSValueList>(style->getPropertyCSSValue(CSSPropertySrc));
+    RefPtr unicodeRange = style->getPropertyCSSValue(CSSPropertyUnicodeRange);
+    RefPtr rangeList = downcast<CSSValueList>(unicodeRange.get());
+    RefPtr featureSettings = style->getPropertyCSSValue(CSSPropertyFontFeatureSettings);
+    RefPtr display = style->getPropertyCSSValue(CSSPropertyFontDisplay);
+    RefPtr sizeAdjust = style->getPropertyCSSValue(CSSPropertySizeAdjust);
+    if (!fontFamily || !srcList || (unicodeRange && !rangeList))
         return;
 
-    CSSValueList& familyList = downcast<CSSValueList>(*fontFamily);
-    if (!familyList.length())
-        return;
-
-    CSSValueList* rangeList = downcast<CSSValueList>(unicodeRange.get());
-
-    CSSValueList& srcList = downcast<CSSValueList>(*src);
-    if (!srcList.length())
+    if (!srcList->length())
         return;
 
     SetForScope creatingFont(m_creatingFont, true);
     auto fontFace = CSSFontFace::create(*this, &fontFaceRule);
 
-    fontFace->setFamilies(familyList);
+    fontFace->setFamily(*fontFamily);
     if (fontStyle)
         fontFace->setStyle(*fontStyle);
     if (fontWeight)
         fontFace->setWeight(*fontWeight);
-    if (fontStretch)
-        fontFace->setStretch(*fontStretch);
+    if (fontWidth)
+        fontFace->setWidth(*fontWidth);
     if (rangeList)
         fontFace->setUnicodeRange(*rangeList);
     if (featureSettings)
@@ -217,7 +214,7 @@ void CSSFontSelector::addFontFaceRule(StyleRuleFontFace& fontFaceRule, bool isIn
     if (sizeAdjust)
         fontFace->setSizeAdjust(*sizeAdjust);
 
-    CSSFontFace::appendSources(fontFace, srcList, m_context.get(), isInitiatingElementInUserAgentShadowTree);
+    CSSFontFace::appendSources(fontFace, *srcList, protectedScriptExecutionContext().ptr(), isInitiatingElementInUserAgentShadowTree);
 
     if (RefPtr<CSSFontFace> existingFace = m_cssFontFaceSet->lookUpByCSSConnection(fontFaceRule)) {
         // This adoption is fairly subtle. Script can trigger a purge of m_cssFontFaceSet at any time,
@@ -231,7 +228,7 @@ void CSSFontSelector::addFontFaceRule(StyleRuleFontFace& fontFaceRule, bool isIn
         // to enter the correct state() during the next pump(). This approach of making a new CSSFontFace is
         // simpler than computing and applying a diff of the StyleProperties.
         m_cssFontFaceSet->remove(*existingFace);
-        if (auto* existingWrapper = existingFace->existingWrapper())
+        if (RefPtr existingWrapper = existingFace->existingWrapper())
             existingWrapper->adopt(fontFace.get());
     }
 
@@ -241,9 +238,16 @@ void CSSFontSelector::addFontFaceRule(StyleRuleFontFace& fontFaceRule, bool isIn
 
 void CSSFontSelector::addFontPaletteValuesRule(const StyleRuleFontPaletteValues& fontPaletteValuesRule)
 {
-    AtomString fontFamily = fontPaletteValuesRule.fontFamily().isNull() ? emptyAtom() : fontPaletteValuesRule.fontFamily();
-    AtomString name = fontPaletteValuesRule.name().isNull() ? emptyAtom() : fontPaletteValuesRule.name();
-    m_paletteMap.set(std::make_pair(fontFamily, name), fontPaletteValuesRule.fontPaletteValues());
+
+    auto& name = fontPaletteValuesRule.name();
+    ASSERT(!name.isNull());
+
+    auto& fontFamilies = fontPaletteValuesRule.fontFamilies();
+    if (fontFamilies.isEmpty())
+        return;
+
+    for (auto& fontFamily : fontFamilies)
+        m_paletteMap.set(std::make_pair(fontFamily, name), fontPaletteValuesRule.fontPaletteValues());
 
     ++m_version;
 }
@@ -255,8 +259,7 @@ void CSSFontSelector::addFontFeatureValuesRule(const StyleRuleFontFeatureValues&
     for (const auto& fontFamily : fontFeatureValuesRule.fontFamilies()) {
         // https://www.w3.org/TR/css-fonts-3/#font-family-casing
         auto lowercased = fontFamily.string().convertToLowercaseWithoutLocale();
-        auto exist = m_featureValues.get(lowercased);
-        if (exist)
+        if (RefPtr exist = m_featureValues.get(lowercased))
             exist->updateOrInsert(fontFeatureValues.get());
         else
             m_featureValues.set(lowercased, fontFeatureValues);
@@ -279,8 +282,10 @@ void CSSFontSelector::dispatchInvalidationCallbacks()
 {
     ++m_version;
 
-    for (auto& client : copyToVector(m_clients))
-        client->fontsNeedUpdate(*this);
+    for (auto& client : copyToVector(m_clients)) {
+        if (m_clients.contains(client))
+            client->fontsNeedUpdate(*this);
+    }
 }
 
 void CSSFontSelector::opportunisticallyStartFontDataURLLoading(const FontCascadeDescription& description, const AtomString& familyName)
@@ -288,8 +293,15 @@ void CSSFontSelector::opportunisticallyStartFontDataURLLoading(const FontCascade
     const auto& segmentedFontFace = m_cssFontFaceSet->fontFace(description.fontSelectionRequest(), familyName);
     if (!segmentedFontFace)
         return;
+
+    RefPtr context = m_context.get();
+    if (!context)
+        return;
+
+    auto trustedType = context->settingsValues().downloadableBinaryFontTrustedTypes;
+
     for (auto& face : segmentedFontFace->constituentFaces())
-        face->opportunisticallyStartFontDataURLLoading();
+        face->opportunisticallyStartFontDataURLLoading(trustedType);
 }
 
 void CSSFontSelector::fontLoaded(CSSFontFace&)
@@ -305,8 +317,8 @@ void CSSFontSelector::fontModified()
 
 void CSSFontSelector::updateStyleIfNeeded()
 {
-    if (is<Document>(m_context))
-        downcast<Document>(*m_context).updateStyleIfNeeded();
+    if (RefPtr document = dynamicDowncast<Document>(m_context.get()))
+        document->updateStyleIfNeeded();
 }
 
 void CSSFontSelector::updateStyleIfNeeded(CSSFontFace&)
@@ -328,7 +340,7 @@ std::optional<AtomString> CSSFontSelector::resolveGenericFamily(const FontDescri
     if (!m_context)
         return std::nullopt;
 
-    const auto& settings = m_context->settingsValues();
+    const auto& settings = protectedScriptExecutionContext()->settingsValues();
 
     UScriptCode script = fontDescription.script();
     auto familyNameIndex = m_fontFamilyNames.find(familyName);
@@ -340,7 +352,7 @@ std::optional<AtomString> CSSFontSelector::resolveGenericFamily(const FontDescri
     return std::nullopt;
 }
 
-const FontPaletteValues& CSSFontSelector::lookupFontPaletteValues(const AtomString& familyName, const FontDescription& fontDescription)
+const FontPaletteValues& CSSFontSelector::lookupFontPaletteValues(const AtomString& familyName, const FontDescription& fontDescription) const
 {
     static NeverDestroyed<FontPaletteValues> emptyFontPaletteValues;
     if (fontDescription.fontPalette().type != FontPalette::Type::Custom)
@@ -351,10 +363,11 @@ const FontPaletteValues& CSSFontSelector::lookupFontPaletteValues(const AtomStri
     auto iterator = m_paletteMap.find(std::make_pair(familyName, paletteName));
     if (iterator == m_paletteMap.end())
         return emptyFontPaletteValues.get();
+
     return iterator->value;
 }
 
-RefPtr<FontFeatureValues> CSSFontSelector::lookupFontFeatureValues(const AtomString& familyName)
+RefPtr<FontFeatureValues> CSSFontSelector::lookupFontFeatureValues(const AtomString& familyName) const
 {
     // https://www.w3.org/TR/css-fonts-3/#font-family-casing
     auto lowercased = familyName.string().convertToLowercaseWithoutLocale();
@@ -365,6 +378,56 @@ RefPtr<FontFeatureValues> CSSFontSelector::lookupFontFeatureValues(const AtomStr
     return iterator->value.ptr();
 }
 
+// Fonts with appropriate Unicode coverage and OpenType features are required for good math
+// rendering. These requirements as well as the up-to-date list of known math fonts to fulfill
+// these requirements are listed on http://trac.webkit.org/wiki/MathML/Fonts.
+using MathFontList = std::array<AtomString, 19>;
+static const MathFontList& mathFontList()
+{
+    static const NeverDestroyed list = MathFontList {
+        // This font has Computer Modern style and is provided with most TeX & Linux distributions.
+        // We put it as the default because its style is familiar to TeX, Wikipedia and math people.
+        "Latin Modern Math"_s,
+        // The following fonts have Times style and are provided with most TeX & Linux distributions.
+        // We put XITS & STIX as a second option because they have very good unicode coverage.
+        // STIX Two is a complete redesign of STIX that fixes serious bugs in version one so we put it in first position.
+        // XITS is a fork of STIX with bug fixes and more Arabic/RTL features so we put it in second position.
+        "STIX Two Math"_s,
+        "XITS Math"_s,
+        "STIX Math"_s,
+        "Libertinus Math"_s,
+        "TeX Gyre Termes Math"_s,
+        // These fonts respectively have style compatible with Bookman Old and Century Schoolbook.
+        // They are provided with most TeX & Linux distributions.
+        "TeX Gyre Bonum Math"_s,
+        "TeX Gyre Schola"_s,
+        // DejaVu is pre-installed on many Linux distributions and is included in LibreOffice.
+        "DejaVu Math TeX Gyre"_s,
+        // The following fonts have Palatino style and are provided with most TeX & Linux distributions.
+        // Asana Math has some rendering issues (e.g. missing italic correction) so we put it after.
+        "TeX Gyre Pagella Math"_s,
+        "Asana Math"_s,
+        // The following fonts are proprietary and have not much been tested so we put them at the end.
+        // Cambria Math it is pre-installed on Windows 7 and higher.
+        "Cambria Math"_s,
+        "Lucida Bright Math"_s,
+        "Minion Math"_s,
+        // The following fonts do not satisfy the requirements for good mathematical rendering.
+        // These are pre-installed on Mac and iOS so we list them to provide minimal unicode-based
+        // mathematical rendering. For more explanation of fallback mechanisms and missing features see
+        // http://trac.webkit.org/wiki/MathML/Fonts#ObsoleteFontsandFallbackMechanisms.
+        // STIX fonts have best unicode coverage so we put them first.
+        "STIXGeneral"_s,
+        "STIXSizeOneSym"_s,
+        "Symbol"_s,
+        "Times New Roman"_s,
+        // Mathematical fonts generally use "serif" style. Hence we append the generic "serif" family
+        // as a fallback in order to increase our chance to find a mathematical font.
+        "serif"_s,
+    };
+    return list;
+}
+
 FontRanges CSSFontSelector::fontRangesForFamily(const FontDescription& fontDescription, const AtomString& familyName)
 {
     // If this ASSERT() fires, it usually means you forgot a document.updateStyleIfNeeded() somewhere.
@@ -374,33 +437,53 @@ FontRanges CSSFontSelector::fontRangesForFamily(const FontDescription& fontDescr
     bool resolveGenericFamilyFirst = familyName == m_fontFamilyNames.at(FamilyNamesIndex::StandardFamily);
 
     AtomString familyForLookup = familyName;
-    std::optional<FontDescription> overrideFontDescription;
+    auto isGenericFontFamily = IsGenericFontFamily::No;
     const FontDescription* fontDescriptionForLookup = &fontDescription;
-    auto resolveAndAssignGenericFamily = [&]() {
-        if (auto genericFamilyOptional = resolveGenericFamily(fontDescription, familyName))
+    auto resolveAndAssignGenericFamily = [&] {
+        if (auto genericFamilyOptional = resolveGenericFamily(fontDescription, familyName)) {
             familyForLookup = *genericFamilyOptional;
+            isGenericFontFamily = IsGenericFontFamily::Yes;
+        }
     };
 
     const auto& fontPaletteValues = lookupFontPaletteValues(familyName, fontDescription);
     auto fontFeatureValues = lookupFontFeatureValues(familyName);
 
+    // Handle the generic math font family a bit differently.
+    if (familyName == m_fontFamilyNames.at(FamilyNamesIndex::MathFamily)) {
+        // First check if the user has defined a preference.
+        const auto& settings = protectedScriptExecutionContext()->settingsValues();
+        const String& preferredMathFamily = settings.fontGenericFamilies.mathFontFamily(fontDescription.script());
+        if (!preferredMathFamily.isEmpty() && familyName != preferredMathFamily) {
+            auto ranges = fontRangesForFamily(fontDescription, AtomString(preferredMathFamily));
+            if (!ranges.isNull())
+                return { WTFMove(ranges), IsGenericFontFamily::Yes };
+        }
+
+        // Otherwise, iterate through the font list to find a valid fallback.
+        for (auto& family : mathFontList()) {
+            auto ranges = fontRangesForFamily(fontDescription, family);
+            if (!ranges.isNull())
+                return { WTFMove(ranges), IsGenericFontFamily::Yes };
+        }
+    }
+
     if (resolveGenericFamilyFirst)
         resolveAndAssignGenericFamily();
-    auto* document = dynamicDowncast<Document>(m_context.get());
-    auto* face = m_cssFontFaceSet->fontFace(fontDescriptionForLookup->fontSelectionRequest(), familyForLookup);
-    if (face) {
+    RefPtr document = dynamicDowncast<Document>(m_context.get());
+    if (RefPtr face = m_cssFontFaceSet->fontFace(fontDescriptionForLookup->fontSelectionRequest(), familyForLookup)) {
         if (document && document->settings().webAPIStatisticsEnabled())
-            ResourceLoadObserver::shared().logFontLoad(*document, familyForLookup.string(), true);
-        return face->fontRanges(*fontDescriptionForLookup, fontPaletteValues, fontFeatureValues);
+            ResourceLoadObserver::singleton().logFontLoad(*document, familyForLookup.string(), true);
+        return { face->fontRanges(*fontDescriptionForLookup, fontPaletteValues, fontFeatureValues), isGenericFontFamily };
     }
 
     if (!resolveGenericFamilyFirst)
         resolveAndAssignGenericFamily();
 
-    auto font = FontCache::forCurrentThread().fontForFamily(*fontDescriptionForLookup, familyForLookup, { { }, { }, fontPaletteValues, fontFeatureValues, 1.0 });
+    auto font = FontCache::forCurrentThread()->fontForFamily(*fontDescriptionForLookup, familyForLookup, { { }, { }, fontPaletteValues, fontFeatureValues, 1.0 });
     if (document && document->settings().webAPIStatisticsEnabled())
-        ResourceLoadObserver::shared().logFontLoad(*document, familyForLookup.string(), !!font);
-    return FontRanges { WTFMove(font) };
+        ResourceLoadObserver::singleton().logFontLoad(*document, familyForLookup.string(), !!font);
+    return { FontRanges { WTFMove(font) }, isGenericFontFamily };
 }
 
 void CSSFontSelector::clearFonts()
@@ -415,7 +498,7 @@ size_t CSSFontSelector::fallbackFontCount()
     if (m_isStopped)
         return 0;
 
-    return m_context->settingsValues().fontFallbackPrefersPictographs ? 1 : 0;
+    return protectedScriptExecutionContext()->settingsValues().fontFallbackPrefersPictographs ? 1 : 0;
 }
 
 RefPtr<Font> CSSFontSelector::fallbackFontAt(const FontDescription& fontDescription, size_t index)
@@ -425,14 +508,30 @@ RefPtr<Font> CSSFontSelector::fallbackFontAt(const FontDescription& fontDescript
     if (m_isStopped)
         return nullptr;
 
-    if (!m_context->settingsValues().fontFallbackPrefersPictographs)
+    RefPtr context = m_context.get();
+    if (!context->settingsValues().fontFallbackPrefersPictographs)
         return nullptr;
-    auto& pictographFontFamily = m_context->settingsValues().fontGenericFamilies.pictographFontFamily();
-    auto font = FontCache::forCurrentThread().fontForFamily(fontDescription, pictographFontFamily);
-    if (is<Document>(m_context) && m_context->settingsValues().webAPIStatisticsEnabled)
-        ResourceLoadObserver::shared().logFontLoad(downcast<Document>(*m_context), pictographFontFamily, !!font);
+    auto& pictographFontFamily = context->settingsValues().fontGenericFamilies.pictographFontFamily();
+    RefPtr font = FontCache::forCurrentThread()->fontForFamily(fontDescription, pictographFontFamily);
+    if (RefPtr document = dynamicDowncast<Document>(context.get()); document && document->settingsValues().webAPIStatisticsEnabled)
+        ResourceLoadObserver::singleton().logFontLoad(*document, pictographFontFamily, !!font);
 
     return font;
 }
 
+bool CSSFontSelector::isSimpleFontSelectorForDescription() const
+{
+    // font face rules still pending
+    if (m_stagingArea.size())
+        return false;
+
+    // FIXME: remove this when we fix counter style rules mutation.
+    if (RefPtr document = dynamicDowncast<Document>(m_context.get())) {
+        if (document->counterStyleRegistry().hasAuthorCounterStyles())
+            return false;
+    }
+
+    return !m_cssFontFaceSet->faceCount() && m_featureValues.isEmpty() && m_paletteMap.isEmpty();
+}
+
 }
