@@ -1,7 +1,7 @@
 /*
  * (C) 1999-2003 Lars Knoll (knoll@kde.org)
  * (C) 2002-2003 Dirk Mueller (mueller@kde.org)
- * Copyright (C) 2002, 2006, 2008, 2012 Apple Inc. All rights reserved.
+ * Copyright (C) 2002-2020 Apple Inc. All rights reserved.
  * Copyright (C) 2006 Samuel Weinig (sam@webkit.org)
  *
  * This library is free software; you can redistribute it and/or
@@ -22,33 +22,41 @@
 
 #pragma once
 
-#include "CSSGroupingRule.h"
+#include <WebCore/CSSConditionRule.h>
 
 namespace WebCore {
 
 class MediaList;
 class StyleRuleMedia;
 
-class CSSMediaRule final : public CSSGroupingRule {
+namespace MQ {
+struct MediaQuery;
+using MediaQueryList = Vector<MediaQuery>;
+}
+
+class CSSMediaRule final : public CSSConditionRule {
 public:
     static Ref<CSSMediaRule> create(StyleRuleMedia& rule, CSSStyleSheet* sheet) { return adoptRef(*new CSSMediaRule(rule, sheet)); }
-
     virtual ~CSSMediaRule();
 
     WEBCORE_EXPORT MediaList* media() const;
 
 private:
+    friend class MediaList;
+
     CSSMediaRule(StyleRuleMedia&, CSSStyleSheet*);
 
-    CSSRule::Type type() const final { return MEDIA_RULE; }
-    void reattach(StyleRuleBase&) final;
+    StyleRuleType styleRuleType() const final { return StyleRuleType::Media; }
     String cssText() const final;
+    String cssText(const CSS::SerializationContext&) const final;
+    String conditionText() const final;
+
+    const MQ::MediaQueryList& mediaQueries() const;
+    void setMediaQueries(MQ::MediaQueryList&&);
 
-    MediaQuerySet* mediaQueries() const;
-    
     mutable RefPtr<MediaList> m_mediaCSSOMWrapper;
 };
 
 } // namespace WebCore
 
-SPECIALIZE_TYPE_TRAITS_CSS_RULE(CSSMediaRule, CSSRule::MEDIA_RULE)
+SPECIALIZE_TYPE_TRAITS_CSS_RULE(CSSMediaRule, StyleRuleType::Media)
