@@ -2,8 +2,9 @@
  * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
  *           (C) 2001 Dirk Mueller (mueller@kde.org)
- * Copyright (C) 2004, 2005, 2006, 2010 Apple Inc. All rights reserved.
+ * Copyright (C) 2004-2022 Apple Inc. All rights reserved.
  *           (C) 2006 Alexey Proskuryakov (ap@nypop.com)
+ * Copyright (C) 2018 Google Inc. All rights reserved.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -28,61 +29,37 @@
 #include "ElementIterator.h"
 #include "HTMLFieldSetElement.h"
 #include "HTMLNames.h"
+#include "SelectionRestorationMode.h"
+#include <wtf/TZoneMallocInlines.h>
 
 namespace WebCore {
 
-using namespace HTMLNames;
-
+WTF_MAKE_TZONE_OR_ISO_ALLOCATED_IMPL(HTMLLegendElement);
 
 inline HTMLLegendElement::HTMLLegendElement(const QualifiedName& tagName, Document& document)
     : HTMLElement(tagName, document)
 {
-    ASSERT(hasTagName(legendTag));
+    ASSERT(hasTagName(HTMLNames::legendTag));
 }
 
-RefPtr<HTMLLegendElement> HTMLLegendElement::create(const QualifiedName& tagName, Document& document)
+Ref<HTMLLegendElement> HTMLLegendElement::create(const QualifiedName& tagName, Document& document)
 {
-    return adoptRef(new HTMLLegendElement(tagName, document));
+    return adoptRef(*new HTMLLegendElement(tagName, document));
 }
 
-HTMLFormControlElement* HTMLLegendElement::associatedControl()
-{
-    // Check if there's a fieldset belonging to this legend.
-    auto enclosingFieldset = ancestorsOfType<HTMLFieldSetElement>(*this).first();
-    if (!enclosingFieldset)
-        return nullptr;
-
-    // Find first form element inside the fieldset that is not a legend element.
-    // FIXME: Should we consider tabindex?
-    return descendantsOfType<HTMLFormControlElement>(*enclosingFieldset).first();
-}
-
-void HTMLLegendElement::focus(bool, FocusDirection direction)
-{
-    if (isFocusable())
-        Element::focus(true, direction);
-        
-    // To match other browsers' behavior, never restore previous selection.
-    if (HTMLFormControlElement* control = associatedControl())
-        control->focus(false, direction);
-}
-
-void HTMLLegendElement::accessKeyAction(bool sendMouseEvents)
-{
-    if (HTMLFormControlElement* control = associatedControl())
-        control->accessKeyAction(sendMouseEvents);
-}
-
-HTMLFormElement* HTMLLegendElement::virtualForm() const
+HTMLFormElement* HTMLLegendElement::form() const
 {
     // According to the specification, If the legend has a fieldset element as
     // its parent, then the form attribute must return the same value as the
     // form attribute on that fieldset element. Otherwise, it must return null.
-    ContainerNode* fieldset = parentNode();
-    if (!is<HTMLFieldSetElement>(fieldset))
-        return nullptr;
+    RefPtr fieldset = dynamicDowncast<HTMLFieldSetElement>(parentNode());
+    return fieldset ? fieldset->form() : nullptr;
+}
 
-    return downcast<HTMLFieldSetElement>(*fieldset).form();
+RefPtr<HTMLFormElement> HTMLLegendElement::formForBindings() const
+{
+    // FIXME: The downcast should be unnecessary, but the WPT was written before https://github.com/WICG/webcomponents/issues/1072 was resolved. Update once the WPT has been updated.
+    return dynamicDowncast<HTMLFormElement>(retargetReferenceTargetForBindings(form()));
 }
     
 } // namespace
