@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013 Google Inc. All Rights Reserved.
+ * Copyright (C) 2013 Google Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -23,23 +23,29 @@
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#ifndef HTMLResourcePreloader_h
-#define HTMLResourcePreloader_h
+#pragma once
 
 #include "CachedResource.h"
 #include "CachedResourceRequest.h"
+#include "ScriptType.h"
+#include <wtf/RefCountedAndCanMakeWeakPtr.h>
+#include <wtf/TZoneMalloc.h>
+#include <wtf/WeakRef.h>
 
 namespace WebCore {
 
 class PreloadRequest {
-    WTF_MAKE_FAST_ALLOCATED;
+    WTF_MAKE_TZONE_ALLOCATED(PreloadRequest);
 public:
-    PreloadRequest(const String& initiator, const String& resourceURL, const URL& baseURL, CachedResource::Type resourceType, const String& mediaAttribute)
-        : m_initiator(initiator)
+    PreloadRequest(ASCIILiteral initiatorType, const String& resourceURL, const URL& baseURL, CachedResource::Type resourceType, const String& mediaAttribute, ScriptType scriptType, const ReferrerPolicy& referrerPolicy, RequestPriority fetchPriority = RequestPriority::Auto)
+        : m_initiatorType(initiatorType)
         , m_resourceURL(resourceURL)
         , m_baseURL(baseURL.isolatedCopy())
         , m_resourceType(resourceType)
         , m_mediaAttribute(mediaAttribute)
+        , m_scriptType(scriptType)
+        , m_referrerPolicy(referrerPolicy)
+        , m_fetchPriority(fetchPriority)
     {
     }
 
@@ -49,41 +55,43 @@ public:
     const String& media() const { return m_mediaAttribute; }
     void setCharset(const String& charset) { m_charset = charset.isolatedCopy(); }
     void setCrossOriginMode(const String& mode) { m_crossOriginMode = mode; }
+    void setNonce(const String& nonce) { m_nonceAttribute = nonce; }
+    void setScriptIsAsync(bool value) { m_scriptIsAsync = value; }
     CachedResource::Type resourceType() const { return m_resourceType; }
 
 private:
     URL completeURL(Document&);
 
-    String m_initiator;
+    ASCIILiteral m_initiatorType;
     String m_resourceURL;
     URL m_baseURL;
     String m_charset;
     CachedResource::Type m_resourceType;
     String m_mediaAttribute;
     String m_crossOriginMode;
+    String m_nonceAttribute;
+    bool m_scriptIsAsync { false };
+    ScriptType m_scriptType;
+    ReferrerPolicy m_referrerPolicy;
+    RequestPriority m_fetchPriority;
 };
 
 typedef Vector<std::unique_ptr<PreloadRequest>> PreloadRequestStream;
 
-class HTMLResourcePreloader {
-    WTF_MAKE_NONCOPYABLE(HTMLResourcePreloader); WTF_MAKE_FAST_ALLOCATED;
+class HTMLResourcePreloader : public RefCountedAndCanMakeWeakPtr<HTMLResourcePreloader> {
+    WTF_MAKE_TZONE_ALLOCATED(HTMLResourcePreloader);
+    WTF_MAKE_NONCOPYABLE(HTMLResourcePreloader);
 public:
-    explicit HTMLResourcePreloader(Document& document)
-        : m_document(document)
-        , m_weakFactory(this)
-    {
-    }
+    static Ref<HTMLResourcePreloader> create(Document&);
+    ~HTMLResourcePreloader();
 
     void preload(PreloadRequestStream);
     void preload(std::unique_ptr<PreloadRequest>);
 
-    WeakPtr<HTMLResourcePreloader> createWeakPtr() { return m_weakFactory.createWeakPtr(); }
-
 private:
-    Document& m_document;
-    WeakPtrFactory<HTMLResourcePreloader> m_weakFactory;
+    explicit HTMLResourcePreloader(Document&);
+
+    WeakPtr<Document, WeakPtrImplWithEventTargetData> m_document;
 };
 
-}
-
-#endif
+} // namespace WebCore
