@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2017-2023 Apple Inc.  All rights reserved.
+ * Copyright (C) 2017-2023 Apple Inc. All rights reserved.
+ * Copyright (C) 2025 Samuel Weinig <sam@webkit.org>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -29,7 +30,6 @@
 #include "AffineTransform.h"
 #include "CSSStyleImageValue.h"
 #include "CachedImage.h"
-#include "CanvasBase.h"
 #include "CanvasGradient.h"
 #include "CanvasPattern.h"
 #include "CanvasRenderingContext2D.h"
@@ -65,35 +65,16 @@
 #include "JSWebGL2RenderingContext.h"
 #include "JSWebGLRenderingContext.h"
 #include "Path2D.h"
-#include "Pattern.h"
-#include "RecordingSwizzleType.h"
 #include "SVGPathUtilities.h"
 #include "StringAdaptors.h"
-#include "WebGL2RenderingContext.h"
-#include "WebGLBuffer.h"
-#include "WebGLFramebuffer.h"
-#include "WebGLProgram.h"
-#include "WebGLQuery.h"
-#include "WebGLRenderbuffer.h"
-#include "WebGLRenderingContext.h"
-#include "WebGLRenderingContextBase.h"
-#include "WebGLSampler.h"
-#include "WebGLShader.h"
-#include "WebGLSync.h"
-#include "WebGLTexture.h"
-#include "WebGLTransformFeedback.h"
-#include "WebGLUniformLocation.h"
-#include "WebGLVertexArrayObject.h"
-#include <JavaScriptCore/ArrayBuffer.h>
-#include <JavaScriptCore/ArrayBufferView.h>
 #include <JavaScriptCore/IdentifiersFactory.h>
+#include <JavaScriptCore/JSCInlines.h>
 #include <JavaScriptCore/ScriptCallStackFactory.h>
-#include <JavaScriptCore/TypedArrays.h>
-#include <variant>
 #include <wtf/Function.h>
 #include <wtf/RefPtr.h>
 #include <wtf/Scope.h>
 #include <wtf/Vector.h>
+#include <wtf/text/MakeString.h>
 #include <wtf/text/WTFString.h>
 
 #if ENABLE(OFFSCREEN_CANVAS)
@@ -206,511 +187,6 @@ template<typename T> static Ref<JSON::ArrayOf<JSON::Value>> buildArrayForVector(
     return array;
 }
 
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(CanvasDirection argument)
-{
-    return {{ valueIndexForData(convertEnumerationToString(argument)), RecordingSwizzleType::String }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(CanvasFillRule argument)
-{
-    return {{ valueIndexForData(convertEnumerationToString(argument)), RecordingSwizzleType::String }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(CanvasImageSource& argument)
-{
-    return WTF::switchOn(argument, [&] (auto& value) {
-        return processArgument(value);
-    });
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(CanvasLineCap argument)
-{
-    return {{ valueIndexForData(convertEnumerationToString(argument)), RecordingSwizzleType::String }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(CanvasLineJoin argument)
-{
-    return {{ valueIndexForData(convertEnumerationToString(argument)), RecordingSwizzleType::String }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(CanvasRenderingContext2DBase::StyleVariant& argument)
-{
-    return WTF::switchOn(argument, [&] (auto& value) {
-        return processArgument(value);
-    });
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(CanvasTextAlign argument)
-{
-    return {{ valueIndexForData(convertEnumerationToString(argument)), RecordingSwizzleType::String }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(CanvasTextBaseline argument)
-{
-    return {{ valueIndexForData(convertEnumerationToString(argument)), RecordingSwizzleType::String }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(DOMMatrix2DInit& argument)
-{
-    auto array = JSON::ArrayOf<double>::create();
-    array->addItem(argument.a.value_or(1));
-    array->addItem(argument.b.value_or(0));
-    array->addItem(argument.c.value_or(0));
-    array->addItem(argument.d.value_or(1));
-    array->addItem(argument.e.value_or(0));
-    array->addItem(argument.f.value_or(0));
-    return {{ WTFMove(array), RecordingSwizzleType::DOMMatrix }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(Element* argument)
-{
-    if (!argument)
-        return std::nullopt;
-
-    // Elements are not serializable, so add a string as a placeholder since the actual
-    // element cannot be reconstructed in the frontend.
-    return {{ valueIndexForData("Element"_s), RecordingSwizzleType::None }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(HTMLImageElement* argument)
-{
-    if (!argument)
-        return std::nullopt;
-    return {{ valueIndexForData(argument), RecordingSwizzleType::Image }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(ImageBitmap* argument)
-{
-    if (!argument)
-        return std::nullopt;
-    return {{ valueIndexForData(argument), RecordingSwizzleType::ImageBitmap }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(ImageData* argument)
-{
-    if (!argument)
-        return std::nullopt;
-    return {{ valueIndexForData(argument), RecordingSwizzleType::ImageData }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(ImageDataSettings&)
-{
-    // FIXME: Implement.
-    return std::nullopt;
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(ImageSmoothingQuality argument)
-{
-    return {{ valueIndexForData(convertEnumerationToString(argument)), RecordingSwizzleType::String }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(std::optional<double>& argument)
-{
-    if (!argument)
-        return std::nullopt;
-    return {{ JSON::Value::create(*argument), RecordingSwizzleType::Number }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(std::optional<float>& argument)
-{
-    if (!argument)
-        return std::nullopt;
-    return {{ JSON::Value::create(static_cast<double>(*argument)), RecordingSwizzleType::Number }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(Path2D* argument)
-{
-    if (!argument)
-        return std::nullopt;
-    return {{ valueIndexForData(buildStringFromPath(argument->path())), RecordingSwizzleType::Path2D }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(PredefinedColorSpace argument)
-{
-    return { { valueIndexForData(convertEnumerationToString(argument)), RecordingSwizzleType::String } };
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(RefPtr<CanvasGradient>& argument)
-{
-    if (!argument)
-        return std::nullopt;
-    return {{ valueIndexForData(argument), RecordingSwizzleType::CanvasGradient }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(RefPtr<CanvasPattern>& argument)
-{
-    if (!argument)
-        return std::nullopt;
-    return {{ valueIndexForData(argument), RecordingSwizzleType::CanvasPattern }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(RefPtr<HTMLCanvasElement>& argument)
-{
-    if (!argument)
-        return std::nullopt;
-    return {{ valueIndexForData(argument), RecordingSwizzleType::Image }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(RefPtr<HTMLImageElement>& argument)
-{
-    return processArgument(argument.get());
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(RefPtr<ImageBitmap>& argument)
-{
-    return processArgument(argument.get());
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(RefPtr<ImageData>& argument)
-{
-    return processArgument(argument.get());
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(Ref<JSC::ArrayBuffer>&)
-{
-    return {{ JSON::Value::create(0), RecordingSwizzleType::TypedArray }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(Ref<JSC::ArrayBufferView>&)
-{
-    return {{ JSON::Value::create(0), RecordingSwizzleType::TypedArray }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(RefPtr<JSC::ArrayBuffer>& argument)
-{
-    if (!argument)
-        return std::nullopt;
-    return {{ JSON::Value::create(0), RecordingSwizzleType::TypedArray }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(RefPtr<JSC::ArrayBufferView>& argument)
-{
-    if (!argument)
-        return std::nullopt;
-    return {{ JSON::Value::create(0), RecordingSwizzleType::TypedArray }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(RefPtr<JSC::Float32Array>& argument)
-{
-    if (!argument)
-        return std::nullopt;
-    return {{ JSON::Value::create(0), RecordingSwizzleType::TypedArray }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(RefPtr<JSC::Int32Array>& argument)
-{
-    if (!argument)
-        return std::nullopt;
-    return {{ JSON::Value::create(0), RecordingSwizzleType::TypedArray }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(RefPtr<JSC::Uint32Array>& argument)
-{
-    if (!argument)
-        return std::nullopt;
-    return {{ JSON::Value::create(0), RecordingSwizzleType::TypedArray }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(String& argument)
-{
-    return {{ valueIndexForData(argument), RecordingSwizzleType::String }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(Vector<String>& argument)
-{
-    auto deduplicated = argument.map([&] (const String& item) {
-        return indexForData(item);
-    });
-    return {{ buildArrayForVector(WTFMove(deduplicated)), RecordingSwizzleType::String }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(Vector<double>& argument)
-{
-    return {{ buildArrayForVector(argument), RecordingSwizzleType::Array }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(Vector<float>& argument)
-{
-    return {{ buildArrayForVector(argument), RecordingSwizzleType::Array }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(Vector<uint32_t>& argument)
-{
-    auto mapped = argument.map([&] (uint32_t item) {
-        return static_cast<double>(item);
-    });
-    return {{ buildArrayForVector(mapped), RecordingSwizzleType::Array }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(Vector<int32_t>& argument)
-{
-    auto mapped = argument.map([&] (int32_t item) {
-        return static_cast<double>(item);
-    });
-    return {{ buildArrayForVector(mapped), RecordingSwizzleType::Array }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(CanvasPath::RadiusVariant& argument)
-{
-    return WTF::switchOn(argument,
-        [](DOMPointInit) -> std::optional<InspectorCanvasCallTracer::ProcessedArgument> {
-            // FIXME We'd likely want to either create a new RecordingSwizzleType::DOMPointInit or RecordingSwizzleType::Object to avoid
-            // encoding the same data multiple times. See https://webkit.org/b/233255
-            return std::nullopt;
-        },
-        [](double radius) -> std::optional<InspectorCanvasCallTracer::ProcessedArgument> {
-            return { { JSON::Value::create(radius), RecordingSwizzleType::Number } };
-        });
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(WTF::Vector<CanvasPath::RadiusVariant>& argument)
-{
-    auto processed = argument.map([&](const CanvasPath::RadiusVariant& item) -> Ref<JSON::Value> {
-        return WTF::switchOn(item,
-            [](DOMPointInit point) -> Ref<JSON::Value> {
-                auto object = JSON::Object::create();
-                object->setDouble("x"_s, point.x);
-                object->setDouble("y"_s, point.y);
-                object->setDouble("z"_s, point.z);
-                object->setDouble("w"_s, point.w);
-                // FIXME We'd likely want to either create a new RecordingSwizzleType::DOMPointInit or RecordingSwizzleType::Object to avoid
-                // encoding the same data multiple times
-                return object;
-            },
-            [](double radius) -> Ref<JSON::Value> {
-                return JSON::Value::create(radius);
-            });
-    });
-    // Did not use buildArrayForVector due to WTFMov'ing the Ref<Value> to the vector as Value copy constructor was deleted.
-    auto array = JSON::ArrayOf<JSON::Value>::create();
-    for (auto& item : processed)
-        array->addItem(WTFMove(item));
-    return { { array, RecordingSwizzleType::Array } };
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(double argument)
-{
-    return {{ JSON::Value::create(argument), RecordingSwizzleType::Number }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(float argument)
-{
-    return {{ JSON::Value::create(static_cast<double>(argument)), RecordingSwizzleType::Number }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(uint64_t argument)
-{
-    return {{ JSON::Value::create(static_cast<double>(argument)), RecordingSwizzleType::Number }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(int64_t argument)
-{
-    return {{ JSON::Value::create(static_cast<double>(argument)), RecordingSwizzleType::Number }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(uint32_t argument)
-{
-    return {{ JSON::Value::create(static_cast<double>(argument)), RecordingSwizzleType::Number }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(int32_t argument)
-{
-    return {{ JSON::Value::create(static_cast<double>(argument)), RecordingSwizzleType::Number }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(uint8_t argument)
-{
-    return {{ JSON::Value::create(static_cast<int>(argument)), RecordingSwizzleType::Number }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(bool argument)
-{
-    return {{ JSON::Value::create(argument), RecordingSwizzleType::Boolean }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(RefPtr<CSSStyleImageValue>& argument)
-{
-    if (!argument)
-        return std::nullopt;
-    return {{ valueIndexForData(argument), RecordingSwizzleType::Image }};
-}
-
-#if ENABLE(OFFSCREEN_CANVAS)
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(RefPtr<OffscreenCanvas>& argument)
-{
-    if (!argument)
-        return std::nullopt;
-    return {{ valueIndexForData(argument), RecordingSwizzleType::Image }};
-}
-
-#endif // ENABLE(OFFSCREEN_CANVAS)
-
-#if ENABLE(VIDEO)
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(RefPtr<HTMLVideoElement>& argument)
-{
-    if (!argument)
-        return std::nullopt;
-    return {{ valueIndexForData(argument), RecordingSwizzleType::Image }};
-}
-
-#endif // ENABLE(VIDEO)
-
-#if ENABLE(WEB_CODECS)
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(RefPtr<WebCodecsVideoFrame>& argument)
-{
-    if (!argument)
-        return std::nullopt;
-    return {{ JSON::Value::create(0), RecordingSwizzleType::Image }};
-}
-
-#endif // ENABLE(WEB_CODECS)
-
-#if ENABLE(WEBGL)
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(std::optional<WebGLRenderingContextBase::BufferDataSource>& argument)
-{
-    if (!argument)
-        return std::nullopt;
-
-    return WTF::switchOn(*argument, [&] (auto& value) {
-        return processArgument(value);
-    });
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(std::optional<WebGLRenderingContextBase::TexImageSource>& argument)
-{
-    if (!argument)
-        return std::nullopt;
-
-    return WTF::switchOn(*argument, [&] (auto& value) {
-        return processArgument(value);
-    });
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(WebGLBuffer* argument)
-{
-    if (!argument)
-        return std::nullopt;
-    return {{ JSON::Value::create(static_cast<int>(argument->object())), RecordingSwizzleType::WebGLBuffer }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(WebGLFramebuffer* argument)
-{
-    if (!argument)
-        return std::nullopt;
-    return {{ JSON::Value::create(static_cast<int>(argument->object())), RecordingSwizzleType::WebGLFramebuffer }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(WebGLProgram* argument)
-{
-    if (!argument)
-        return std::nullopt;
-    return {{ JSON::Value::create(static_cast<int>(argument->object())), RecordingSwizzleType::WebGLProgram }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(WebGLQuery* argument)
-{
-    if (!argument)
-        return std::nullopt;
-    return {{ JSON::Value::create(static_cast<int>(argument->object())), RecordingSwizzleType::WebGLQuery }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(WebGLRenderbuffer* argument)
-{
-    if (!argument)
-        return std::nullopt;
-    return {{ JSON::Value::create(static_cast<int>(argument->object())), RecordingSwizzleType::WebGLRenderbuffer }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(WebGLRenderingContextBase::BufferDataSource& argument)
-{
-    return WTF::switchOn(argument, [&] (auto& value) {
-        return processArgument(value);
-    });
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(WebGLRenderingContextBase::Float32List::VariantType& argument)
-{
-    return WTF::switchOn(argument, [&] (auto& value) {
-        return processArgument(value);
-    });
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(WebGLRenderingContextBase::Int32List::VariantType& argument)
-{
-    return WTF::switchOn(argument, [&] (auto& value) {
-        return processArgument(value);
-    });
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(WebGLRenderingContextBase::TexImageSource& argument)
-{
-    return WTF::switchOn(argument, [&] (auto& value) {
-        return processArgument(value);
-    });
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(WebGLSampler* argument)
-{
-    if (!argument)
-        return std::nullopt;
-    return {{ JSON::Value::create(static_cast<int>(argument->object())), RecordingSwizzleType::WebGLSampler }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(WebGLShader* argument)
-{
-    if (!argument)
-        return std::nullopt;
-    return {{ JSON::Value::create(static_cast<int>(argument->object())), RecordingSwizzleType::WebGLShader }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(WebGLSync* argument)
-{
-    if (!argument)
-        return std::nullopt;
-    return {{ JSON::Value::create(static_cast<int>(argument->object())), RecordingSwizzleType::WebGLSync }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(WebGLTexture* argument)
-{
-    if (!argument)
-        return std::nullopt;
-    return {{ JSON::Value::create(static_cast<int>(argument->object())), RecordingSwizzleType::WebGLTexture }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(WebGLUniformLocation* argument)
-{
-    if (!argument)
-        return std::nullopt;
-    return {{ JSON::Value::create(argument->location()), RecordingSwizzleType::WebGLUniformLocation }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(WebGLVertexArrayObject* argument)
-{
-    if (!argument)
-        return std::nullopt;
-    return {{ JSON::Value::create(static_cast<int>(argument->object())), RecordingSwizzleType::WebGLVertexArrayObject }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(WebGLTransformFeedback* argument)
-{
-    if (!argument)
-        return std::nullopt;
-    return {{ JSON::Value::create(static_cast<int>(argument->object())), RecordingSwizzleType::WebGLTransformFeedback }};
-}
-
-std::optional<InspectorCanvasCallTracer::ProcessedArgument> InspectorCanvas::processArgument(WebGL2RenderingContext::Uint32List::VariantType& argument)
-{
-    return WTF::switchOn(argument, [&] (auto& value) {
-        return processArgument(value);
-    });
-}
-
-#endif // ENABLE(WEBGL)
-
 static bool shouldSnapshotBitmapRendererAction(const String& name)
 {
     return name == "transferFromImageBitmap"_s;
@@ -734,7 +210,7 @@ static bool shouldSnapshotWebGL2Action(const String& name)
 }
 #endif
 
-void InspectorCanvas::recordAction(String&& name, InspectorCanvasCallTracer::ProcessedArguments&& arguments)
+void InspectorCanvas::recordAction(String&& name, InspectorCanvasProcessedArguments&& arguments)
 {
     if (!m_initialState) {
         // We should only construct the initial state for the first action of the recording.
@@ -783,7 +259,7 @@ void InspectorCanvas::finalizeFrame()
     appendActionSnapshotIfNeeded();
 
     if (m_frames && m_frames->length() && !m_currentFrameStartTime.isNaN()) {
-        auto currentFrame = static_reference_cast<Inspector::Protocol::Recording::Frame>(m_frames->get(m_frames->length() - 1));
+        auto currentFrame = unsafeRefDowncast<Inspector::Protocol::Recording::Frame>(m_frames->get(m_frames->length() - 1));
         currentFrame->setDuration((MonotonicTime::now() - m_currentFrameStartTime).milliseconds());
 
         m_currentFrameStartTime = MonotonicTime::nan();
@@ -797,7 +273,7 @@ void InspectorCanvas::markCurrentFrameIncomplete()
     if (!m_currentActions || !m_frames || !m_frames->length())
         return;
 
-    auto currentFrame = static_reference_cast<Inspector::Protocol::Recording::Frame>(m_frames->get(m_frames->length() - 1));
+    auto currentFrame = unsafeRefDowncast<Inspector::Protocol::Recording::Frame>(m_frames->get(m_frames->length() - 1));
     currentFrame->setIncomplete(true);
 }
 
@@ -1072,7 +548,7 @@ int InspectorCanvas::indexForData(DuplicateDataVariant data)
             if (CachedImage* cachedImage = imageElement->cachedImage()) {
                 Image* image = cachedImage->image();
                 if (image && image != &Image::nullImage()) {
-                    auto imageBuffer = ImageBuffer::create(image->size(), RenderingPurpose::Unspecified, 1, DestinationColorSpace::SRGB(), ImageBufferPixelFormat::BGRA8);
+                    auto imageBuffer = ImageBuffer::create(image->size(), RenderingMode::Unaccelerated, RenderingPurpose::Unspecified, 1, DestinationColorSpace::SRGB(), PixelFormat::BGRA8);
                     imageBuffer->context().drawImage(*image, FloatPoint(0, 0));
                     dataURL = imageBuffer->toDataURL("image/png"_s);
                 }
@@ -1086,7 +562,7 @@ int InspectorCanvas::indexForData(DuplicateDataVariant data)
 
             unsigned videoWidth = videoElement->videoWidth();
             unsigned videoHeight = videoElement->videoHeight();
-            auto imageBuffer = ImageBuffer::create(FloatSize(videoWidth, videoHeight), RenderingPurpose::Unspecified, 1, DestinationColorSpace::SRGB(), ImageBufferPixelFormat::BGRA8);
+            auto imageBuffer = ImageBuffer::create(FloatSize(videoWidth, videoHeight), RenderingMode::Unaccelerated, RenderingPurpose::Unspecified, 1, DestinationColorSpace::SRGB(), PixelFormat::BGRA8);
             if (imageBuffer) {
                 videoElement->paintCurrentFrameInContext(imageBuffer->context(), FloatRect(0, 0, videoWidth, videoHeight));
                 dataURL = imageBuffer->toDataURL("image/png"_s);
@@ -1150,7 +626,7 @@ int InspectorCanvas::indexForData(DuplicateDataVariant data)
             if (auto* cachedImage = cssImageValue->image()) {
                 auto* image = cachedImage->image();
                 if (image && image != &Image::nullImage()) {
-                    auto imageBuffer = ImageBuffer::create(image->size(), RenderingPurpose::Unspecified, 1, DestinationColorSpace::SRGB(), ImageBufferPixelFormat::BGRA8);
+                    auto imageBuffer = ImageBuffer::create(image->size(), RenderingMode::Unaccelerated, RenderingPurpose::Unspecified, 1, DestinationColorSpace::SRGB(), PixelFormat::BGRA8);
                     imageBuffer->context().drawImage(*image, FloatPoint(0, 0));
                     dataURL = imageBuffer->toDataURL("image/png"_s);
                 }
@@ -1263,7 +739,7 @@ Ref<Inspector::Protocol::Recording::InitialState> InspectorCanvas::buildInitialS
             else if (auto canvasPattern = state.strokeStyle.canvasPattern())
                 strokeStyleIndex = indexForData(canvasPattern);
             else
-                strokeStyleIndex = indexForData(state.strokeStyle.color());
+                strokeStyleIndex = indexForData(state.strokeStyle.colorString());
             statePayload->setInteger(stringIndexForKey("strokeStyle"_s), strokeStyleIndex);
 
             int fillStyleIndex;
@@ -1272,7 +748,7 @@ Ref<Inspector::Protocol::Recording::InitialState> InspectorCanvas::buildInitialS
             else if (auto canvasPattern = state.fillStyle.canvasPattern())
                 fillStyleIndex = indexForData(canvasPattern);
             else
-                fillStyleIndex = indexForData(state.fillStyle.color());
+                fillStyleIndex = indexForData(state.fillStyle.colorString());
             statePayload->setInteger(stringIndexForKey("fillStyle"_s), fillStyleIndex);
 
             statePayload->setBoolean(stringIndexForKey("imageSmoothingEnabled"_s), state.imageSmoothingEnabled);
@@ -1304,7 +780,7 @@ Ref<Inspector::Protocol::Recording::InitialState> InspectorCanvas::buildInitialS
     return initialStatePayload;
 }
 
-Ref<JSON::ArrayOf<JSON::Value>> InspectorCanvas::buildAction(String&& name, InspectorCanvasCallTracer::ProcessedArguments&& arguments)
+Ref<JSON::ArrayOf<JSON::Value>> InspectorCanvas::buildAction(String&& name, InspectorCanvasProcessedArguments&& arguments)
 {
     auto action = JSON::ArrayOf<JSON::Value>::create();
     action->addItem(indexForData(WTFMove(name)));
@@ -1394,12 +870,8 @@ Ref<JSON::ArrayOf<JSON::Value>> InspectorCanvas::buildArrayForCanvasPattern(cons
 
 Ref<JSON::ArrayOf<JSON::Value>> InspectorCanvas::buildArrayForImageData(const ImageData& imageData)
 {
-    auto data = JSON::ArrayOf<int>::create();
-    for (size_t i = 0; i < imageData.data().length(); ++i)
-        data->addItem(imageData.data().item(i));
-
     auto array = JSON::ArrayOf<JSON::Value>::create();
-    array->addItem(WTFMove(data));
+    array->addItem(imageData.data().copyToJSONArray());
     array->addItem(imageData.width());
     array->addItem(imageData.height());
     return array;
