@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2018 Apple Inc. All rights reserved.
+ * Copyright (C) 2018-2024 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -27,23 +27,26 @@
 #include "LayoutBox.h"
 
 #include "LayoutBoxGeometry.h"
+#include "LayoutBoxInlines.h"
 #include "LayoutContainingBlockChainIterator.h"
 #include "LayoutElementBox.h"
 #include "LayoutInitialContainingBlock.h"
 #include "LayoutPhase.h"
+#include "LayoutShape.h"
 #include "LayoutState.h"
 #include "RenderObject.h"
 #include "RenderStyleInlines.h"
-#include "Shape.h"
-#include <wtf/IsoMallocInlines.h>
 #include <wtf/NeverDestroyed.h>
+#include <wtf/TZoneMallocInlines.h>
 
 namespace WebCore {
 namespace Layout {
 
-WTF_MAKE_ISO_ALLOCATED_IMPL(Box);
+WTF_MAKE_TZONE_OR_ISO_ALLOCATED_IMPL(Box);
+WTF_MAKE_TZONE_ALLOCATED_IMPL(Box::BoxRareData);
 
-Box::Box(ElementAttributes&& elementAttributes, RenderStyle&& style, std::unique_ptr<RenderStyle>&& firstLineStyle, OptionSet<BaseTypeFlag> baseTypeFlags)
+
+Box::Box(ElementAttributes&& elementAttributes, RenderStyle&& style, std::unique_ptr<RenderStyle>&& firstLineStyle, EnumSet<BaseTypeFlag> baseTypeFlags)
     : m_nodeType(elementAttributes.nodeType)
     , m_isAnonymous(static_cast<bool>(elementAttributes.isAnonymous))
     , m_baseTypeFlags(baseTypeFlags.toRaw())
@@ -55,7 +58,7 @@ Box::Box(ElementAttributes&& elementAttributes, RenderStyle&& style, std::unique
 
 Box::~Box()
 {
-    if (UNLIKELY(m_hasRareData))
+    if (m_hasRareData) [[unlikely]]
         removeRareData();
     if (m_renderer)
         m_renderer->clearLayoutBox();
@@ -82,6 +85,8 @@ void Box::updateStyle(RenderStyle&& newStyle, std::unique_ptr<RenderStyle>&& new
     m_style = WTFMove(newStyle);
     if (newFirstLineStyle)
         ensureRareData().firstLineStyle = WTFMove(newFirstLineStyle);
+    else if (hasRareData())
+        rareData().firstLineStyle = { };
 }
 
 bool Box::establishesFormattingContext() const
@@ -92,6 +97,7 @@ bool Box::establishesFormattingContext() const
         || establishesBlockFormattingContext()
         || establishesTableFormattingContext()
         || establishesFlexFormattingContext()
+        || establishesGridFormattingContext()
         || establishesIndependentFormattingContext();
 }
 
@@ -118,7 +124,7 @@ bool Box::establishesBlockFormattingContext() const
     if (isFloatingPositioned()) {
         // Not all floating or out-of-positioned block level boxes establish BFC.
         // See [9.7 Relationships between 'display', 'position', and 'float'] for details.
-        return style().display() == DisplayType::Block;
+        return isBlockContainer();
     }
 
     if (isBlockContainer() && !isBlockBox())
@@ -162,9 +168,14 @@ bool Box::establishesFlexFormattingContext() const
     return isFlexBox();
 }
 
+bool Box::establishesGridFormattingContext() const
+{
+    return isGridFormattingContext();
+}
+
 bool Box::establishesIndependentFormattingContext() const
 {
-    return isLayoutContainmentBox() || isAbsolutelyPositioned() || isFlexItem();
+    return isLayoutContainmentBox() || isAbsolutelyPositioned() || isFlexItem() || isGridItem();
 }
 
 bool Box::isRelativelyPositioned() const
@@ -227,6 +238,7 @@ bool Box::isBlockLevelBox() const
         || display == DisplayType::Table
         || display == DisplayType::Flex
         || display == DisplayType::Grid
+        || display == DisplayType::GridLanes
         || display == DisplayType::FlowRoot;
 }
 
@@ -244,6 +256,7 @@ bool Box::isInlineLevelBox() const
         || display == DisplayType::InlineBox
         || display == DisplayType::InlineFlex
         || display == DisplayType::InlineGrid
+        || display == DisplayType::InlineGridLanes
         || display == DisplayType::Ruby
         || display == DisplayType::RubyBase
         || display == DisplayType::RubyAnnotation
@@ -259,7 +272,7 @@ bool Box::isInlineBox() const
     return (display == DisplayType::Inline || display == DisplayType::Ruby || display == DisplayType::RubyBase) && !isReplacedBox();
 }
 
-bool Box::isAtomicInlineLevelBox() const
+bool Box::isAtomicInlineBox() const
 {
     // Inline-level boxes that are not inline boxes (such as replaced inline-level elements, inline-block elements, and inline-table elements)
     // are called atomic inline-level boxes because they participate in their inline formatting context as a single opaque box.
@@ -272,6 +285,11 @@ bool Box::isFlexItem() const
     return isInFlow() && parent().isFlexBox();
 }
 
+bool Box::isGridItem() const
+{
+    return isInFlow() && parent().isGridFormattingContext();
+}
+
 bool Box::isBlockContainer() const
 {
     auto display = m_style.display();
@@ -295,10 +313,10 @@ bool Box::isLayoutContainmentBox() const
         if (isInternalRubyBox())
             return false;
         if (isInlineLevelBox())
-            return isAtomicInlineLevelBox();
+            return isAtomicInlineBox();
         return true;
     };
-    return m_style.usedContain().contains(Containment::Layout) && supportsLayoutContainment();
+    return m_style.usedContain().contains(Style::ContainValue::Layout) && supportsLayoutContainment();
 }
 
 bool Box::isRubyAnnotationBox() const
@@ -308,7 +326,7 @@ bool Box::isRubyAnnotationBox() const
 
 bool Box::isInterlinearRubyAnnotationBox() const
 {
-    return isRubyAnnotationBox() && m_style.rubyPosition() != RubyPosition::InterCharacter;
+    return isRubyAnnotationBox() && !m_style.isInterCharacterRubyPosition();
 }
 
 bool Box::isInternalRubyBox() const
@@ -327,10 +345,10 @@ bool Box::isSizeContainmentBox() const
         if (isInternalRubyBox())
             return false;
         if (isInlineLevelBox())
-            return isAtomicInlineLevelBox();
+            return isAtomicInlineBox();
         return true;
     };
-    return m_style.usedContain().contains(Containment::Size) && supportsSizeContainment();
+    return m_style.usedContain().contains(Style::ContainValue::Size) && supportsSizeContainment();
 }
 
 bool Box::isInternalTableBox() const
@@ -356,6 +374,14 @@ const Box* Box::nextInFlowOrFloatingSibling() const
     return nextSibling;
 }
 
+const Box* Box::nextOutOfFlowSibling() const
+{
+    auto* nextSibling = this->nextSibling();
+    while (nextSibling && !nextSibling->isOutOfFlowPositioned())
+        nextSibling = nextSibling->nextSibling();
+    return nextSibling;
+}
+
 const Box* Box::previousInFlowSibling() const
 {
     auto* previousSibling = this->previousSibling();
@@ -372,17 +398,46 @@ const Box* Box::previousInFlowOrFloatingSibling() const
     return previousSibling;
 }
 
-bool Box::isDescendantOf(const ElementBox& ancestor) const
+const Box* Box::previousOutOfFlowSibling() const
 {
-    if (ancestor.isInitialContainingBlock())
-        return true;
-    for (auto& containingBlock : containingBlockChain(*this)) {
-        if (&containingBlock == &ancestor)
+    auto* previousSibling = this->previousSibling();
+    while (previousSibling && !previousSibling->isOutOfFlowPositioned())
+        previousSibling = previousSibling->previousSibling();
+    return previousSibling;
+}
+
+bool Box::isDescendantOf(const Box& box) const
+{
+
+    for (auto* ancestor = &parent(); !ancestor->isInitialContainingBlock(); ancestor = &ancestor->parent()) {
+        if (ancestor == &box)
             return true;
     }
     return false;
 }
 
+bool Box::isDescendantOfWithinFormattingContext(const Box& box) const
+{
+    for (auto* ancestor = &parent(); !ancestor->establishesFormattingContext(); ancestor = &ancestor->parent()) {
+        if (ancestor == &box)
+            return true;
+    }
+    return false;
+}
+
+bool Box::isInFormattingContextEstablishedBy(const ElementBox& formattingContextRoot) const
+{
+    ASSERT(formattingContextRoot.establishesFormattingContext());
+
+    auto* ancestor = &parent();
+    while (true) {
+        if (ancestor->establishesFormattingContext())
+            break;
+        ancestor = &ancestor->parent();
+    }
+    return ancestor == &formattingContextRoot;
+}
+
 bool Box::isOverflowVisible() const
 {
     auto isOverflowVisible = m_style.overflowX() == Overflow::Visible || m_style.overflowY() == Overflow::Visible;
@@ -471,14 +526,14 @@ std::optional<LayoutUnit> Box::columnWidth() const
     return rareData().columnWidth;
 }
 
-const Shape* Box::shape() const
+const LayoutShape* Box::shape() const
 {
     if (!hasRareData())
         return nullptr;
     return rareData().shape.get();
 }
 
-void Box::setShape(RefPtr<const Shape> shape)
+void Box::setShape(RefPtr<const LayoutShape> shape)
 {
     ensureRareData().shape = WTFMove(shape);
 }
@@ -507,6 +562,12 @@ const Box::BoxRareData& Box::rareData() const
     return *rareDataMap().get(this);
 }
 
+Box::BoxRareData& Box::rareData()
+{
+    ASSERT(hasRareData());
+    return *rareDataMap().get(this);
+}
+
 Box::BoxRareData& Box::ensureRareData()
 {
     setHasRareData(true);
