@@ -28,10 +28,12 @@
 
 #include "ColorHash.h"
 #include "Cursor.h"
+#include "DocumentView.h"
 #include "ElementIterator.h"
 #include "FloatRoundedRect.h"
 #include "Gradient.h"
 #include "GraphicsContext.h"
+#include "GraphicsLayer.h"
 #include "HitTestResult.h"
 #include "InteractionRegion.h"
 #include "LocalFrameView.h"
@@ -44,10 +46,17 @@
 #include "RemoteFrame.h"
 #include "RenderLayer.h"
 #include "RenderLayerBacking.h"
+#include "RenderObjectInlines.h"
 #include "RenderView.h"
 #include "ScrollingCoordinator.h"
 #include "Settings.h"
+
+#if PLATFORM(COCOA)
+#include <pal/cocoa/EnhancedSecurityCocoa.h>
+#endif
+
 #include <wtf/SortedArrayMap.h>
+#include <wtf/text/MakeString.h>
 
 namespace WebCore {
 
@@ -81,7 +90,7 @@ protected:
     virtual bool updateRegion() = 0;
     void drawRegion(GraphicsContext&, const Region&, const Color&, const IntRect& dirtyRect);
     
-    SingleThreadWeakPtr<Page> m_page;
+    WeakPtr<Page> m_page;
     RefPtr<PageOverlay> m_overlay;
     std::unique_ptr<Region> m_region;
     Color m_color;
@@ -118,8 +127,8 @@ bool MouseWheelRegionOverlay::updateRegion()
     return false;
 #else
     auto region = makeUnique<Region>();
-    
-    for (RefPtr frame = &page->mainFrame(); frame; frame = frame->tree().traverseNext()) {
+
+    for (RefPtr frame = page->mainFrame(); frame; frame = frame->tree().traverseNext()) {
         RefPtr localFrame = dynamicDowncast<LocalFrame>(frame);
         if (!localFrame)
             continue;
@@ -127,11 +136,11 @@ bool MouseWheelRegionOverlay::updateRegion()
             continue;
 
         Ref document = *localFrame->document();
-        auto frameRegion = document->absoluteRegionForEventTargets(document->wheelEventTargets());
+        auto frameRegion = document->absoluteRegionForWheelEventTargets();
         frameRegion.first.translate(toIntSize(localFrame->protectedView()->contentsToRootView(IntPoint())));
         region->unite(frameRegion.first);
     }
-    
+
     region->translate(m_overlay->viewToOverlayOffset());
 
     bool regionChanged = !m_region || !(*m_region == *region);
@@ -170,7 +179,7 @@ bool NonFastScrollableRegionOverlay::updateRegion()
         return false;
     bool regionChanged = false;
 
-    if (RefPtr scrollingCoordinator = m_page->scrollingCoordinator()) {
+    if (RefPtr scrollingCoordinator = page->scrollingCoordinator()) {
         EventTrackingRegions eventTrackingRegions = scrollingCoordinator->absoluteEventTrackingRegions();
 
         if (eventTrackingRegions != m_eventTrackingRegions) {
@@ -304,7 +313,7 @@ private:
 
 bool InteractionRegionOverlay::updateRegion()
 {
-    m_overlay->setNeedsDisplay();
+    protectedOverlay()->setNeedsDisplay();
     return true;
 }
 
@@ -328,17 +337,17 @@ std::optional<std::pair<RenderLayer&, GraphicsLayer&>> InteractionRegionOverlay:
         HitTestRequest::Type::AllowChildFrameContent
     };
     HitTestResult result(m_mouseLocationInContentCoordinates);
-    RefPtr localMainFrame = dynamicDowncast<LocalFrame>(page->mainFrame());
-    if (!localMainFrame)
+    RefPtr localTopDocument = page->localTopDocument();
+    if (!localTopDocument)
         return std::nullopt;
 
-    localMainFrame->document()->hitTest(hitType, result);
+    localTopDocument->hitTest(hitType, result);
 
     RefPtr hitNode = result.innerNode();
     if (!hitNode || !hitNode->renderer())
         return std::nullopt;
 
-    CheckedPtr rendererLayer = hitNode->renderer()->enclosingLayer();
+    CheckedPtr rendererLayer = hitNode->checkedRenderer()->enclosingLayer();
     if (!rendererLayer)
         return std::nullopt;
 
@@ -372,7 +381,7 @@ std::optional<InteractionRegion> InteractionRegionOverlay::activeRegion() const
     IntRect hitRectInOverlayCoordinates;
     float hitRegionArea = 0;
 
-    auto* localMainFrame = dynamicDowncast<LocalFrame>(page->mainFrame());
+    RefPtr localMainFrame = page->localMainFrame();
     if (!localMainFrame)
         return std::nullopt;
 
@@ -455,7 +464,7 @@ static void drawCheckbox(const String& text, GraphicsContext& context, const Fon
 
 FloatRect InteractionRegionOverlay::rectForSettingAtIndex(unsigned index) const
 {
-    RefPtr mainFrameView = m_page->mainFrame().virtualView();
+    RefPtr mainFrameView = RefPtr { m_page.get() }->protectedMainFrame()->virtualView();
     if (!mainFrameView)
         return FloatRect();
     auto viewSize = mainFrameView->layoutSize();
@@ -487,7 +496,7 @@ void InteractionRegionOverlay::drawSettings(GraphicsContext& context)
     for (unsigned i = 1; i < m_settings.size(); i++)
         rect.unite(rectForSettingAtIndex(i));
 
-    rect.expand(FloatBoxExtent { 4, 4, 4, 4 });
+    rect.expand(FloatBoxExtent { 4.0f, 4.0f, 4.0f, 4.0f });
 
     {
         GraphicsContextStateSaver stateSaver(context);
@@ -548,8 +557,32 @@ void InteractionRegionOverlay::drawRect(PageOverlay&, GraphicsContext& context,
         bool shouldClip = valueForSetting("clip"_s);
         Vector<Path> clipPaths;
 
-        if (shouldClip)
-            clipPaths = pathsForRect(region->rectInLayerCoordinates, region->cornerRadius);
+        if (shouldClip) {
+            const auto rectInLayerCoordinates = region->rectInLayerCoordinates;
+
+            if (auto clipPath = region->clipPath) {
+                Path existingClip = *clipPath;
+                AffineTransform transform;
+
+                transform.translate(rectInLayerCoordinates.location());
+                if (RefPtr page = m_page.get())
+                    transform.scale(page->pageScaleFactor());
+
+                existingClip.transform(transform);
+                clipPaths.append(existingClip);
+            } else {
+                auto scaleFactor = 1.f;
+                if (RefPtr page = m_page.get())
+                    scaleFactor = page->pageScaleFactor();
+
+                if (region->useContinuousCorners) {
+                    Path path;
+                    path.addContinuousRoundedRect(rectInLayerCoordinates, region->cornerRadius * scaleFactor);
+                    clipPaths.append(path);
+                } else
+                    clipPaths = pathsForRect(rectInLayerCoordinates, region->cornerRadius * scaleFactor);
+            }
+        }
 
         bool shouldUseBackdropGradient = !shouldClip || !region || (!valueForSetting("wash"_s) && valueForSetting("clip"_s));
 
@@ -610,7 +643,7 @@ bool InteractionRegionOverlay::mouseEvent(PageOverlay& overlay, const PlatformMo
     RefPtr page = m_page.get();
     if (!page)
         return false;
-    RefPtr localMainFrame = dynamicDowncast<LocalFrame>(page->mainFrame());
+    RefPtr localMainFrame = page->localMainFrame();
     if (!localMainFrame)
         return false;
     RefPtr mainFrameView = localMainFrame->view();
@@ -622,7 +655,7 @@ bool InteractionRegionOverlay::mouseEvent(PageOverlay& overlay, const PlatformMo
     else if (!valueForSetting("hover"_s))
         cursorToSet = pointerCursor();
 
-    auto eventInContentsCoordinates = mainFrameView->windowToContents(event.position());
+    auto eventInContentsCoordinates = mainFrameView->windowToContents(flooredIntPoint(event.position()));
     for (unsigned i = 0; i < m_settings.size(); i++) {
         if (!rectForSettingAtIndex(i).contains(eventInContentsCoordinates))
             continue;
@@ -647,64 +680,47 @@ bool InteractionRegionOverlay::mouseEvent(PageOverlay& overlay, const PlatformMo
 }
 
 #if COMPILER(CLANG)
-#pragma mark - SiteIsolationOverlay
+#pragma mark - EnhancedSecurityOverlay
 #endif
 
-class SiteIsolationOverlay final : public RegionOverlay {
+class EnhancedSecurityOverlay final : public RegionOverlay {
 public:
-    static Ref<SiteIsolationOverlay> create(Page& page)
+    static Ref<EnhancedSecurityOverlay> create(Page& page)
     {
-        return adoptRef(*new SiteIsolationOverlay(page));
+        return adoptRef(*new EnhancedSecurityOverlay(page));
     }
 
 private:
-    explicit SiteIsolationOverlay(Page& page)
-        : RegionOverlay(page, Color::green.colorWithAlphaByte(102))
+    explicit EnhancedSecurityOverlay(Page& page)
+        : RegionOverlay(page, Color::transparentBlack)
     {
     }
 
-    bool updateRegion() final;
+    bool updateRegion() final
+    {
+        protectedOverlay()->setNeedsDisplay();
+        return true;
+    }
     void drawRect(PageOverlay&, GraphicsContext&, const IntRect& dirtyRect) final;
-
-    bool mouseEvent(PageOverlay&, const PlatformMouseEvent&) final;
 };
 
-bool SiteIsolationOverlay::updateRegion()
-{
-    m_overlay->setNeedsDisplay();
-    return true;
-}
-
-void SiteIsolationOverlay::drawRect(PageOverlay&, GraphicsContext& context, const IntRect&)
+void EnhancedSecurityOverlay::drawRect(PageOverlay&, GraphicsContext& context, const IntRect& dirtyRect)
 {
     RefPtr page = m_page.get();
     if (!page)
         return;
-    GraphicsContextStateSaver stateSaver(context);
 
-    FontCascadeDescription fontDescription;
-    fontDescription.setOneFamily("Helvetica"_s);
-    fontDescription.setSpecifiedSize(12);
-    fontDescription.setComputedSize(12);
-    fontDescription.setWeight(FontSelectionValue(500));
-    FontCascade font(WTFMove(fontDescription));
-    font.update(nullptr);
+#if PLATFORM(COCOA)
+    bool isEnhancedSecurityEnabled = PAL::isEnhancedSecurityEnabledForCurrentProcess();
+#else
+    bool isEnhancedSecurityEnabled = false;
+#endif
 
-    for (RefPtr frame = &page->mainFrame(); frame; frame = frame->tree().traverseNext()) {
-        if (!frame->virtualView())
-            continue;
-        auto frameView = frame->virtualView();
-        auto debugStr = makeString(is<RemoteFrame>(frame) ? "remote("_s : "local("_s, frame->frameID().toString(), ')');
-        TextRun textRun = TextRun(debugStr);
-        context.setFillColor(Color::black);
+    Color overlayColor = isEnhancedSecurityEnabled
+        ? Color::green.colorWithAlphaByte(32) // Green overlay when enabled
+        : Color::red.colorWithAlphaByte(32); // Red overlay when disabled
 
-        context.drawText(font, textRun, FloatPoint { static_cast<float>(frameView->x()), static_cast<float>(frameView->y() + 12) });
-    }
-}
-
-bool SiteIsolationOverlay::mouseEvent(PageOverlay& , const PlatformMouseEvent&)
-{
-    return false;
+    context.fillRect(dirtyRect, overlayColor);
 }
 
 #if COMPILER(CLANG)
@@ -720,8 +736,8 @@ Ref<RegionOverlay> RegionOverlay::create(Page& page, DebugPageOverlays::RegionTy
         return NonFastScrollableRegionOverlay::create(page);
     case DebugPageOverlays::RegionType::InteractionRegion:
         return InteractionRegionOverlay::create(page);
-    case DebugPageOverlays::RegionType::SiteIsolationRegion:
-        return SiteIsolationOverlay::create(page);
+    case DebugPageOverlays::RegionType::EnhancedSecurity:
+        return EnhancedSecurityOverlay::create(page);
     }
     ASSERT_NOT_REACHED();
     return MouseWheelRegionOverlay::create(page);
@@ -812,7 +828,7 @@ static inline size_t indexOf(DebugPageOverlays::RegionType regionType)
     return static_cast<size_t>(regionType);
 }
 
-RegionOverlay& DebugPageOverlays::ensureRegionOverlayForPage(Page& page, RegionType regionType)
+Ref<RegionOverlay> DebugPageOverlays::ensureRegionOverlayForPage(Page& page, RegionType regionType)
 {
     auto it = m_pageRegionOverlays.find(page);
     if (it != m_pageRegionOverlays.end()) {
@@ -823,7 +839,7 @@ RegionOverlay& DebugPageOverlays::ensureRegionOverlayForPage(Page& page, RegionT
     }
 
     Vector<RefPtr<RegionOverlay>> visualizers(NumberOfRegionTypes);
-    auto visualizer = RegionOverlay::create(page, regionType);
+    Ref visualizer = RegionOverlay::create(page, regionType);
     visualizers[indexOf(regionType)] = visualizer.copyRef();
     m_pageRegionOverlays.add(page, WTFMove(visualizers));
     return visualizer;
@@ -831,8 +847,8 @@ RegionOverlay& DebugPageOverlays::ensureRegionOverlayForPage(Page& page, RegionT
 
 void DebugPageOverlays::showRegionOverlay(Page& page, RegionType regionType)
 {
-    auto& visualizer = ensureRegionOverlayForPage(page, regionType);
-    page.pageOverlayController().installPageOverlay(visualizer.overlay(), PageOverlay::FadeMode::DoNotFade);
+    Ref visualizer = ensureRegionOverlayForPage(page, regionType);
+    page.pageOverlayController().installPageOverlay(visualizer->protectedOverlay(), PageOverlay::FadeMode::DoNotFade);
 }
 
 void DebugPageOverlays::hideRegionOverlay(Page& page, RegionType regionType)
@@ -843,7 +859,7 @@ void DebugPageOverlays::hideRegionOverlay(Page& page, RegionType regionType)
     auto& visualizer = it->value[indexOf(regionType)];
     if (!visualizer)
         return;
-    page.pageOverlayController().uninstallPageOverlay(visualizer->overlay(), PageOverlay::FadeMode::DoNotFade);
+    page.pageOverlayController().uninstallPageOverlay(visualizer->protectedOverlay(), PageOverlay::FadeMode::DoNotFade);
     visualizer = nullptr;
 }
 
@@ -887,16 +903,16 @@ void DebugPageOverlays::updateOverlayRegionVisibility(Page& page, OptionSet<Debu
         showRegionOverlay(page, RegionType::WheelEventHandlers);
     else
         hideRegionOverlay(page, RegionType::WheelEventHandlers);
-    
+
     if (visibleRegions.contains(DebugOverlayRegions::InteractionRegion))
         showRegionOverlay(page, RegionType::InteractionRegion);
     else
         hideRegionOverlay(page, RegionType::InteractionRegion);
 
-    if (visibleRegions.contains(DebugOverlayRegions::SiteIsolationRegion))
-        showRegionOverlay(page, RegionType::SiteIsolationRegion);
+    if (visibleRegions.contains(DebugOverlayRegions::EnhancedSecurity))
+        showRegionOverlay(page, RegionType::EnhancedSecurity);
     else
-        hideRegionOverlay(page, RegionType::SiteIsolationRegion);
+        hideRegionOverlay(page, RegionType::EnhancedSecurity);
 }
 
 void DebugPageOverlays::settingsChanged(Page& page)
