@@ -25,9 +25,9 @@
 
 #pragma once
 
-#include "AudioTrackPrivateClient.h"
-#include "PlatformAudioTrackConfiguration.h"
-#include "TrackPrivateBase.h"
+#include <WebCore/AudioTrackPrivateClient.h>
+#include <WebCore/PlatformAudioTrackConfiguration.h>
+#include <WebCore/TrackPrivateBase.h>
 #include <wtf/Function.h>
 #include <wtf/TZoneMallocInlines.h>
 
@@ -35,7 +35,7 @@
 
 namespace WebCore {
 
-struct AudioInfo;
+class AudioInfo;
 
 class AudioTrackPrivate : public TrackPrivateBase {
     WTF_MAKE_TZONE_ALLOCATED_INLINE(AudioTrackPrivate);
