@@ -37,10 +37,13 @@
 #include <wtf/MainThread.h>
 #include <wtf/MainThreadData.h>
 #include <wtf/RefPtr.h>
+#include <wtf/Scope.h>
+#include <wtf/glib/GMallocString.h>
 #include <wtf/glib/WTFGType.h>
 #include <wtf/text/AtomString.h>
 #include <wtf/text/AtomStringHash.h>
 #include <wtf/text/CString.h>
+#include <wtf/text/MakeString.h>
 
 using namespace WTF;
 using namespace WebCore;
@@ -48,29 +51,29 @@ using namespace WebCore;
 GST_DEBUG_CATEGORY_STATIC(webkit_media_src_debug);
 #define GST_CAT_DEFAULT webkit_media_src_debug
 
-#define webkit_media_src_parent_class parent_class
-#define WEBKIT_MEDIA_SRC_CATEGORY_INIT GST_DEBUG_CATEGORY_INIT(webkit_media_src_debug, "webkitmediasrc", 0, "websrc element");
-
-static GstStaticPadTemplate srcTemplate = GST_STATIC_PAD_TEMPLATE("src_%s", GST_PAD_SRC,
+static GstStaticPadTemplate mseSrcTemplate = GST_STATIC_PAD_TEMPLATE("src_%s", GST_PAD_SRC,
     GST_PAD_SOMETIMES, GST_STATIC_CAPS_ANY);
 
 enum {
-    PROP_0,
-    PROP_N_AUDIO,
-    PROP_N_VIDEO,
-    PROP_N_TEXT,
-    PROP_LAST
+    WEBKIT_MEDIA_SRC_PROP_0,
+    WEBKIT_MEDIA_SRC_PROP_N_AUDIO,
+    WEBKIT_MEDIA_SRC_PROP_N_VIDEO,
+    WEBKIT_MEDIA_SRC_PROP_N_TEXT,
+    WEBKIT_MEDIA_SRC_PROP_LAST
 };
 
 struct Stream;
 
 struct WebKitMediaSrcPrivate {
-    HashMap<AtomString, RefPtr<Stream>> streams;
-    Stream* streamByName(const AtomString& name)
+    TrackIDHashMap<RefPtr<Stream>> streams;
+    Stream* streamById(TrackID id)
     {
         ASSERT(isMainThread());
-        Stream* stream = streams.get(name);
-        ASSERT(stream);
+        Stream* stream = streams.get(id);
+        if (!stream) {
+            GST_INFO_OBJECT(player.get() ? player.get()->webKitMediaSrc() : nullptr, "Track %" PRIu64
+                " not present in this WebKitMediaSrc", id);
+        }
         return stream;
     }
 
@@ -86,7 +89,7 @@ struct WebKitMediaSrcPrivate {
     double rate { 1.0 };
 
     // Only used by URI Handler API implementation.
-    GUniquePtr<char> uri;
+    GMallocString uri;
 
     ThreadSafeWeakPtr<MediaPlayerPrivateGStreamerMSE> player;
 };
@@ -96,14 +99,12 @@ static void webKitMediaSrcConstructed(GObject*);
 static GstStateChangeReturn webKitMediaSrcChangeState(GstElement*, GstStateChange);
 static gboolean webKitMediaSrcActivateMode(GstPad*, GstObject*, GstPadMode, gboolean activate);
 static void webKitMediaSrcLoop(void*);
-static void webKitMediaSrcTearDownStream(WebKitMediaSrc* source, const AtomString& name);
+static void webKitMediaSrcTearDownStream(WebKitMediaSrc* source, TrackID);
 static void webKitMediaSrcGetProperty(GObject*, unsigned propId, GValue*, GParamSpec*);
 static void webKitMediaSrcStreamFlush(Stream*, bool isSeekingFlush);
 static gboolean webKitMediaSrcSendEvent(GstElement*, GstEvent*);
 static RefPtr<MediaPlayerPrivateGStreamerMSE> webKitMediaSrcPlayer(WebKitMediaSrc*);
 
-#define webkit_media_src_parent_class parent_class
-
 struct WebKitMediaSrcPadPrivate {
     ThreadSafeWeakPtr<Stream> stream;
 };
@@ -119,25 +120,8 @@ struct WebKitMediaSrcPadClass {
 
 namespace WTF {
 
-template<> GRefPtr<WebKitMediaSrcPad> adoptGRef(WebKitMediaSrcPad* ptr)
-{
-    ASSERT(!ptr || !g_object_is_floating(ptr));
-    return GRefPtr<WebKitMediaSrcPad>(ptr, GRefPtrAdopt);
-}
-
-template<> WebKitMediaSrcPad* refGPtr<WebKitMediaSrcPad>(WebKitMediaSrcPad* ptr)
-{
-    if (ptr)
-        gst_object_ref_sink(GST_OBJECT(ptr));
-
-    return ptr;
-}
-
-template<> void derefGPtr<WebKitMediaSrcPad>(WebKitMediaSrcPad* ptr)
-{
-    if (ptr)
-        gst_object_unref(ptr);
-}
+WTF_DEFINE_GREF_TRAITS(WebKitMediaSrc, gst_object_ref_sink, gst_object_unref, g_object_is_floating)
+WTF_DEFINE_GREF_TRAITS_INLINE(WebKitMediaSrcPad, gst_object_ref_sink, gst_object_unref, g_object_is_floating)
 
 } // namespace WTF
 
@@ -152,7 +136,7 @@ static void webkit_media_src_pad_class_init(WebKitMediaSrcPadClass*)
 
 WEBKIT_DEFINE_TYPE_WITH_CODE(WebKitMediaSrc, webkit_media_src, GST_TYPE_ELEMENT,
     G_IMPLEMENT_INTERFACE(GST_TYPE_URI_HANDLER, webKitMediaSrcUriHandlerInit);
-    GST_DEBUG_CATEGORY_INIT(webkit_media_src_debug, "webkitmediasrc", 0, "WebKit MSE source element"));
+    GST_DEBUG_CATEGORY_INIT(webkit_media_src_debug, "webkitmsesrc", 0, "WebKit MSE source element"));
 
 struct Stream : public ThreadSafeRefCountedAndCanMakeThreadSafeWeakPtr<Stream> {
     Stream(WebKitMediaSrc* source, GRefPtr<GstPad>&& pad, Ref<MediaSourceTrackGStreamer>&& track, GRefPtr<GstStream>&& streamInfo)
@@ -210,14 +194,11 @@ static GRefPtr<GstElement> findPipeline(GRefPtr<GstElement> element)
 }
 #endif // GST_DISABLE_GST_DEBUG
 
-static void dumpPipeline(ASCIILiteral description, const RefPtr<Stream>& stream)
+static void dumpPipeline([[maybe_unused]] ASCIILiteral description, [[maybe_unused]] const RefPtr<Stream>& stream)
 {
-#ifdef GST_DISABLE_GST_DEBUG
-    [[maybe_unused]] description;
-    [[maybe_unused]] stream;
-#else
+#ifndef GST_DISABLE_GST_DEBUG
     auto pipeline = findPipeline(GRefPtr<GstElement>(GST_ELEMENT(stream->source)));
-    auto fileName = makeString(span(GST_OBJECT_NAME(pipeline.get())), '-', stream->track->stringId(), '-', description);
+    auto fileName = makeString(unsafeSpan(GST_OBJECT_NAME(pipeline.get())), '-', stream->track->id(), '-', description);
     GST_DEBUG_BIN_TO_DOT_FILE_WITH_TS(GST_BIN_CAST(pipeline.get()), GST_DEBUG_GRAPH_SHOW_ALL, fileName.utf8().data());
 #endif
 }
@@ -238,25 +219,32 @@ static GstStreamType gstStreamType(TrackPrivateBaseGStreamer::TrackType type)
 }
 
 #ifndef GST_DISABLE_GST_DEBUG
-static const char* streamTypeToString(TrackPrivateBaseGStreamer::TrackType type)
+static ASCIILiteral streamTypeToString(TrackPrivateBaseGStreamer::TrackType type)
 {
     switch (type) {
     case TrackPrivateBaseGStreamer::TrackType::Audio:
-        return "Audio";
+        return "Audio"_s;
     case TrackPrivateBaseGStreamer::TrackType::Video:
-        return "Video";
+        return "Video"_s;
     case TrackPrivateBaseGStreamer::TrackType::Text:
-        return "Text";
+        return "Text"_s;
     default:
     case TrackPrivateBaseGStreamer::TrackType::Unknown:
-        return "Unknown";
+        return "Unknown"_s;
     }
 }
 #endif // GST_DISABLE_GST_DEBUG
 
 static gboolean webKitMediaSrcQuery(GstElement* element, GstQuery* query)
 {
-    gboolean result = GST_ELEMENT_CLASS(parent_class)->query(element, query);
+#if GST_CHECK_VERSION(1, 22, 0)
+    if (GST_QUERY_TYPE(query) == GST_QUERY_SELECTABLE) {
+        gst_query_set_selectable(query, TRUE);
+        return TRUE;
+    }
+#endif
+
+    gboolean result = GST_ELEMENT_CLASS(webkit_media_src_parent_class)->query(element, query);
 
     if (GST_QUERY_TYPE(query) != GST_QUERY_SCHEDULING)
         return result;
@@ -277,7 +265,7 @@ static void webkit_media_src_class_init(WebKitMediaSrcClass* klass)
     oklass->constructed = webKitMediaSrcConstructed;
     oklass->get_property = webKitMediaSrcGetProperty;
 
-    gst_element_class_add_static_pad_template_with_gtype(eklass, &srcTemplate, webkit_media_src_pad_get_type());
+    gst_element_class_add_static_pad_template_with_gtype(eklass, &mseSrcTemplate, webkit_media_src_pad_get_type());
 
     gst_element_class_set_static_metadata(eklass, "WebKit MediaSource source element", "Source/Network", "Feeds samples coming from WebKit MediaSource object", "Igalia <aboya@igalia.com>");
 
@@ -286,29 +274,31 @@ static void webkit_media_src_class_init(WebKitMediaSrcClass* klass)
 
     // In GStreamer 1.20 and older urisourcebin mishandles source elements with dynamic pads. This
     // is not an issue in 1.22.
-    if (webkitGstCheckVersion(1, 22, 0))
+    if (gst_check_version(1, 22, 0))
         eklass->query = GST_DEBUG_FUNCPTR(webKitMediaSrcQuery);
 
     g_object_class_install_property(oklass,
-        PROP_N_AUDIO,
+        WEBKIT_MEDIA_SRC_PROP_N_AUDIO,
         g_param_spec_int("n-audio", nullptr, nullptr,
-        0, G_MAXINT, 0, GParamFlags(G_PARAM_READABLE | G_PARAM_STATIC_STRINGS)));
+            0, G_MAXINT, 0, GParamFlags(G_PARAM_READABLE | G_PARAM_STATIC_STRINGS)));
     g_object_class_install_property(oklass,
-        PROP_N_VIDEO,
+        WEBKIT_MEDIA_SRC_PROP_N_VIDEO,
         g_param_spec_int("n-video", nullptr, nullptr,
-        0, G_MAXINT, 0, GParamFlags(G_PARAM_READABLE | G_PARAM_STATIC_STRINGS)));
+            0, G_MAXINT, 0, GParamFlags(G_PARAM_READABLE | G_PARAM_STATIC_STRINGS)));
     g_object_class_install_property(oklass,
-        PROP_N_TEXT,
+        WEBKIT_MEDIA_SRC_PROP_N_TEXT,
         g_param_spec_int("n-text", nullptr, nullptr,
-        0, G_MAXINT, 0, GParamFlags(G_PARAM_READABLE | G_PARAM_STATIC_STRINGS)));
+            0, G_MAXINT, 0, GParamFlags(G_PARAM_READABLE | G_PARAM_STATIC_STRINGS)));
 }
 
 static void webKitMediaSrcConstructed(GObject* object)
 {
-    GST_CALL_PARENT(G_OBJECT_CLASS, constructed, (object));
+    G_OBJECT_CLASS(webkit_media_src_parent_class)->constructed(object);
 
     ASSERT(isMainThread());
+IGNORE_WARNINGS_BEGIN("cast-align")
     GST_OBJECT_FLAG_SET(object, GST_ELEMENT_FLAG_SOURCE);
+IGNORE_WARNINGS_END
 }
 
 void webKitMediaSrcEmitStreams(WebKitMediaSrc* source, const Vector<RefPtr<MediaSourceTrackGStreamer>>& tracks)
@@ -320,30 +310,38 @@ void webKitMediaSrcEmitStreams(WebKitMediaSrc* source, const Vector<RefPtr<Media
     source->priv->collection = adoptGRef(gst_stream_collection_new("WebKitMediaSrc"));
     for (const auto& track : tracks) {
 #ifndef GST_DISABLE_GST_DEBUG
-        GST_DEBUG_OBJECT(source, "Adding stream with trackId '%s' of type %s with caps %" GST_PTR_FORMAT, track->stringId().string().utf8().data(), streamTypeToString(track->type()), track->initialCaps().get());
+        GST_DEBUG_OBJECT(source, "Adding stream with trackId '%" PRIu64 "' of type %s with caps %" GST_PTR_FORMAT,
+            track->id(), streamTypeToString(track->type()).characters(), track->initialCaps().get());
 #endif // GST_DISABLE_GST_DEBUG
-        if (source->priv->streams.contains(track->stringId())) {
-            GST_ERROR_OBJECT(source, "stream with trackId '%s' already exists", track->stringId().string().utf8().data());
+        if (source->priv->streams.contains(track->id())) {
+            GST_ERROR_OBJECT(source, "stream with trackId '%" PRIu64 "' already exists", track->id());
             ASSERT_NOT_REACHED();
             continue;
         }
 
-        GRefPtr<WebKitMediaSrcPad> pad = WEBKIT_MEDIA_SRC_PAD(g_object_new(webkit_media_src_pad_get_type(), "name", makeString("src_"_s, track->stringId()).utf8().data(), "direction", GST_PAD_SRC, NULL));
+        GRefPtr<WebKitMediaSrcPad> pad = WEBKIT_MEDIA_SRC_PAD(g_object_new(webkit_media_src_pad_get_type(), "name", makeString("src_"_s, String::number(track->id())).utf8().data(), "direction", GST_PAD_SRC, NULL));
         gst_pad_set_activatemode_function(GST_PAD(pad.get()), webKitMediaSrcActivateMode);
 
         ASSERT(track->initialCaps());
         auto stream = adoptRef(new Stream(source, GRefPtr<GstPad>(GST_PAD(pad.get())), *track,
-            adoptGRef(gst_stream_new(track->stringId().string().utf8().data(), track->initialCaps().get(), gstStreamType(track->type()), GST_STREAM_FLAG_SELECT))));
+            adoptGRef(gst_stream_new(makeString(track->id()).utf8().data(), track->initialCaps().get(), gstStreamType(track->type()), GST_STREAM_FLAG_SELECT))));
         pad->priv->stream = ThreadSafeWeakPtr { *stream.get() };
 
         gst_stream_collection_add_stream(source->priv->collection.get(), GRefPtr<GstStream>(stream->streamInfo.get()).leakRef());
-        source->priv->streams.set(track->stringId(), WTFMove(stream));
+        source->priv->streams.set(track->id(), WTFMove(stream));
     }
 
     gst_element_post_message(GST_ELEMENT(source), gst_message_new_stream_collection(GST_OBJECT(source), source->priv->collection.get()));
 
     for (const RefPtr<Stream>& stream: source->priv->streams.values()) {
-        if (!webkitGstCheckVersion(1, 20, 6)) {
+        // Block data flow until pad is exposed.
+        gulong blockId = gst_pad_add_probe(
+            GST_PAD(stream->pad.get()), GST_PAD_PROBE_TYPE_BLOCK_DOWNSTREAM,
+            [](GstPad*, GstPadProbeInfo*, gpointer) -> GstPadProbeReturn {
+                return GST_PAD_PROBE_OK;
+            }, nullptr, nullptr);
+
+        if (!gst_check_version(1, 20, 6)) {
             // Workaround: gst_element_add_pad() should already call gst_pad_set_active() if the element is PAUSED or
             // PLAYING. Unfortunately, as of GStreamer 1.18.2 it does so with the element lock taken, causing a deadlock
             // in gst_pad_start_task(), who tries to post a `stream-status` message in the element, which also requires
@@ -354,8 +352,9 @@ void webKitMediaSrcEmitStreams(WebKitMediaSrc* source, const Vector<RefPtr<Media
             if (state > GST_STATE_READY)
                 gst_pad_set_active(GST_PAD(stream->pad.get()), true);
         }
-        GST_DEBUG_OBJECT(source, "Adding pad '%s' for stream with name '%s'", GST_OBJECT_NAME(stream->pad.get()), stream->track->stringId().string().utf8().data());
+        GST_DEBUG_OBJECT(source, "Adding pad '%s' for stream with id '%" PRIu64 "'", GST_OBJECT_NAME(stream->pad.get()), stream->track->id());
         gst_element_add_pad(GST_ELEMENT(source), GST_PAD(stream->pad.get()));
+        gst_pad_remove_probe(GST_PAD(stream->pad.get()), blockId);
     }
     GST_DEBUG_OBJECT(source, "All pads added");
 }
@@ -370,11 +369,16 @@ void webKitMediaSrcSetPlayer(WebKitMediaSrc* source, ThreadSafeWeakPtr<MediaPlay
     source->priv->player = WTFMove(player);
 }
 
-static void webKitMediaSrcTearDownStream(WebKitMediaSrc* source, const AtomString& name)
+static void webKitMediaSrcTearDownStream(WebKitMediaSrc* source, TrackID id)
 {
     ASSERT(isMainThread());
-    Stream* stream = source->priv->streamByName(name);
-    GST_DEBUG_OBJECT(source, "Tearing down stream '%s'", name.string().utf8().data());
+    Stream* stream = source->priv->streamById(id);
+    GST_DEBUG_OBJECT(source, "Tearing down stream '%" PRIu64 "'", id);
+
+    if (!stream) {
+        GST_INFO_OBJECT(source, "Ignoring teardown on track not present in this WebKitMediaSrc");
+        return;
+    }
 
     // Flush the source element **and** downstream. We want to stop the streaming thread and for that we need all elements downstream to be idle.
     webKitMediaSrcStreamFlush(stream, false);
@@ -386,10 +390,10 @@ static void webKitMediaSrcTearDownStream(WebKitMediaSrc* source, const AtomStrin
         gst_element_remove_pad(GST_ELEMENT(source), GST_PAD(pad));
         pad->priv->stream = nullptr;
     }
-    source->priv->streams.remove(name);
+    source->priv->streams.remove(id);
 }
 
-static gboolean webKitMediaSrcActivateMode(GstPad* pad, GstObject* source, GstPadMode mode, gboolean active)
+static gboolean webKitMediaSrcActivateMode(GstPad* pad, [[maybe_unused]] GstObject* source, GstPadMode mode, gboolean active)
 {
     if (mode != GST_PAD_MODE_PUSH) {
         GST_ERROR_OBJECT(source, "Unexpected pad mode in WebKitMediaSrc");
@@ -436,7 +440,7 @@ static void webKitMediaSrcWaitForPadLinkedOrFlush(GstPad* pad, DataMutexLocker<S
 {
     {
         auto locker = GstObjectLocker(pad);
-        if (LIKELY(GST_PAD_IS_LINKED(pad)))
+        if (GST_PAD_IS_LINKED(pad)) [[likely]]
             return;
 
         GST_DEBUG_OBJECT(pad, "Waiting for the pad to be linked...");
@@ -483,7 +487,7 @@ static void webKitMediaSrcLoop(void* userData)
 
     if (!streamingMembers->hasPushedStreamCollectionEvent) {
         GST_DEBUG_OBJECT(pad, "Pushing STREAM_COLLECTION event.");
-        bool wasStreamCollectionSent = gst_pad_push_event(stream->pad.get(), gst_event_new_stream_collection(stream->source->priv->collection.get()));
+        [[maybe_unused]] bool wasStreamCollectionSent = gst_pad_push_event(stream->pad.get(), gst_event_new_stream_collection(stream->source->priv->collection.get()));
         streamingMembers->hasPushedStreamCollectionEvent = true;
         GST_DEBUG_OBJECT(pad, "STREAM_COLLECTION event has been pushed, %s was returned.", boolForPrinting(wasStreamCollectionSent));
         // Initial events like this must go through, flushes (including tearing down the element) is not allowed until
@@ -492,8 +496,8 @@ static void webKitMediaSrcLoop(void* userData)
     }
 
     if (!streamingMembers->wasStreamStartSent) {
-        GUniquePtr<char> streamId { g_strdup_printf("mse/%s", stream->track->stringId().string().utf8().data()) };
-        GRefPtr<GstEvent> event = adoptGRef(gst_event_new_stream_start(streamId.get()));
+        auto streamId = GMallocString::unsafeAdoptFromUTF8(g_strdup_printf("mse/%" PRIu64 "", stream->track->id()));
+        GRefPtr<GstEvent> event = adoptGRef(gst_event_new_stream_start(streamId.utf8()));
         gst_event_set_group_id(event.get(), stream->source->priv->groupId);
         gst_event_set_stream(event.get(), stream->streamInfo.get());
 
@@ -508,7 +512,7 @@ static void webKitMediaSrcLoop(void* userData)
         GRefPtr<GstEvent> event = adoptGRef(gst_event_new_caps(streamingMembers->pendingInitialCaps.get()));
 
         GST_DEBUG_OBJECT(pad, "Pushing initial CAPS event: %" GST_PTR_FORMAT, streamingMembers->pendingInitialCaps.get());
-        bool wasCapsEventSent = gst_pad_push_event(pad, event.leakRef());
+        [[maybe_unused]] bool wasCapsEventSent = gst_pad_push_event(pad, event.leakRef());
         GST_DEBUG_OBJECT(pad, "Pushed initial CAPS event, %s was returned.", boolForPrinting(wasCapsEventSent));
 
         streamingMembers->previousCaps = WTFMove(streamingMembers->pendingInitialCaps);
@@ -555,7 +559,7 @@ static void webKitMediaSrcLoop(void* userData)
     // omit the flush (see webKitMediaSrcFlush) we actually emit the updated, correct segment.
     if (streamingMembers->doesNeedSegmentEvent) {
         GST_DEBUG_OBJECT(pad, "Need new SEGMENT event, pushing it: %" GST_SEGMENT_FORMAT, &streamingMembers->segment);
-        bool result = gst_pad_push_event(pad, gst_event_new_segment(&streamingMembers->segment));
+        [[maybe_unused]] bool result = gst_pad_push_event(pad, gst_event_new_segment(&streamingMembers->segment));
         GST_DEBUG_OBJECT(pad, "SEGMENT event pushed, result = %s.", boolForPrinting(result));
         ASSERT(result);
         streamingMembers->doesNeedSegmentEvent = false;
@@ -569,8 +573,11 @@ static void webKitMediaSrcLoop(void* userData)
             streamingMembers->previousCaps = gst_sample_get_caps(sample.get());
             // This CAPS event may block, so we release the lock and reevaluate later if there's been a flush in the meantime.
             streamingMembers.runUnlocked([&stream, &sample, &pad]() {
+#ifdef GST_DISABLE_GST_DEBUG
+                UNUSED_VARIABLE(pad);
+#endif
                 GST_DEBUG_OBJECT(pad, "Pushing new CAPS event: %" GST_PTR_FORMAT, gst_sample_get_caps(sample.get()));
-                bool result = gst_pad_push_event(stream->pad.get(), gst_event_new_caps(gst_sample_get_caps(sample.get())));
+                [[maybe_unused]] bool result = gst_pad_push_event(stream->pad.get(), gst_event_new_caps(gst_sample_get_caps(sample.get())));
                 GST_DEBUG_OBJECT(pad, "CAPS event pushed, result = %s.", boolForPrinting(result));
                 ASSERT(result);
             });
@@ -596,16 +603,21 @@ static void webKitMediaSrcLoop(void* userData)
         ASSERT(GST_BUFFER_PTS_IS_VALID(buffer.get()));
         GST_TRACE_OBJECT(pad, "Pushing buffer downstream: %" GST_PTR_FORMAT, buffer.get());
         GstFlowReturn result = gst_pad_push(pad, buffer.leakRef());
-        if (result != GST_FLOW_OK && result != GST_FLOW_FLUSHING) {
+        if (result == GST_FLOW_NOT_LINKED && stream->track->type() == TrackPrivateBaseGStreamer::TrackType::Video) {
+            // We allow multiple video tracks and all of them except one may be unlinked. Just drop the buffer.
+            GST_TRACE_OBJECT(pad, "Buffer not pushed because pad is not-linked, ignoring");
+        } else if (result != GST_FLOW_OK && result != GST_FLOW_FLUSHING) {
             gst_pad_pause_task(pad);
             GST_ELEMENT_ERROR(stream->source, CORE, PAD, ("Failed to push buffer"), ("gst_pad_push() returned %s", gst_flow_get_name(result)));
         } else if (pushingFirstBuffer) {
             GST_DEBUG_OBJECT(pad, "First buffer on this pad was pushed (ret = %s).", gst_flow_get_name(result));
             dumpPipeline("first-frame-after"_s, stream);
         }
+IGNORE_WARNINGS_BEGIN("cast-align")
     } else if (GST_IS_EVENT(object.get())) {
         // EOS events and other enqueued events are also sent unlocked so they can react to flushes if necessary.
         GRefPtr<GstEvent> event = GRefPtr<GstEvent>(GST_EVENT(object.leakRef()));
+IGNORE_WARNINGS_END
 
         streamingMembers.unlockEarly();
         GST_DEBUG_OBJECT(pad, "Pushing event downstream: %" GST_PTR_FORMAT, event.get());
@@ -620,14 +632,14 @@ static void webKitMediaSrcStreamFlush(Stream* stream, bool isSeekingFlush)
 {
     ASSERT(isMainThread());
     bool skipFlush = false;
-    GST_DEBUG_OBJECT(stream->source, "Flush requested for stream '%s'. isSeekingFlush = %s",
-        stream->track->stringId().string().utf8().data(), boolForPrinting(isSeekingFlush));
+    GST_DEBUG_OBJECT(stream->source, "Flush requested for stream '%" PRIu64 "'. isSeekingFlush = %s",
+        stream->track->id(), boolForPrinting(isSeekingFlush));
 
     {
         DataMutexLocker streamingMembers { stream->streamingMembersDataMutex };
 
         if (!streamingMembers->hasPoppedFirstObject) {
-            GST_DEBUG_OBJECT(stream->source, "Flush request for stream '%s' occurred before hasPoppedFirstObject, just clearing the queue and readjusting the segment.", stream->track->stringId().string().utf8().data());
+            GST_DEBUG_OBJECT(stream->source, "Flush request for stream '%" PRIu64 "' occurred before hasPoppedFirstObject, just clearing the queue and readjusting the segment.", stream->track->id());
             DataMutexLocker queue { stream->track->queueDataMutex() };
             // We use clear() instead of flush() because the WebKitMediaSrc streaming thread could be waiting
             // for the queue. flush() would cancel the notEmptyCallback therefore leaving the streaming thread
@@ -682,9 +694,11 @@ static void webKitMediaSrcStreamFlush(Stream* stream, bool isSeekingFlush)
             // We need to increase the base by the running time accumulated during the previous segment.
             GstClockTime pipelineRunningTime = gst_segment_to_running_time(&streamingMembers->segment, GST_FORMAT_TIME, pipelineStreamTime);
             if ((GST_CLOCK_TIME_IS_VALID(pipelineRunningTime))) {
-                GST_DEBUG_OBJECT(stream->source, "Resetting segment to current pipeline running time (%" GST_TIME_FORMAT " and stream time (%" GST_TIME_FORMAT " = %s)",
-                    GST_TIME_ARGS(pipelineRunningTime), GST_TIME_ARGS(pipelineStreamTime), streamTime.toString().ascii().data());
+                GST_DEBUG_OBJECT(stream->source, "Resetting segment to current pipeline running time (%" GST_TIME_FORMAT
+                    " and stream time (%" GST_TIME_FORMAT " = %s), updating rate to %f", GST_TIME_ARGS(pipelineRunningTime),
+                    GST_TIME_ARGS(pipelineStreamTime), streamTime.toString().ascii().data(), stream->source->priv->rate);
                 streamingMembers->segment.base = pipelineRunningTime;
+                streamingMembers->segment.rate = stream->source->priv->rate;
                 streamingMembers->segment.start = streamingMembers->segment.time = static_cast<GstClockTime>(pipelineStreamTime);
             }
         }
@@ -701,7 +715,7 @@ static void webKitMediaSrcStreamFlush(Stream* stream, bool isSeekingFlush)
             streamingMembers->isFlushing = false;
             streamingMembers->doesNeedSegmentEvent = true;
 
-            if (!webkitGstCheckVersion(1, 22, 0)) {
+            if (!gst_check_version(1, 22, 0)) {
                 // In older GST versions STREAM_COLLECTION event is delivered to decodebin3
                 // from parsebin src pad probe. On the way, this event is cached inside
                 // parser element (GstBaseParse) and pushed downstream with first frame.
@@ -730,15 +744,20 @@ static void webKitMediaSrcStreamFlush(Stream* stream, bool isSeekingFlush)
         gst_pad_start_task(stream->pad.get(), webKitMediaSrcLoop, stream->pad.get(), nullptr);
     }
 
-    GST_DEBUG_OBJECT(stream->source, "Flush request for stream '%s' (isSeekingFlush = %s) satisfied.",
-        stream->track->stringId().string().utf8().data(), boolForPrinting(isSeekingFlush));
+    GST_DEBUG_OBJECT(stream->source, "Flush request for stream '%" PRIu64 "' (isSeekingFlush = %s) satisfied.",
+        stream->track->id(), boolForPrinting(isSeekingFlush));
 }
 
-void webKitMediaSrcFlush(WebKitMediaSrc* source, const AtomString& streamName)
+void webKitMediaSrcFlush(WebKitMediaSrc* source, TrackID streamId)
 {
     ASSERT(isMainThread());
-    GST_DEBUG_OBJECT(source, "Received non-seek flush request for stream '%s'.", streamName.string().utf8().data());
-    Stream* stream = source->priv->streamByName(streamName);
+    GST_DEBUG_OBJECT(source, "Received non-seek flush request for stream '%" PRIu64 "'.", streamId);
+    Stream* stream = source->priv->streamById(streamId);
+
+    if (!stream) {
+        GST_INFO_OBJECT(source, "Ignoring flush on track not present in this WebKitMediaSrc");
+        return;
+    }
 
     webKitMediaSrcStreamFlush(stream, false);
 }
@@ -767,13 +786,13 @@ static void webKitMediaSrcGetProperty(GObject* object, unsigned propId, GValue*
     WebKitMediaSrc* source = WEBKIT_MEDIA_SRC(object);
 
     switch (propId) {
-    case PROP_N_AUDIO:
+    case WEBKIT_MEDIA_SRC_PROP_N_AUDIO:
         g_value_set_int(value, countStreamsOfType(source, WebCore::TrackPrivateBaseGStreamer::TrackType::Audio));
         break;
-    case PROP_N_VIDEO:
+    case WEBKIT_MEDIA_SRC_PROP_N_VIDEO:
         g_value_set_int(value, countStreamsOfType(source, WebCore::TrackPrivateBaseGStreamer::TrackType::Video));
         break;
-    case PROP_N_TEXT:
+    case WEBKIT_MEDIA_SRC_PROP_N_TEXT:
         g_value_set_int(value, countStreamsOfType(source, WebCore::TrackPrivateBaseGStreamer::TrackType::Text));
         break;
     default:
@@ -819,6 +838,25 @@ static gboolean webKitMediaSrcSendEvent(GstElement* element, GstEvent* eventTran
         webKitMediaSrcSeek(WEBKIT_MEDIA_SRC(element), start, rate);
         return true;
     }
+    case GST_EVENT_CUSTOM_DOWNSTREAM_OOB: {
+        WebKitMediaSrc* source = WEBKIT_MEDIA_SRC(element);
+        bool forwardToAllPads = GStreamerQuirksManager::singleton().analyzeWebKitMediaSrcCustomEvent(event);
+        bool wasEventHandledByAnyStream = false;
+        bool wasEventHandledByAllStreams = false;
+        if (forwardToAllPads) {
+            wasEventHandledByAllStreams = !source->priv->streams.isEmpty();
+            for (const RefPtr<Stream>& stream : source->priv->streams.values()) {
+                bool wasHandled = gst_pad_push_event(stream->pad.get(), event.ref());
+                wasEventHandledByAllStreams &= wasHandled;
+                wasEventHandledByAnyStream |= wasHandled;
+            }
+        } else
+            wasEventHandledByAnyStream = GST_ELEMENT_CLASS(webkit_media_src_parent_class)->send_event(element, event.ref());
+        auto rate = GStreamerQuirksManager::singleton().processWebKitMediaSrcCustomEvent(event, wasEventHandledByAnyStream, wasEventHandledByAllStreams);
+        if (rate.has_value())
+            source->priv->rate = rate.value();
+        return forwardToAllPads ? wasEventHandledByAllStreams : wasEventHandledByAnyStream;
+    }
     default:
         return GST_ELEMENT_CLASS(webkit_media_src_parent_class)->send_event(element, event.leakRef());
     }
@@ -842,7 +880,7 @@ static gchar* webKitMediaSrcGetUri(GstURIHandler* handler)
     WebKitMediaSrc* source = WEBKIT_MEDIA_SRC(handler);
 
     auto locker = GstObjectLocker(source);
-    return g_strdup(source->priv->uri.get());
+    return g_strdup(source->priv->uri.utf8());
 }
 
 static gboolean webKitMediaSrcSetUri(GstURIHandler* handler, const gchar* uri, GError**)
@@ -855,7 +893,7 @@ static gboolean webKitMediaSrcSetUri(GstURIHandler* handler, const gchar* uri, G
     }
 
     auto locker = GstObjectLocker(source);
-    source->priv->uri = GUniquePtr<char>(g_strdup(uri));
+    source->priv->uri = GMallocString::unsafeAdoptFromUTF8(g_strdup(uri));
     return TRUE;
 }
 
@@ -869,26 +907,4 @@ static void webKitMediaSrcUriHandlerInit(void* gIface, void*)
     iface->set_uri = webKitMediaSrcSetUri;
 }
 
-namespace WTF {
-template <> GRefPtr<WebKitMediaSrc> adoptGRef(WebKitMediaSrc* ptr)
-{
-    ASSERT(!ptr || !g_object_is_floating(G_OBJECT(ptr)));
-    return GRefPtr<WebKitMediaSrc>(ptr, GRefPtrAdopt);
-}
-
-template <> WebKitMediaSrc* refGPtr<WebKitMediaSrc>(WebKitMediaSrc* ptr)
-{
-    if (ptr)
-        gst_object_ref_sink(GST_OBJECT(ptr));
-
-    return ptr;
-}
-
-template <> void derefGPtr<WebKitMediaSrc>(WebKitMediaSrc* ptr)
-{
-    if (ptr)
-        gst_object_unref(ptr);
-}
-} // namespace WTF
-
 #endif // USE(GSTREAMER)
