@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2006 Apple Inc.
+ * Copyright (C) 2006-2023 Apple Inc. All rights reserved.
+ * Copyright (C) 2014 Google Inc.
  * Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies)
  *
  * This library is free software; you can redistribute it and/or
@@ -19,54 +20,69 @@
  *
 */
 
-#ifndef HitTestResult_h
-#define HitTestResult_h
+#pragma once
 
-#include "FloatQuad.h"
-#include "FloatRect.h"
-#include "HitTestLocation.h"
-#include "HitTestRequest.h"
-#include "LayoutRect.h"
-#include "TextFlags.h"
-#include <memory>
+#include <WebCore/DoublePoint.h>
+#include <WebCore/HitTestLocation.h>
+#include <WebCore/HitTestRequest.h>
+#include <WebCore/PseudoElementIdentifier.h>
 #include <wtf/Forward.h>
 #include <wtf/ListHashSet.h>
-#include <wtf/RefPtr.h>
+#include <wtf/TZoneMalloc.h>
 
 namespace WebCore {
 
 class Element;
 class Frame;
-#if ENABLE(VIDEO)
+class HTMLImageElement;
 class HTMLMediaElement;
-#endif
 class Image;
-class URL;
+class LocalFrame;
 class Node;
 class Scrollbar;
 
+enum class HitTestProgress { Stop, Continue };
+
 class HitTestResult {
+    WTF_MAKE_TZONE_ALLOCATED_EXPORT(HitTestResult, WEBCORE_EXPORT);
 public:
-    typedef ListHashSet<RefPtr<Node>> NodeSet;
+    using NodeSet = ListHashSet<Ref<Node>>;
 
     WEBCORE_EXPORT HitTestResult();
+    WEBCORE_EXPORT explicit HitTestResult(const IntPoint&);
     WEBCORE_EXPORT explicit HitTestResult(const LayoutPoint&);
-    // Pass non-negative padding values to perform a rect-based hit test.
-    WEBCORE_EXPORT HitTestResult(const LayoutPoint& centerPoint, unsigned topPadding, unsigned rightPadding, unsigned bottomPadding, unsigned leftPadding);
+    WEBCORE_EXPORT explicit HitTestResult(const DoublePoint&);
+
+    WEBCORE_EXPORT explicit HitTestResult(const LayoutRect&);
     WEBCORE_EXPORT explicit HitTestResult(const HitTestLocation&);
     WEBCORE_EXPORT HitTestResult(const HitTestResult&);
     WEBCORE_EXPORT ~HitTestResult();
+
     WEBCORE_EXPORT HitTestResult& operator=(const HitTestResult&);
 
+    WEBCORE_EXPORT void setInnerNode(Node*);
     Node* innerNode() const { return m_innerNode.get(); }
-    WEBCORE_EXPORT Element* innerElement() const;
-    Node* innerNonSharedNode() const { return m_innerNonSharedNode.get(); }
-    WEBCORE_EXPORT Element* innerNonSharedElement() const;
-    Element* URLElement() const { return m_innerURLElement.get(); }
-    Scrollbar* scrollbar() const { return m_scrollbar.get(); }
-    bool isOverWidget() const { return m_isOverWidget; }
 
-    WEBCORE_EXPORT const AtomicString& URLElementDownloadAttribute() const;
+    void setInnerNonSharedNode(Node*);
+    Node* innerNonSharedNode() const { return m_innerNonSharedNode.get(); }
+    WEBCORE_EXPORT RefPtr<Node> protectedInnerNonSharedNode() const;
+
+    WEBCORE_EXPORT Element* innerNonSharedElement() const;
+
+    void setURLElement(Element*);
+    Element* URLElement() const { return m_innerURLElement.get(); }
+    WEBCORE_EXPORT RefPtr<Element> protectedURLElement() const;
+
+    void setScrollbar(RefPtr<Scrollbar>&&);
+    Scrollbar* scrollbar() const { return m_scrollbar.get(); }
+
+    bool isOverWidget() const { return m_isOverWidget; }
+    void setIsOverWidget(bool isOverWidget) { m_isOverWidget = isOverWidget; }
+
+    std::optional<Style::PseudoElementIdentifier> pseudoElementIdentifier() const;
+    void setPseudoElementIdentifier(std::optional<Style::PseudoElementIdentifier>);
+
+    WEBCORE_EXPORT String linkSuggestedFilename() const;
 
     // Forwarded from HitTestLocation
     bool isRectBasedTest() const { return m_hitTestLocation.isRectBasedTest(); }
@@ -76,55 +92,57 @@ public:
     IntPoint roundedPointInMainFrame() const { return roundedIntPoint(pointInMainFrame()); }
 
     // The hit-tested point in the coordinates of the innerNode frame, the frame containing innerNode.
-    const LayoutPoint& pointInInnerNodeFrame() const { return m_pointInInnerNodeFrame; }
+    const LayoutPoint pointInInnerNodeFrame() const { return LayoutPoint(m_doublePointInInnerNodeFrame); }
+    const DoublePoint& doublePointInInnerNodeFrame() const { return m_doublePointInInnerNodeFrame; }
     IntPoint roundedPointInInnerNodeFrame() const { return roundedIntPoint(pointInInnerNodeFrame()); }
-    WEBCORE_EXPORT Frame* innerNodeFrame() const;
+    WEBCORE_EXPORT LocalFrame* innerNodeFrame() const;
 
     // The hit-tested point in the coordinates of the inner node.
     const LayoutPoint& localPoint() const { return m_localPoint; }
-    void setLocalPoint(const LayoutPoint& p) { m_localPoint = p; }
+    void setLocalPoint(const LayoutPoint&);
 
-    void setToNonUserAgentShadowAncestor();
+    WEBCORE_EXPORT void setToNonUserAgentShadowAncestor();
 
     const HitTestLocation& hitTestLocation() const { return m_hitTestLocation; }
 
-    WEBCORE_EXPORT void setInnerNode(Node*);
-    void setInnerNonSharedNode(Node*);
-    void setURLElement(Element*);
-    void setScrollbar(Scrollbar*);
-    void setIsOverWidget(bool b) { m_isOverWidget = b; }
-
-    WEBCORE_EXPORT Frame* targetFrame() const;
+    WEBCORE_EXPORT LocalFrame* frame() const;
+    WEBCORE_EXPORT RefPtr<Frame> targetFrame() const;
     WEBCORE_EXPORT bool isSelected() const;
+    WEBCORE_EXPORT bool allowsFollowingLink() const;
+    WEBCORE_EXPORT bool allowsFollowingImageURL() const;
     WEBCORE_EXPORT String selectedText() const;
     WEBCORE_EXPORT String spellingToolTip(TextDirection&) const;
     String replacedString() const;
     WEBCORE_EXPORT String title(TextDirection&) const;
-    String innerTextIfTruncated(TextDirection&) const;
+    WEBCORE_EXPORT String innerTextIfTruncated(TextDirection&) const;
     WEBCORE_EXPORT String altDisplayString() const;
     WEBCORE_EXPORT String titleDisplayString() const;
     WEBCORE_EXPORT Image* image() const;
     WEBCORE_EXPORT IntRect imageRect() const;
+    WEBCORE_EXPORT bool hasEntireImage() const;
     WEBCORE_EXPORT URL absoluteImageURL() const;
     WEBCORE_EXPORT URL absolutePDFURL() const;
     WEBCORE_EXPORT URL absoluteMediaURL() const;
     WEBCORE_EXPORT URL absoluteLinkURL() const;
-#if ENABLE(ATTACHMENT_ELEMENT)
-    WEBCORE_EXPORT URL absoluteAttachmentURL() const;
-#endif
+    WEBCORE_EXPORT bool hasLocalDataForLinkURL() const;
     WEBCORE_EXPORT String textContent() const;
     bool isOverLink() const;
     WEBCORE_EXPORT bool isContentEditable() const;
     void toggleMediaControlsDisplay() const;
     void toggleMediaLoopPlayback() const;
+    void toggleShowMediaStats() const;
     WEBCORE_EXPORT bool mediaIsInFullscreen() const;
+    bool mediaIsInVideoViewer() const;
+    void toggleVideoViewer() const;
     void toggleMediaFullscreenState() const;
     void enterFullscreenForVideo() const;
     bool mediaControlsEnabled() const;
     bool mediaLoopEnabled() const;
+    bool mediaStatsShowing() const;
     bool mediaPlaying() const;
     bool mediaSupportsFullscreen() const;
     void toggleMediaPlayState() const;
+    WEBCORE_EXPORT bool hasMediaElement() const;
     WEBCORE_EXPORT bool mediaHasAudio() const;
     WEBCORE_EXPORT bool mediaIsVideo() const;
     bool mediaMuted() const;
@@ -133,27 +151,42 @@ public:
     bool mediaIsInEnhancedFullscreen() const;
     void toggleEnhancedFullscreenForVideo() const;
 
+#if ENABLE(ACCESSIBILITY_ANIMATION_CONTROL)
+    void pauseAnimation() const;
+    void playAnimation() const;
+    bool isAnimating() const;
+#endif
+
     WEBCORE_EXPORT bool isDownloadableMedia() const;
     WEBCORE_EXPORT bool isOverTextInsideFormControlElement() const;
-    WEBCORE_EXPORT bool allowsCopy() const;
 
-    // Returns true if it is rect-based hit test and needs to continue until the rect is fully
-    // enclosed by the boundaries of a node.
-    bool addNodeToRectBasedTestResult(Node*, const HitTestRequest&, const HitTestLocation& pointInContainer, const LayoutRect& = LayoutRect());
-    bool addNodeToRectBasedTestResult(Node*, const HitTestRequest&, const HitTestLocation& pointInContainer, const FloatRect&);
-    void append(const HitTestResult&);
+    HitTestProgress addNodeToListBasedTestResult(Node*, const HitTestRequest&, const HitTestLocation& pointInContainer, const LayoutRect& = LayoutRect());
+    HitTestProgress addNodeToListBasedTestResult(Node*, const HitTestRequest&, const HitTestLocation& pointInContainer, const FloatRect&);
+    void append(const HitTestResult&, const HitTestRequest&);
 
-    // If m_rectBasedTestResult is 0 then set it to a new NodeSet. Return *m_rectBasedTestResult. Lazy allocation makes
+    // If m_listBasedTestResult is 0 then set it to a new NodeSet. Return *m_listBasedTestResult. Lazy allocation makes
     // sense because the NodeSet is seldom necessary, and it's somewhat expensive to allocate and initialize. This method does
-    // the same thing as mutableRectBasedTestResult(), but here the return value is const.
-    WEBCORE_EXPORT const NodeSet& rectBasedTestResult() const;
+    // the same thing as mutableListBasedTestResult(), but here the return value is const.
+    WEBCORE_EXPORT const NodeSet& listBasedTestResult() const;
 
     Vector<String> dictationAlternatives() const;
 
-    Node* targetNode() const;
+    Node* targetNode() const { return innerNode(); }
+    WEBCORE_EXPORT RefPtr<Node> protectedTargetNode() const;
+    WEBCORE_EXPORT Element* targetElement() const;
+    RefPtr<Element> protectedTargetElement() const;
 
 private:
-    NodeSet& mutableRectBasedTestResult(); // See above.
+    NodeSet& mutableListBasedTestResult(); // See above.
+
+    template<typename RectType> HitTestProgress addNodeToListBasedTestResultCommon(Node*, const HitTestRequest&, const HitTestLocation&, const RectType&);
+
+    RefPtr<Node> nodeForImageData() const;
+
+#if ENABLE(ACCESSIBILITY_ANIMATION_CONTROL)
+    void setAllowsAnimation(bool /* allowAnimation */) const;
+    HTMLImageElement* imageElement() const;
+#endif
 
 #if ENABLE(VIDEO)
     HTMLMediaElement* mediaElement() const;
@@ -162,18 +195,17 @@ private:
 
     RefPtr<Node> m_innerNode;
     RefPtr<Node> m_innerNonSharedNode;
-    LayoutPoint m_pointInInnerNodeFrame; // The hit-tested point in innerNode frame coordinates.
+    DoublePoint m_doublePointInInnerNodeFrame; // The hit-tested point in innerNode frame coordinates.
     LayoutPoint m_localPoint; // A point in the local coordinate space of m_innerNonSharedNode's renderer. Allows us to efficiently
                               // determine where inside the renderer we hit on subsequent operations.
     RefPtr<Element> m_innerURLElement;
     RefPtr<Scrollbar> m_scrollbar;
-    bool m_isOverWidget; // Returns true if we are over a widget (and not in the border/padding area of a RenderWidget for example).
+    bool m_isOverWidget { false }; // Returns true if we are over a widget (and not in the border/padding area of a RenderWidget for example).
+    std::optional<Style::PseudoElementIdentifier> m_pseudoElementIdentifier;
 
-    mutable std::unique_ptr<NodeSet> m_rectBasedTestResult;
+    mutable std::unique_ptr<NodeSet> m_listBasedTestResult;
 };
 
 WEBCORE_EXPORT String displayString(const String&, const Node*);
 
 } // namespace WebCore
-
-#endif // HitTestResult_h
