@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007, 2008, 2009, 2010 Apple Inc. All rights reserved.
+ * Copyright (C) 2007-2025 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -27,43 +27,40 @@
 
 #if ENABLE(VIDEO)
 
-#include "HTMLMediaElement.h"
-#include "RenderImage.h"
+#include <WebCore/RenderImage.h>
 
 namespace WebCore {
 
 class RenderMedia : public RenderImage {
+    WTF_MAKE_TZONE_OR_ISO_ALLOCATED(RenderMedia);
+    WTF_OVERRIDE_DELETE_FOR_CHECKED_PTR(RenderMedia);
 public:
-    RenderMedia(HTMLMediaElement&, RenderStyle&&);
-    RenderMedia(HTMLMediaElement&, RenderStyle&&, const IntSize& intrinsicSize);
+    RenderMedia(Type, HTMLMediaElement&, RenderStyle&&);
     virtual ~RenderMedia();
 
-    HTMLMediaElement& mediaElement() const { return downcast<HTMLMediaElement>(nodeForNonAnonymous()); }
+    inline HTMLMediaElement& mediaElement() const; // Defined in RenderMediaInlines.h
+    inline Ref<HTMLMediaElement> protectedMediaElement() const; // Defined in RenderMediaInlines.h
+
+    bool shouldDisplayBrokenImageIcon() const final { return false; }
 
 protected:
     void layout() override;
+    void styleDidChange(Style::Difference, const RenderStyle* oldStyle) override;
+
+    void visibleInViewportStateChanged() override { }
 
 private:
     void element() const = delete;
 
     bool canHaveChildren() const final { return true; }
 
-    const char* renderName() const override { return "RenderMedia"; }
-    bool isMedia() const final { return true; }
+    ASCIILiteral renderName() const override { return "RenderMedia"_s; }
     bool isImage() const final { return false; }
     void paintReplaced(PaintInfo&, const LayoutPoint&) override;
-
-    bool shadowControlsNeedCustomLayoutMetrics() const override { return true; }
-    void styleDidChange(StyleDifference, const RenderStyle* oldStyle) final;
 };
 
-inline RenderMedia* HTMLMediaElement::renderer() const
-{
-    return downcast<RenderMedia>(HTMLElement::renderer());
-}
-
 } // namespace WebCore
 
-SPECIALIZE_TYPE_TRAITS_RENDER_OBJECT(RenderMedia, isMedia())
+SPECIALIZE_TYPE_TRAITS_RENDER_OBJECT(RenderMedia, isRenderMedia())
 
 #endif // ENABLE(VIDEO)
