@@ -1,6 +1,7 @@
 /*
  * Copyright (C) 1999 Antti Koivisto (koivisto@kde.org)
  * Copyright (C) 2004-2017 Apple Inc. All rights reserved.
+ * Copyright (C) 2025 Samuel Weinig <sam@webkit.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -22,21 +23,25 @@
 #include "config.h"
 #include "StyleBackgroundData.h"
 
-#include "RenderStyle.h"
+#include "BorderData.h"
 #include "RenderStyleConstants.h"
+#include "RenderStyleDifference.h"
+#include "RenderStyleInlines.h"
 
 namespace WebCore {
 
+DEFINE_ALLOCATOR_WITH_HEAP_IDENTIFIER(StyleBackgroundData);
+
 StyleBackgroundData::StyleBackgroundData()
-    : background(FillLayer::create(FillLayerType::Background))
-    , color(RenderStyle::initialBackgroundColor())
+    : background(CSS::Keyword::None { })
+    , backgroundColor(RenderStyle::initialBackgroundColor())
 {
 }
 
 inline StyleBackgroundData::StyleBackgroundData(const StyleBackgroundData& other)
     : RefCounted<StyleBackgroundData>()
     , background(other.background)
-    , color(other.color)
+    , backgroundColor(other.backgroundColor)
     , outline(other.outline)
 {
 }
@@ -48,32 +53,36 @@ Ref<StyleBackgroundData> StyleBackgroundData::copy() const
 
 bool StyleBackgroundData::operator==(const StyleBackgroundData& other) const
 {
-    return background == other.background && color == other.color && outline == other.outline;
+    return background == other.background
+        && backgroundColor == other.backgroundColor
+        && outline == other.outline;
 }
 
-bool StyleBackgroundData::isEquivalentForPainting(const StyleBackgroundData& other, bool currentColorDiffers) const
+bool StyleBackgroundData::containsCurrentColor() const
 {
-    if (background != other.background || color != other.color)
-        return false;
-    if (currentColorDiffers && color == RenderStyle::currentColor())
-        return false;
-    if (!outline.isVisible() && !other.outline.isVisible())
-        return true;
-    if (currentColorDiffers && outline.color() == RenderStyle::currentColor())
-        return false;
-    return outline == other.outline;
+    return backgroundColor.containsCurrentColor()
+        || outline.outlineColor.containsCurrentColor();
 }
 
 void StyleBackgroundData::dump(TextStream& ts, DumpStyleValues behavior) const
 {
-    if (behavior == DumpStyleValues::All || *background != FillLayer::create(FillLayerType::Background).get())
-        ts.dumpProperty("background-image", background);
-    if (behavior == DumpStyleValues::All || color != RenderStyle::initialBackgroundColor())
-        ts.dumpProperty("background-color", color);
+    if (behavior == DumpStyleValues::All || background != Style::BackgroundLayers { CSS::Keyword::None { } })
+        ts.dumpProperty("background-image"_s, background);
+    if (behavior == DumpStyleValues::All || backgroundColor != RenderStyle::initialBackgroundColor())
+        ts.dumpProperty("background-color"_s, backgroundColor);
     if (behavior == DumpStyleValues::All || outline != OutlineValue())
-        ts.dumpProperty("outline", outline);
+        ts.dumpProperty("outline"_s, outline);
 }
 
+#if !LOG_DISABLED
+void StyleBackgroundData::dumpDifferences(TextStream& ts, const StyleBackgroundData& other) const
+{
+    LOG_IF_DIFFERENT(background);
+    LOG_IF_DIFFERENT(backgroundColor);
+    LOG_IF_DIFFERENT(outline);
+}
+#endif
+
 TextStream& operator<<(TextStream& ts, const StyleBackgroundData& backgroundData)
 {
     backgroundData.dump(ts);
