@@ -28,14 +28,13 @@ WI.loaded = function()
     // Register observers for events from the InspectorBackend.
     // The initialization order should match the same in Main.js.
     InspectorBackend.registerAnimationDispatcher(WI.AnimationObserver);
-    InspectorBackend.registerApplicationCacheDispatcher(WI.ApplicationCacheObserver);
+    InspectorBackend.registerBrowserDispatcher(WI.BrowserObserver);
     InspectorBackend.registerCPUProfilerDispatcher(WI.CPUProfilerObserver);
     InspectorBackend.registerCSSDispatcher(WI.CSSObserver);
     InspectorBackend.registerCanvasDispatcher(WI.CanvasObserver);
     InspectorBackend.registerConsoleDispatcher(WI.ConsoleObserver);
     InspectorBackend.registerDOMDispatcher(WI.DOMObserver);
     InspectorBackend.registerDOMStorageDispatcher(WI.DOMStorageObserver);
-    InspectorBackend.registerDatabaseDispatcher(WI.DatabaseObserver);
     InspectorBackend.registerDebuggerDispatcher(WI.DebuggerObserver);
     InspectorBackend.registerHeapDispatcher(WI.HeapObserver);
     InspectorBackend.registerInspectorDispatcher(WI.InspectorObserver);
@@ -51,10 +50,10 @@ WI.loaded = function()
 
     // Instantiate controllers used by tests.
     WI.managers = [
+        WI.browserManager = new WI.BrowserManager,
         WI.targetManager = new WI.TargetManager,
         WI.networkManager = new WI.NetworkManager,
         WI.domStorageManager = new WI.DOMStorageManager,
-        WI.databaseManager = new WI.DatabaseManager,
         WI.indexedDBManager = new WI.IndexedDBManager,
         WI.domManager = new WI.DOMManager,
         WI.cssManager = new WI.CSSManager,
@@ -62,7 +61,6 @@ WI.loaded = function()
         WI.runtimeManager = new WI.RuntimeManager,
         WI.heapManager = new WI.HeapManager,
         WI.memoryManager = new WI.MemoryManager,
-        WI.applicationCacheManager = new WI.ApplicationCacheManager,
         WI.timelineManager = new WI.TimelineManager,
         WI.auditManager = new WI.AuditManager,
         WI.debuggerManager = new WI.DebuggerManager,
@@ -75,13 +73,14 @@ WI.loaded = function()
 
     // Register for events.
     document.addEventListener("DOMContentLoaded", WI.contentLoaded);
+    WI.browserManager.enable();
 
     // Targets.
     WI.backendTarget = null;
-    WI._backendTargetAvailablePromise = new WI.WrappedPromise;
+    WI._backendTargetAvailablePromise = Promise.withResolvers();
 
     WI.pageTarget = null;
-    WI._pageTargetAvailablePromise = new WI.WrappedPromise;
+    WI._pageTargetAvailablePromise = Promise.withResolvers();
 
     if (InspectorBackend.hasDomain("Target"))
         WI.targetManager.createMultiplexingBackendTarget();
@@ -93,9 +92,7 @@ WI.contentLoaded = function()
 {
     // Things that would normally get called by the UI, that we still want to do in tests.
     WI.animationManager.enable();
-    WI.applicationCacheManager.enable();
     WI.canvasManager.enable();
-    WI.databaseManager.enable();
     WI.domStorageManager.enable();
     WI.heapManager.enable();
     WI.indexedDBManager.enable();
@@ -122,7 +119,7 @@ WI.performOneTimeFrontendInitializationsUsingTarget = function(target)
     // FIXME: This slows down test debug logging considerably.
     if (!WI.__didPerformCSSInitialization && target.hasDomain("CSS")) {
         WI.__didPerformCSSInitialization = true;
-        WI.CSSCompletions.initializeCSSCompletions(target);
+        WI.cssManager.initializeCSSPropertyNameCompletions(target);
     }
 };
 
@@ -155,7 +152,8 @@ WI.assumingMainTarget = () => WI.mainTarget;
 WI.isDebugUIEnabled = () => false;
 
 WI.isEngineeringBuild = false;
-WI.isExperimentalBuild = true;
+
+WI.engineeringSettingsAllowed = () => WI.isEngineeringBuild;
 
 WI.unlocalizedString = (string) => string;
 WI.UIString = (string, key, comment) => string;
@@ -174,6 +172,15 @@ WI.resolvedLayoutDirection = () => { return InspectorFrontendHost.userInterfaceL
 WI.updateDockedState = () => {};
 WI.updateDockingAvailability = () => {};
 WI.updateVisibilityState = () => {};
+WI.updateFindString = () => {};
+
+// Assists in the transition of moving agent implementations from page to frame.
+Object.defineProperty(WI, "mainFrameTarget",
+{
+    get() {
+        return WI.targets.find((x) => x.type === WI.TargetType.Frame && x.parentTarget === WI.backendTarget);
+    }
+});
 
 // FIXME: <https://webkit.org/b/201149> Web Inspector: replace all uses of `window.*Agent` with a target-specific call
 (function() {
@@ -185,15 +192,12 @@ WI.updateVisibilityState = () => {};
     }
     makeAgentGetter("Animation");
     makeAgentGetter("Audit");
-    makeAgentGetter("ApplicationCache");
     makeAgentGetter("CPUProfiler");
     makeAgentGetter("CSS");
     makeAgentGetter("Canvas");
-    makeAgentGetter("Console");
     makeAgentGetter("DOM");
     makeAgentGetter("DOMDebugger");
     makeAgentGetter("DOMStorage");
-    makeAgentGetter("Database");
     makeAgentGetter("Debugger");
     makeAgentGetter("Heap");
     makeAgentGetter("IndexedDB");
