@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2020 Apple Inc. All rights reserved.
+ * Copyright (C) 2020-2025 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -47,7 +47,7 @@ using namespace WebCore;
 WTF_MAKE_TZONE_ALLOCATED_IMPL(RemoteAudioSessionProxy);
 
 RemoteAudioSessionProxy::RemoteAudioSessionProxy(GPUConnectionToWebProcess& gpuConnection)
-: m_gpuConnection(gpuConnection)
+    : m_gpuConnection(gpuConnection)
 {
 }
 
@@ -78,6 +78,7 @@ RemoteAudioSessionConfiguration RemoteAudioSessionProxy::configuration()
         m_active,
         m_sceneIdentifier,
         m_soundStageSize,
+        session->categoryOverride(),
     };
 }
 
@@ -171,7 +172,7 @@ void RemoteAudioSessionProxy::setSoundStageSize(AudioSession::SoundStageSize siz
 
 RemoteAudioSessionProxyManager& RemoteAudioSessionProxy::audioSessionManager()
 {
-    return m_gpuConnection.get()->protectedGPUProcess()->audioSessionManager();
+    return m_gpuConnection.get()->gpuProcess().audioSessionManager();
 }
 
 Ref<RemoteAudioSessionProxyManager> RemoteAudioSessionProxy::protectedAudioSessionManager()
@@ -181,17 +182,17 @@ Ref<RemoteAudioSessionProxyManager> RemoteAudioSessionProxy::protectedAudioSessi
 
 Ref<IPC::Connection> RemoteAudioSessionProxy::protectedConnection() const
 {
-    return m_gpuConnection.get()->protectedConnection();
+    return m_gpuConnection.get()->connection();
 }
 
 void RemoteAudioSessionProxy::triggerBeginInterruptionForTesting()
 {
-    AudioSession::protectedSharedSession()->beginInterruptionForTesting();
+    AudioSession::singleton().beginInterruptionForTesting();
 }
 
 void RemoteAudioSessionProxy::triggerEndInterruptionForTesting()
 {
-    AudioSession::protectedSharedSession()->endInterruptionForTesting();
+    AudioSession::singleton().endInterruptionForTesting();
 }
 
 std::optional<SharedPreferencesForWebProcess> RemoteAudioSessionProxy::sharedPreferencesForWebProcess() const
