@@ -37,7 +37,7 @@
 #include <wtf/CompletionHandler.h>
 #include <wtf/FileSystem.h>
 #include <wtf/RefPtr.h>
-#include <wtf/glib/GRefPtr.h>
+#include <wtf/glib/GSpanExtras.h>
 #include <wtf/glib/GUniquePtr.h>
 #include <wtf/glib/WTFGType.h>
 
@@ -85,7 +85,7 @@ struct _WebKitUserContentFilterStorePrivate {
 #endif
 };
 
-WEBKIT_DEFINE_FINAL_TYPE_IN_2022_API(WebKitUserContentFilterStore, webkit_user_content_filter_store, G_TYPE_OBJECT)
+WEBKIT_DEFINE_FINAL_TYPE(WebKitUserContentFilterStore, webkit_user_content_filter_store, G_TYPE_OBJECT, GObject)
 
 static void webkitUserContentFilterStoreGetProperty(GObject* object, guint propID, GValue* value, GParamSpec* paramSpec)
 {
@@ -188,15 +188,14 @@ const char* webkit_user_content_filter_store_get_path(WebKitUserContentFilterSto
 #if ENABLE(CONTENT_EXTENSIONS)
 static void webkitUserContentFilterStoreSaveBytes(GRefPtr<GTask>&& task, String&& identifier, GRefPtr<GBytes>&& source)
 {
-    size_t sourceSize;
-    const char* sourceData = static_cast<const char*>(g_bytes_get_data(source.get(), &sourceSize));
-    if (!sourceSize) {
+    auto sourceData = span(source);
+    if (!sourceData.size()) {
         g_task_return_error(task.get(), g_error_new_literal(WEBKIT_USER_CONTENT_FILTER_ERROR, WEBKIT_USER_CONTENT_FILTER_ERROR_INVALID_SOURCE, "Source JSON rule set cannot be empty"));
         return;
     }
 
     auto* store = WEBKIT_USER_CONTENT_FILTER_STORE(g_task_get_source_object(task.get()));
-    store->priv->store->compileContentRuleList(WTFMove(identifier), String::fromUTF8(sourceData, sourceSize), [task = WTFMove(task)](RefPtr<API::ContentRuleList> contentRuleList, std::error_code error) {
+    store->priv->store->compileContentRuleList(WTFMove(identifier), String::fromUTF8(sourceData), [task = WTFMove(task)](RefPtr<API::ContentRuleList> contentRuleList, std::error_code error) {
         if (g_task_return_error_if_cancelled(task.get()))
             return;
 
@@ -216,7 +215,7 @@ static void webkitUserContentFilterStoreSaveBytes(GRefPtr<GTask>&& task, String&
  * @source: #GBytes containing the rule set in JSON format
  * @cancellable: (allow-none): a #GCancellable or %NULL to ignore
  * @callback: (scope async): a #GAsyncReadyCallback to call when saving is completed
- * @user_data: (closure): the data to pass to the callback function
+ * @user_data: the data to pass to the callback function
  *
  * Asynchronously save a content filter from a set source rule.
  *
@@ -282,7 +281,7 @@ WEBKIT_DEFINE_ASYNC_DATA_STRUCT(SaveTaskData)
  * @file: a #GFile containing the rule set in JSON format
  * @cancellable: (allow-none): a #GCancellable or %NULL to ignore
  * @callback: (scope async): a #GAsyncReadyCallback to call when saving is completed
- * @user_data: (closure): the data to pass to the callback function
+ * @user_data: the data to pass to the callback function
  *
  * Asynchronously save a content filter from the contents of a file.
  *
@@ -365,7 +364,7 @@ WebKitUserContentFilter* webkit_user_content_filter_store_save_from_file_finish(
  * @identifier: a filter identifier
  * @cancellable: (allow-none): a #GCancellable or %NULL to ignore
  * @callback: (scope async): a #GAsyncReadyCallback to call when the removal is completed
- * @user_data: (closure): the data to pass to the callback function
+ * @user_data: the data to pass to the callback function
  *
  * Asynchronously remove a content filter given its @identifier.
  *
@@ -424,7 +423,7 @@ gboolean webkit_user_content_filter_store_remove_finish(WebKitUserContentFilterS
  * @identifier: a filter identifier
  * @cancellable: (allow-none): a #GCancellable or %NULL to ignore
  * @callback: (scope async): a #GAsyncReadyCallback to call when the load is completed
- * @user_data: (closure): the data to pass to the callback function
+ * @user_data: the data to pass to the callback function
  *
  * Asynchronously load a content filter given its @identifier.
  *
@@ -485,7 +484,7 @@ WebKitUserContentFilter* webkit_user_content_filter_store_load_finish(WebKitUser
  * @store: a #WebKitUserContentFilterStore
  * @cancellable: (allow-none): a #GCancellable or %NULL to ignore
  * @callback: (scope async): a #GAsyncReadyCallback to call when the removal is completed
- * @user_data: (closure): the data to pass to the callback function
+ * @user_data: the data to pass to the callback function
  *
  * Asynchronously retrieve a list of the identifiers for all the stored filters.
  *
@@ -506,10 +505,21 @@ void webkit_user_content_filter_store_fetch_identifiers(WebKitUserContentFilterS
         if (g_task_return_error_if_cancelled(task.get()))
             return;
 
-        GStrv result = static_cast<GStrv>(g_new0(gchar*, identifiers.size() + 1));
-        for (size_t i = 0; i < identifiers.size(); ++i)
-            result[i] = g_strdup(identifiers[i].utf8().data());
-        g_task_return_pointer(task.get(), result, reinterpret_cast<GDestroyNotify>(g_strfreev));
+        auto bytesToAllocate = checkedSum<size_t>(identifiers.size(), 1);
+        bytesToAllocate *= sizeof(gchar*);
+        if (bytesToAllocate.hasOverflowed()) [[unlikely]] {
+            g_task_return_new_error(task.get(), G_IO_ERROR, g_io_error_from_errno(EOVERFLOW), "Too many items in result");
+            return;
+        }
+
+        auto result = GMallocSpan<gchar*>::malloc(bytesToAllocate);
+        for (size_t i = 0; i < identifiers.size(); ++i) {
+            const auto identifier = identifiers[i].utf8();
+            result[i] = g_strndup(identifier.data(), identifier.length());
+        }
+        result[identifiers.size()] = nullptr;
+
+        g_task_return_pointer(task.get(), result.leakSpan().data(), reinterpret_cast<GDestroyNotify>(g_strfreev));
     });
 #else
     g_task_return_new_error(task.get(), WEBKIT_USER_CONTENT_FILTER_ERROR, WEBKIT_USER_CONTENT_FILTER_ERROR_NOT_FOUND, "Content Extensions disabled");
