@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014 Apple Inc. All rights reserved.
+ * Copyright (C) 2014-2025 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -28,10 +28,11 @@
 
 #include "APIArray.h"
 #include "APIContentWorld.h"
-#include "APISerializedScriptValue.h"
+#include "APIJSBuffer.h"
 #include "APIUserScript.h"
 #include "APIUserStyleSheet.h"
 #include "InjectUserScriptImmediately.h"
+#include "JavaScriptEvaluationResult.h"
 #include "NetworkContentRuleListManagerMessages.h"
 #include "NetworkProcessProxy.h"
 #include "WebPageCreationParameters.h"
@@ -40,8 +41,8 @@
 #include "WebScriptMessageHandler.h"
 #include "WebUserContentControllerDataTypes.h"
 #include "WebUserContentControllerMessages.h"
-#include "WebUserContentControllerProxyMessages.h"
 #include <WebCore/SerializedScriptValue.h>
+#include <WebCore/SharedMemory.h>
 #include <wtf/CheckedPtr.h>
 
 #if ENABLE(CONTENT_EXTENSIONS)
@@ -77,33 +78,13 @@ WebUserContentControllerProxy::WebUserContentControllerProxy()
 
 WebUserContentControllerProxy::~WebUserContentControllerProxy()
 {
-    for (const auto& identifier : m_associatedContentWorlds) {
-        RefPtr world = API::ContentWorld::worldForIdentifier(identifier);
-        RELEASE_ASSERT(world);
-        world->userContentControllerProxyDestroyed(*this);
-    }
-    
     webUserContentControllerProxies().remove(identifier());
-    for (Ref process : m_processes) {
-        process->removeMessageReceiver(Messages::WebUserContentControllerProxy::messageReceiverName(), identifier());
-        process->didDestroyWebUserContentControllerProxy(*this);
-    }
 #if ENABLE(CONTENT_EXTENSIONS)
     for (Ref process : m_networkProcesses)
         process->didDestroyWebUserContentControllerProxy(*this);
 #endif
 }
 
-Ref<API::Array> WebUserContentControllerProxy::protectedUserScripts() const
-{
-    return m_userScripts;
-}
-
-Ref<API::Array> WebUserContentControllerProxy::protectedUserStyleSheets() const
-{
-    return m_userStyleSheets;
-}
-
 #if ENABLE(CONTENT_EXTENSIONS)
 void WebUserContentControllerProxy::addNetworkProcess(NetworkProcessProxy& proxy)
 {
@@ -116,40 +97,34 @@ void WebUserContentControllerProxy::removeNetworkProcess(NetworkProcessProxy& pr
 }
 #endif
 
-void WebUserContentControllerProxy::addProcess(WebProcessProxy& webProcessProxy)
+UserContentControllerParameters WebUserContentControllerProxy::parametersForProcess(WebProcessProxy& process) const
 {
-    ASSERT(!m_processes.hasNullReferences());
-
-    if (m_processes.add(webProcessProxy).isNewEntry)
-        webProcessProxy.addMessageReceiver(Messages::WebUserContentControllerProxy::messageReceiverName(), identifier(), *this);
-}
-
-UserContentControllerParameters WebUserContentControllerProxy::parameters() const
-{
-    auto userContentWorlds = WTF::map(m_associatedContentWorlds, [](auto& identifier) {
-        auto* world = API::ContentWorld::worldForIdentifier(identifier);
-        RELEASE_ASSERT(world);
-        return world->worldData();
-    });
+    m_processes.add(process);
 
     Vector<WebUserScriptData> userScripts;
-    for (auto userScript : protectedUserScripts()->elementsOfType<API::UserScript>())
-        userScripts.append({ userScript->identifier(), userScript->contentWorld().identifier(), userScript->userScript() });
+    for (RefPtr userScript : m_userScripts->elementsOfType<API::UserScript>())
+        userScripts.append({ userScript->identifier(), Ref { userScript->contentWorld() }->worldDataForProcess(process), userScript->userScript() });
 
     Vector<WebUserStyleSheetData> userStyleSheets;
-    for (auto userStyleSheet : protectedUserStyleSheets()->elementsOfType<API::UserStyleSheet>())
-        userStyleSheets.append({ userStyleSheet->identifier(), userStyleSheet->contentWorld().identifier(), userStyleSheet->userStyleSheet() });
+    for (RefPtr userStyleSheet : m_userStyleSheets->elementsOfType<API::UserStyleSheet>())
+        userStyleSheets.append({ userStyleSheet->identifier(), Ref { userStyleSheet->contentWorld() }->worldDataForProcess(process), userStyleSheet->userStyleSheet() });
 
-    auto messageHandlers = WTF::map(m_scriptMessageHandlers, [](auto entry) {
-        return WebScriptMessageHandlerData { entry.value->identifier(), entry.value->world().identifier(), entry.value->name() };
+    Vector<WebJSBufferData> buffers;
+    for (auto& [pair, buffer] : m_buffers) {
+        if (RefPtr world = API::ContentWorld::worldForIdentifier(pair.first))
+            buffers.append(WebJSBufferData { Ref { buffer }->sharedMemory(), world->worldDataForProcess(process), pair.second });
+    }
+
+    auto messageHandlers = WTF::map(m_scriptMessageHandlers, [&](auto entry) {
+        return WebScriptMessageHandlerData { entry.value->identifier(), entry.value->world().worldDataForProcess(process), entry.value->name() };
     });
 
     return {
         identifier()
-        , WTFMove(userContentWorlds)
         , WTFMove(userScripts)
         , WTFMove(userStyleSheets)
         , WTFMove(messageHandlers)
+        , WTFMove(buffers)
 #if ENABLE(CONTENT_EXTENSIONS)
         , contentRuleListData()
 #endif
@@ -165,53 +140,14 @@ Vector<std::pair<WebCompiledContentRuleListData, URL>> WebUserContentControllerP
 }
 #endif
 
-void WebUserContentControllerProxy::removeProcess(WebProcessProxy& webProcessProxy)
-{
-    ASSERT(m_processes.contains(webProcessProxy));
-    ASSERT(!m_processes.hasNullReferences());
-
-    m_processes.remove(webProcessProxy);
-    webProcessProxy.removeMessageReceiver(Messages::WebUserContentControllerProxy::messageReceiverName(), identifier());
-}
-
-void WebUserContentControllerProxy::addContentWorld(API::ContentWorld& world)
-{
-    if (world.identifier() == pageContentWorldIdentifier())
-        return;
-
-    auto addResult = m_associatedContentWorlds.add(world.identifier());
-    if (!addResult.isNewEntry)
-        return;
-
-    world.addAssociatedUserContentControllerProxy(*this);
-    for (Ref process : m_processes)
-        process->send(Messages::WebUserContentController::AddContentWorlds({ world.worldData() }), identifier());
-}
-
-void WebUserContentControllerProxy::contentWorldDestroyed(API::ContentWorld& world)
-{
-    bool result = m_associatedContentWorlds.remove(world.identifier());
-    ASSERT_UNUSED(result, result);
-
-    for (Ref process : m_processes)
-        process->send(Messages::WebUserContentController::RemoveContentWorlds({ world.identifier() }), identifier());
-}
-
-Ref<API::Array> WebUserContentControllerProxy::protectedUserScripts()
-{
-    return m_userScripts.get();
-}
-
 void WebUserContentControllerProxy::addUserScript(API::UserScript& userScript, InjectUserScriptImmediately immediately)
 {
     Ref<API::ContentWorld> world = userScript.contentWorld();
 
-    addContentWorld(world.get());
-
     m_userScripts->elements().append(&userScript);
 
     for (Ref process : m_processes)
-        process->send(Messages::WebUserContentController::AddUserScripts({ { userScript.identifier(), world->identifier(), userScript.userScript() } }, immediately), identifier());
+        process->send(Messages::WebUserContentController::AddUserScripts({ { userScript.identifier(), world->worldDataForProcess(process), userScript.userScript() } }, immediately), identifier());
 }
 
 void WebUserContentControllerProxy::removeUserScript(API::UserScript& userScript)
@@ -229,7 +165,7 @@ void WebUserContentControllerProxy::removeAllUserScripts(API::ContentWorld& worl
     for (Ref process : m_processes)
         process->send(Messages::WebUserContentController::RemoveAllUserScripts({ world.identifier() }), identifier());
 
-    protectedUserScripts()->removeAllOfTypeMatching<API::UserScript>([&](const auto& userScript) {
+    m_userScripts->removeAllOfTypeMatching<API::UserScript>([&](const auto& userScript) {
         return &userScript->contentWorld() == &world;
     });
 }
@@ -245,7 +181,7 @@ void WebUserContentControllerProxy::removeAllUserScripts()
 #endif
     {
         HashCountedSet<RefPtr<API::ContentWorld>> worlds;
-        for (auto userScript : protectedUserScripts()->elementsOfType<API::UserScript>())
+        for (RefPtr userScript : m_userScripts->elementsOfType<API::UserScript>())
             worlds.add(const_cast<API::ContentWorld*>(&userScript->contentWorld()));
 
         auto worldIdentifiers = WTF::map(worlds, [](auto& entry) {
@@ -264,7 +200,7 @@ void WebUserContentControllerProxy::removeAllUserScripts()
     ASSERT(removeWebExtensions == RemoveWebExtensions::No);
 
     Vector<Ref<API::UserScript>> scriptsToRemove;
-    for (auto userScript : protectedUserScripts()->elementsOfType<API::UserScript>()) {
+    for (RefPtr userScript : m_userScripts->elementsOfType<API::UserScript>()) {
         if (WebExtensionMatchPattern::isWebExtensionURL(userScript->userScript().url()))
             continue;
         scriptsToRemove.append(const_cast<API::UserScript&>(*userScript));
@@ -279,12 +215,10 @@ void WebUserContentControllerProxy::addUserStyleSheet(API::UserStyleSheet& userS
 {
     Ref<API::ContentWorld> world = userStyleSheet.contentWorld();
 
-    addContentWorld(world.get());
-
     m_userStyleSheets->elements().append(&userStyleSheet);
 
     for (Ref process : m_processes)
-        process->send(Messages::WebUserContentController::AddUserStyleSheets({ { userStyleSheet.identifier(), world->identifier(), userStyleSheet.userStyleSheet() } }), identifier());
+        process->send(Messages::WebUserContentController::AddUserStyleSheets({ { userStyleSheet.identifier(), world->worldDataForProcess(process), userStyleSheet.userStyleSheet() } }), identifier());
 }
 
 void WebUserContentControllerProxy::removeUserStyleSheet(API::UserStyleSheet& userStyleSheet)
@@ -302,7 +236,7 @@ void WebUserContentControllerProxy::removeAllUserStyleSheets(API::ContentWorld&
     for (Ref process : m_processes)
         process->send(Messages::WebUserContentController::RemoveAllUserStyleSheets({ world.identifier() }), identifier());
 
-    protectedUserStyleSheets()->removeAllOfTypeMatching<API::UserStyleSheet>([&](const auto& userStyleSheet) {
+    m_userStyleSheets->removeAllOfTypeMatching<API::UserStyleSheet>([&](const auto& userStyleSheet) {
         return &userStyleSheet->contentWorld() == &world;
     });
 }
@@ -318,7 +252,7 @@ void WebUserContentControllerProxy::removeAllUserStyleSheets()
 #endif
     {
         HashCountedSet<RefPtr<API::ContentWorld>> worlds;
-        for (auto userStyleSheet : protectedUserStyleSheets()->elementsOfType<API::UserStyleSheet>())
+        for (RefPtr userStyleSheet : m_userStyleSheets->elementsOfType<API::UserStyleSheet>())
             worlds.add(const_cast<API::ContentWorld*>(&userStyleSheet->contentWorld()));
 
         auto worldIdentifiers = WTF::map(worlds, [](auto& entry) {
@@ -337,7 +271,7 @@ void WebUserContentControllerProxy::removeAllUserStyleSheets()
     ASSERT(removeWebExtensions == RemoveWebExtensions::No);
 
     Vector<Ref<API::UserStyleSheet>> styleSheetsToRemove;
-    for (auto userStyleSheet : protectedUserStyleSheets()->elementsOfType<API::UserStyleSheet>()) {
+    for (RefPtr userStyleSheet : m_userStyleSheets->elementsOfType<API::UserStyleSheet>()) {
         if (WebExtensionMatchPattern::isWebExtensionURL(userStyleSheet->userStyleSheet().url()))
             continue;
         styleSheetsToRemove.append(const_cast<API::UserStyleSheet&>(*userStyleSheet));
@@ -357,12 +291,10 @@ bool WebUserContentControllerProxy::addUserScriptMessageHandler(WebScriptMessage
             return false;
     }
 
-    addContentWorld(world);
-
-    m_scriptMessageHandlers.add(handler.identifier(), &handler);
+    m_scriptMessageHandlers.add(handler.identifier(), handler);
 
     for (Ref process : m_processes)
-        process->send(Messages::WebUserContentController::AddUserScriptMessageHandlers({ { handler.identifier(), world->identifier(), handler.name() } }), identifier());
+        process->send(Messages::WebUserContentController::AddUserScriptMessageHandlers({ { handler.identifier(), world->worldDataForProcess(process), handler.name() } }), identifier());
     
     return true;
 }
@@ -399,40 +331,12 @@ void WebUserContentControllerProxy::removeAllUserMessageHandlers()
     m_scriptMessageHandlers.clear();
 }
 
-void WebUserContentControllerProxy::didPostMessage(WebPageProxyIdentifier pageProxyID, FrameInfoData&& frameInfoData, ScriptMessageHandlerIdentifier messageHandlerID, std::span<const uint8_t> dataReference, CompletionHandler<void(std::span<const uint8_t>, const String&)>&& reply)
+void WebUserContentControllerProxy::didPostMessage(WebPageProxy& page, FrameInfoData&& frameInfoData, ScriptMessageHandlerIdentifier messageHandlerID, JavaScriptEvaluationResult&& message, CompletionHandler<void(Expected<WebKit::JavaScriptEvaluationResult, String>&&)>&& reply) const
 {
-    auto page = WebProcessProxy::webPage(pageProxyID);
-    if (!page) {
-        reply({ }, { });
-        return;
-    }
-
-    if (!HashMap<ScriptMessageHandlerIdentifier, RefPtr<WebScriptMessageHandler>>::isValidKey(messageHandlerID)) {
-        reply({ }, { });
-        return;
-    }
-
-    RefPtr<WebScriptMessageHandler> handler = m_scriptMessageHandlers.get(messageHandlerID);
-    if (!handler) {
-        reply({ }, { });
-        return;
-    }
-
-    if (!handler->client().supportsAsyncReply()) {
-        handler->client().didPostMessage(*page, WTFMove(frameInfoData), handler->protectedWorld(),  WebCore::SerializedScriptValue::createFromWireBytes({ dataReference }));
-        reply({ }, { });
-        return;
-    }
-
-    handler->client().didPostMessageWithAsyncReply(*page, WTFMove(frameInfoData), handler->protectedWorld(),  WebCore::SerializedScriptValue::createFromWireBytes({ dataReference }), [reply = WTFMove(reply)](API::SerializedScriptValue* value, const String& errorMessage) mutable {
-        if (errorMessage.isNull()) {
-            ASSERT(value);
-            reply({ value->internalRepresentation().wireBytes() }, { });
-            return;
-        }
-
-        reply({ }, errorMessage);
-    });
+    RefPtr handler = m_scriptMessageHandlers.get(messageHandlerID);
+    if (!handler)
+        return reply(makeUnexpected(String()));
+    handler->client().didPostMessage(page, WTFMove(frameInfoData), handler->world(), WTFMove(message), WTFMove(reply));
 }
 
 #if ENABLE(CONTENT_EXTENSIONS)
@@ -499,4 +403,18 @@ void WebUserContentControllerProxy::removeAllContentRuleLists()
 }
 #endif
 
+void WebUserContentControllerProxy::addJSBuffer(API::JSBuffer& buffer, API::ContentWorld& world, const String& name)
+{
+    m_buffers.set({ world.identifier(), name }, buffer);
+    for (Ref process : m_processes)
+        process->send(Messages::WebUserContentController::AddJSBuffer(WebJSBufferData { buffer.sharedMemory(), world.worldDataForProcess(process), name }), identifier());
+}
+
+void WebUserContentControllerProxy::removeJSBuffer(API::ContentWorld& world, const String& name)
+{
+    m_buffers.remove({ world.identifier(), name });
+    for (Ref process : m_processes)
+        process->send(Messages::WebUserContentController::RemoveJSBuffer(world.identifier(), name), identifier());
+}
+
 } // namespace WebKit
