@@ -83,9 +83,6 @@ public:
     bool isDeclarativeWebPushEnabled() const { return m_isDeclarativeWebPushEnabled; }
     void setIsDeclarativeWebPushEnabled(bool enabled) { m_isDeclarativeWebPushEnabled = enabled; }
 #endif
-
-    const String& applicationCacheDirectory() const { return m_directories.applicationCacheDirectory; }
-    void setApplicationCacheDirectory(String&& directory) { m_directories.applicationCacheDirectory = WTFMove(directory); }
     
     const String& mediaCacheDirectory() const { return m_directories.mediaCacheDirectory; }
     void setMediaCacheDirectory(String&& directory) { m_directories.mediaCacheDirectory = WTFMove(directory); }
@@ -161,6 +158,11 @@ public:
     const String& deviceIdHashSaltsStorageDirectory() const { return m_directories.deviceIdHashSaltsStorageDirectory; }
     void setDeviceIdHashSaltsStorageDirectory(String&& directory) { m_directories.deviceIdHashSaltsStorageDirectory = WTFMove(directory); }
 
+#if ENABLE(ENCRYPTED_MEDIA)
+    const String& mediaKeysHashSaltsStorageDirectory() const { return m_directories.mediaKeysHashSaltsStorageDirectory; }
+    void setMediaKeysHashSaltsStorageDirectory(String&& directory) { m_directories.mediaKeysHashSaltsStorageDirectory = WTFMove(directory); }
+#endif
+
     const String& cookieStorageFile() const { return m_directories.cookieStorageFile; }
     void setCookieStorageFile(String&& directory) { m_directories.cookieStorageFile = WTFMove(directory); }
     
@@ -181,9 +183,6 @@ public:
 
     const String& webPushPartitionString() const { return m_webPushPartitionString; }
     void setWebPushPartitionString(String&& string) { m_webPushPartitionString = WTFMove(string); }
-
-    const String& applicationCacheFlatFileSubdirectoryName() const { return m_directories.applicationCacheFlatFileSubdirectoryName; }
-    void setApplicationCacheFlatFileSubdirectoryName(String&& directory) { m_directories.applicationCacheFlatFileSubdirectoryName = WTFMove(directory); }
     
     const String& serviceWorkerRegistrationDirectory() const { return m_directories.serviceWorkerRegistrationDirectory; }
     void setServiceWorkerRegistrationDirectory(String&& directory) { m_directories.serviceWorkerRegistrationDirectory = WTFMove(directory); }
@@ -197,6 +196,11 @@ public:
     const String& sourceApplicationSecondaryIdentifier() const { return m_sourceApplicationSecondaryIdentifier; }
     void setSourceApplicationSecondaryIdentifier(String&& identifier) { m_sourceApplicationSecondaryIdentifier = WTFMove(identifier); }
     
+#if ENABLE(CONTENT_EXTENSIONS)
+    const String& resourceMonitorThrottlerDirectory() const { return m_directories.resourceMonitorThrottlerDirectory; }
+    void setResourceMonitorThrottlerDirectory(String&& directory) { m_directories.resourceMonitorThrottlerDirectory = WTFMove(directory); }
+#endif
+
     const URL& httpProxy() const { return m_httpProxy; }
     void setHTTPProxy(URL&& proxy) { m_httpProxy = WTFMove(proxy); }
 
@@ -249,13 +253,25 @@ public:
     void setMemoryFootprintNotificationThresholds(Vector<size_t>&& thresholds) { m_memoryFootprintNotificationThresholds = WTFMove(thresholds); }
     const Vector<size_t>& memoryFootprintNotificationThresholds() { return m_memoryFootprintNotificationThresholds; }
 
+#if HAVE(WEBCONTENTRESTRICTIONS_PATH_SPI)
+    void setWebContentRestrictionsConfigurationFile(String&& file) { m_webContentRestrictionsConfigurationFile = WTFMove(file); }
+    const String& webContentRestrictionsConfigurationFile() const { return m_webContentRestrictionsConfigurationFile; }
+#endif
+
+    void setAdditionalDomainsWithUserInteractionForTesting(String&& domains) { m_additionalDomainsWithUserInteractionForTesting = WTFMove(domains); }
+    const String& additionalDomainsWithUserInteractionForTesting() const { return m_additionalDomainsWithUserInteractionForTesting; }
+
+    const String& enhancedSecurityDirectory() const { return m_directories.enhancedSecurityDirectory; }
+    void setEnhancedSecurityDirectory(String&& directory) { m_directories.enhancedSecurityDirectory = WTFMove(directory); }
+
     struct Directories {
-        String applicationCacheFlatFileSubdirectoryName { "Files"_s };
-        String applicationCacheDirectory;
         String alternativeServicesDirectory;
         String cacheStorageDirectory;
         String cookieStorageFile;
         String deviceIdHashSaltsStorageDirectory;
+#if ENABLE(ENCRYPTED_MEDIA)
+        String mediaKeysHashSaltsStorageDirectory;
+#endif
         String generalStorageDirectory;
         String hstsStorageDirectory;
         String indexedDBDatabaseDirectory;
@@ -271,6 +287,10 @@ public:
 #if ENABLE(ARKIT_INLINE_PREVIEW)
         String modelElementCacheDirectory;
 #endif
+#if ENABLE(CONTENT_EXTENSIONS)
+        String resourceMonitorThrottlerDirectory;
+#endif
+        String enhancedSecurityDirectory;
         Directories isolatedCopy() const&;
         Directories isolatedCopy() &&;
     };
@@ -308,7 +328,11 @@ private:
     bool m_deviceManagementRestrictionsEnabled { false };
     bool m_allLoadsBlockedByDeviceManagementRestrictionsForTesting { false };
     bool m_allowsCellularAccess { true };
+#if ENABLE(TLS_1_2_DEFAULT_MINIMUM)
+    bool m_legacyTLSEnabled { false };
+#else
     bool m_legacyTLSEnabled { true };
+#endif
     bool m_fastServerTrustEvaluationEnabled { false };
     bool m_serviceWorkerProcessTerminationDelayEnabled { true };
     bool m_testingSessionEnabled { false };
@@ -334,6 +358,14 @@ private:
 #endif
     Vector<size_t> m_memoryFootprintNotificationThresholds;
     std::optional<bool> m_defaultTrackingPreventionEnabledOverride;
+#if HAVE(WEBCONTENTRESTRICTIONS_PATH_SPI)
+    String m_webContentRestrictionsConfigurationFile;
+#endif
+    String m_additionalDomainsWithUserInteractionForTesting;
 };
 
 }
+
+SPECIALIZE_TYPE_TRAITS_BEGIN(WebKit::WebsiteDataStoreConfiguration)
+static bool isType(const API::Object& object) { return object.type() == API::Object::Type::WebsiteDataStoreConfiguration; }
+SPECIALIZE_TYPE_TRAITS_END()
