@@ -26,24 +26,32 @@
 #include "config.h"
 #include "WebPage.h"
 
+#include "DrawingAreaCoordinatedGraphics.h"
 #include "EditorState.h"
 #include "InputMethodState.h"
 #include "MessageSenderInlines.h"
+#include "RenderProcessInfo.h"
 #include "UserMessage.h"
 #include "WebKitUserMessage.h"
 #include "WebKitWebPagePrivate.h"
 #include "WebPageProxyMessages.h"
+#include "WebProcess.h"
 #include "WebProcessExtensionManager.h"
+#include <WebCore/DocumentView.h>
 #include <WebCore/Editor.h>
 #include <WebCore/HTMLInputElement.h>
 #include <WebCore/HTMLTextAreaElement.h>
-#include <WebCore/LocalFrame.h>
+#include <WebCore/LocalFrameInlines.h>
 #include <WebCore/LocalFrameView.h>
+#include <WebCore/PlatformDisplay.h>
+#include <WebCore/PointerCharacteristics.h>
 #include <WebCore/Range.h>
+#include <WebCore/Settings.h>
 #include <WebCore/TextIterator.h>
 #include <WebCore/UserAgent.h>
 #include <WebCore/VisiblePosition.h>
 #include <WebCore/VisibleUnits.h>
+#include <wtf/text/MakeString.h>
 
 #if ENABLE(2022_GLIB_API)
 #include "WebKitWebProcessExtension.h"
@@ -51,6 +59,13 @@
 #include "WebKitWebExtension.h"
 #endif
 
+#if USE(GBM)
+#include <WebCore/DRMDeviceManager.h>
+#include <WebCore/GBMDevice.h>
+#include <gbm.h>
+#include <xf86drm.h>
+#endif
+
 namespace WebKit {
 using namespace WebCore;
 
@@ -148,7 +163,7 @@ void WebPage::getPlatformEditorState(LocalFrame& frame, EditorState& result) con
         auto compositionRange = frame.editor().compositionRange();
         if (surroundingRange && compositionRange && contains<ComposedTree>(*surroundingRange, *compositionRange)) {
             auto beforeText = plainText({ surroundingRange->start, compositionRange->start });
-            postLayoutData.surroundingContext = beforeText + plainText({ compositionRange->end, surroundingRange->end });
+            postLayoutData.surroundingContext = makeString(beforeText, plainText({ compositionRange->end, surroundingRange->end }));
             postLayoutData.surroundingContextCursorPosition = beforeText.length();
             postLayoutData.surroundingContextSelectionPosition = postLayoutData.surroundingContextCursorPosition;
         } else {
@@ -161,7 +176,7 @@ void WebPage::getPlatformEditorState(LocalFrame& frame, EditorState& result) con
     }
 }
 
-static std::optional<InputMethodState> inputMethodSateForElement(Element* element)
+static std::optional<InputMethodState> inputMethodStateForElement(Element* element)
 {
     if (!element || !element->shouldUseInputMethod())
         return std::nullopt;
@@ -189,7 +204,7 @@ static std::optional<InputMethodState> inputMethodSateForElement(Element* elemen
 
 void WebPage::setInputMethodState(Element* element)
 {
-    auto state = inputMethodSateForElement(element);
+    auto state = inputMethodStateForElement(element);
     if (m_inputMethodState == state)
         return;
 
@@ -205,4 +220,94 @@ String WebPage::platformUserAgent(const URL& url) const
     return WebCore::standardUserAgentForURL(url);
 }
 
+bool WebPage::hoverSupportedByPrimaryPointingDevice() const
+{
+    return WebProcess::singleton().primaryPointingDevice() == AvailableInputDevices::Mouse;
+}
+
+bool WebPage::hoverSupportedByAnyAvailablePointingDevice() const
+{
+    return WebProcess::singleton().availableInputDevices().contains(AvailableInputDevices::Mouse);
+}
+
+std::optional<PointerCharacteristics> WebPage::pointerCharacteristicsOfPrimaryPointingDevice() const
+{
+    const auto& primaryPointingDevice = WebProcess::singleton().primaryPointingDevice();
+    if (primaryPointingDevice == AvailableInputDevices::Mouse)
+        return PointerCharacteristics::Fine;
+    if (primaryPointingDevice == AvailableInputDevices::Touchscreen)
+        return PointerCharacteristics::Coarse;
+    return std::nullopt;
+}
+
+OptionSet<PointerCharacteristics> WebPage::pointerCharacteristicsOfAllAvailablePointingDevices() const
+{
+    OptionSet<PointerCharacteristics> pointerCharacteristics;
+    const auto& availableInputs = WebProcess::singleton().availableInputDevices();
+    if (availableInputs.contains(AvailableInputDevices::Mouse))
+        pointerCharacteristics.add(PointerCharacteristics::Fine);
+    if (availableInputs.contains(AvailableInputDevices::Touchscreen))
+        pointerCharacteristics.add(PointerCharacteristics::Coarse);
+    return pointerCharacteristics;
+}
+
+void WebPage::getRenderProcessInfo(CompletionHandler<void(RenderProcessInfo&&)>&& completionHandler)
+{
+    RenderProcessInfo info;
+
+    auto* display = PlatformDisplay::sharedDisplayIfExists();
+    if (!display) {
+        completionHandler(WTFMove(info));
+        return;
+    }
+
+    switch (display->type()) {
+#if PLATFORM(GTK) || ENABLE(WPE_PLATFORM)
+    case PlatformDisplay::Type::Surfaceless:
+        info.platform = "Surfaceless"_s;
+        break;
+#if USE(GBM)
+    case PlatformDisplay::Type::GBM:
+        info.platform = "GBM"_s;
+        if (auto device = DRMDeviceManager::singleton().mainGBMDevice(DRMDeviceManager::NodeType::Render)) {
+            if (drmVersion* version = drmGetVersion(gbm_device_get_fd(device->device()))) {
+                info.drmVersion = makeString(unsafeSpan(version->name), " ("_s, unsafeSpan(version->desc), ") "_s, version->version_major, '.', version->version_minor, '.', version->version_patchlevel, ". "_s, unsafeSpan(version->date));
+                drmFreeVersion(version);
+            }
+        }
+        break;
+#endif
+#endif
+#if OS(ANDROID)
+    case PlatformDisplay::Type::Android:
+        info.platform = "Android"_s;
+        break;
+#endif
+#if USE(WPE_RENDERER)
+    case PlatformDisplay::Type::WPE:
+        info.platform = "WPE"_s;
+        break;
+#endif
+    default:
+        break;
+    }
+
+#if USE(SKIA)
+    info.msaaSampleCount = display->msaaSampleCount();
+#endif
+
+#if USE(GBM)
+    if (info.platform != "WPE"_s) {
+        info.supportedBufferFormats = display->bufferFormats().map([](const auto& format) -> RendererBufferFormat::Format {
+            return {
+                format.fourcc.value,
+                format.modifiers,
+            };
+        });
+    }
+#endif // USE(GBM)
+
+    static_cast<DrawingAreaCoordinatedGraphics*>(m_drawingArea.get())->fillGLInformation(WTFMove(info), WTFMove(completionHandler));
+}
+
 } // namespace WebKit
